#ifndef ASTARSEARCHSTRATEGY_H
#define ASTARSEARCHSTRATEGY_H

#include "AgendaSearchStrategy.hpp"
#include "OutsideCostEstimator.hpp"

//! Agenda based parsing using an outside cost 
class AStarSearchStrategy : public AgendaSearchStrategy {
	
public:
  //! Constructor
  AStarSearchStrategy(const string &configFile);

  //! Destructor
  virtual ~AStarSearchStrategy();

  //! Alerts the search strategy to changes in the grammar
  //! This function will have the OutsideCostEstimator precompute
  //! information on the grammar.
  void setGrammar(Grammar *g);

  //! Initializes the Search Strategy
  //! This will also initialize the outside cost estimator
  void initialize(SentenceTuple &sent);

  //! Adds an item to the search strategy
  //! This will compute the outside cost on 
  //! an item and incorporate that with the cost
  void addItem(Item *item);

  void clear();
	
protected:
  OutsideCostEstimator *m_pOutsideCost;
  SentenceTuple m_sent;
};

#endif

