#include "AbsoluteCostPruningStrategy.hpp"
#include <fstream>

#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"

// constructs a absolute cost pruning strategy from the config file.
AbsoluteCostPruningStrategy::
AbsoluteCostPruningStrategy(const string& configFile) : m_curT(0), m_origT(0), m_decrT(0) {
  // compile the config file.
  BPO::options_description config("Configuration");
  config.add_options()
    ("AbsoluteThreshold", BPO::value<SCORE>(&m_origT), "The threshold for Absolute pruning.")
    ("AbsoluteDecrement", BPO::value<SCORE>(&m_decrT)->default_value(0.0), "The value used to lower the absolute threshold in multi-pass parsing.");
  BPO::variables_map vm;
  cerr << "AbsoluteCostPruningStrategy: Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
    cerr << "Option values of \"AbsoluteCostPruningStrategy\"" << endl << endl;
    cerr << OptionsPrinter(config, vm) << endl;
  }

  if (m_origT <= 0.0)
    cerr << "Warning: AbsoluteCostPruningStrategy ineffective with threshold of " << m_origT << endl;

  // initialize the thresholds.
  m_curT = m_origT;

}

// Returns true if the item is prunable.
bool AbsoluteCostPruningStrategy::prunable(const Item* item) const {

  // if the original absolute threshold is less than or equal to 0,
  // then there will be no pruning at all.
  if(m_origT <= 0){
    return false;
  }

  // getScore returns the inside+outside cost
  if(item->getScore() > m_curT) { return true; }
  else { return false; }
}
