#ifndef _AbsoluteCostPruningStrategy_H_
#define _AbsoluteCostPruningStrategy_H_

#include "PruningStrategy.hpp"

//! Use an absolute threshold to prune all items, i.e. all items
// with a cost of more than that threshold will be removed
class AbsoluteCostPruningStrategy : public PruningStrategy {
   typedef PruningStrategy _super;
public:

  /** Constructor. */
  AbsoluteCostPruningStrategy(const string& configFile);

  virtual ~AbsoluteCostPruningStrategy() {}

  /**
   * Returns true if the item is to be pruned. Note that if
   * the original absolute threshold is set to be greater than 1.0,
   * there will be no pruning at all.
   */
  bool prunable(const Item* item) const;

  //! Lower the absolute threshold. If the current threshold
  //! is larger than 1.0, this method does nothing.
  void lowerTh() { if(m_curT > 1.0) return; m_curT += m_decrT; }

  //! Set a new absolute threshold. Notice that the original value
  //! keeps unchanged.
  void setTh(SCORE th) { m_curT = th; }

  /**
   * Gets the original threshold.
   */
  SCORE getOrigTh() const { return m_origT; }

  /**
   * Gets the current threshold (possibly raised from the original).
   */
  SCORE getCurTh() const { return m_curT; }

  //! Sets the current threshold back to the original threshold.
  void reset() { m_curT = m_origT; }

private:

  /**
   * The current threshold.
   */
  SCORE m_curT;

  /**
   * The original threshold.
   */
   SCORE m_origT;

  /** The amount of score to be lowered (decremented)in multi-pass parsing.*/
   SCORE m_decrT;
};

#endif
