#ifndef _AbsoluteCountPruningStrategy_H_
#define _AbsoluteCountPruningStrategy_H_

#include "PruningStrategy.hpp"
#include "ItemIndex.hpp"

class Item;

//! Strategy of item pruning. This strategy applies to
//! both FactoredItemIndex and NaiveItemIndex.
//! Items are pruned if they appear more than 
//! m_itemCount times.
class AbsoluteCountPruningStrategy : public PruningStrategy {

  typedef PruningStrategy _super;

public:

  AbsoluteCountPruningStrategy(const string& configFile);

  ~AbsoluteCountPruningStrategy() {
    delete m_pItemIndex;
  }

  /**
   * Returns true if an item is to be pruned. Note that
   * if the relative pruning threshold is larger than 1, then
   * there will be no pruning at all.
   */
  bool prunable(const Item* inf) const;

  /** Inserts a chart index into the item index table.*/
  void memoize(const Item* item);

  /** Gets the original threshold. */
  SCORE getOrigTh() const { return m_itemCount; }

  /** Gets the current threshold (possibly raised from the original). */
  SCORE getCurTh() const { return m_itemCount; }

  void reset();

private:
  bool prunable(SCORE score, vector<Item*>& edges) const;

  /**
   * The count over a span.
   */
  unsigned int m_itemCount;

  /**
   * The item index.
   */
  ItemIndex* m_pItemIndex;
};

#endif
