//! \brief The interface of all generalized parsers.
/** 
* This is a simple parser interface. It only provides methods for
* initialization and parsing.
*/

#ifndef _AbstractParser_H_
#define _AbstractParser_H_

#include "HeadMTree.hpp"
#include "SentenceTuple.hpp"
#include "Dbug.hpp"
#include "GPDefs.hpp"
#include "ParseStats.hpp"
#include "Timer.hpp"
#include "Exceptions.hpp"
#include <boost/serialization/is_abstract.hpp>

// using namespace std;

/// The generalized parser.
class AbstractParser : public Dbug {

public:

  AbstractParser();

  ~AbstractParser();
  
  //! compute the parse and return the resulting tree
  virtual WTreeVec getParse(SentenceTuple&)=0;

  //! Clears the parser by clearing the contents of its components.
  //! The components are still there, not being deleted.
  //! The grammar is not cleared.
  virtual void clear()=0;

};

BOOST_IS_ABSTRACT(AbstractParser);
#endif
