#ifdef USE_SERIALIZATION
#include "ArchiveMaker.hpp"
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#endif

#include "DebugMacros.hpp"
#include "OptionsPrinter.hpp"
#include <boost/program_options.hpp>
#include "ActiveFilterBuilder.hpp"
#include "ActiveLearningFilter.hpp"
#include "NullActiveFilter.hpp"
#include "UnseenActiveFilter.hpp"
#include "BigramActiveFilter.hpp"

#ifdef USE_SERIALIZATION
#include <boost/serialization/serialization.hpp>
#endif

ActiveFilterBuilder* ActiveFilterBuilder::_instance = 0;

ActiveFilterBuilder* ActiveFilterBuilder::Instance(){
  if(_instance == 0){
    _instance = new ActiveFilterBuilder;
  }
  return _instance;
}

//! build grammar
ActiveLearningFilter* ActiveFilterBuilder::build(const string& configFile){

  string optFilterType;
  string optCountExamples;

  BPO::options_description config ("Active learning filter config options");
  config.add_options()
    ("FilterType", BPO::value<string>(&optFilterType), "")
    ("CountFilteredExamples", BPO::value<string>(&optCountExamples), "")
    ;
  BPO::variables_map vm;

  if (configFile != ""){
    cerr << "ActiveFilterBuilder: Compiling config file " << configFile << endl;
    ifstream ifs(configFile.c_str());
    store(parse_config_file(ifs, config), vm);
  } else {
    optFilterType = "NULL";
  }

  notify(vm);

  if (optFilterType == ""){
    THROW("filter type not specified" << endl);
  }

  ActiveLearningFilter* filter;
  // construct the parser itself
  if (optFilterType == "NULL"){
    filter = new NullActiveFilter();
  } else if (optFilterType == "UNSEEN_WORD"){
    filter = new UnseenActiveFilter();
  } else if (optFilterType == "BIGRAM"){
    filter = new BigramActiveFilter();
  } else {
    THROW (cerr << optFilterType << " is not a recognized parser. " << endl);
  }

  if (optCountExamples == "TRUE"){
    filter->setCountExamples(true);
  }

  return filter;

}

