#ifndef _ActiveLearningFilter_H_
#define _ActiveLearningFilter_H_

#include "SentenceTuple.hpp"

//! Defines an abstract kernel for comparing two parses.
class ActiveLearningFilter : public Dbug {
public:

  ActiveLearningFilter() { };

  virtual ~ActiveLearningFilter() { };

  /**
   *  The filter method will first check if an
   *  example should be used for training.  If
   *  true, it will add the example to the history
   *  of all examples used.
   */
  virtual bool filter(SentenceTuple example) = 0;

  /**
   *  The clear method will empty any history,
   *  causing the filter to act as if no examples
   *  had been seen before.
   */
  virtual void clear() {};

  virtual void setCountExamples(bool count){
    m_countExamples = count;
  };

protected:

  void filterExample(SentenceTuple example){
    m_filteredCount++;
    if (m_countExamples){
      cerr << "Examples filtered: " << m_filteredCount << endl;
    }  
  }


private:

  int m_filteredCount;
  bool m_countExamples;
};

#endif
