#include "Exceptions.hpp"
#include <fstream>
#include "LazyAgenda.hpp"
#include "CompactAgenda.hpp"
#include "SmallestSizeFirstComparator.hpp"
#include "BestFirstComparator.hpp"
#include "LongestSpanComparator.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"
#include "AgendaBuilder.hpp"

AgendaBuilder* AgendaBuilder::_instance = 0;

AgendaBuilder* AgendaBuilder::Instance(){
  if(_instance == 0){
    _instance = new AgendaBuilder;
  }
  return _instance;
}

//! build agenda
Agenda* AgendaBuilder::build(const string& configFile) {
  // the variables defined in the config file.
  string agendaType;
  string searchStrategy;

  /*
   * Compile the config file.
   */
  BPO::options_description config("Configuration");
  config.add_options()
      ("AgendaType", BPO::value<string>(&agendaType), "The Agenda type.")
      ("SearchStrategy", BPO::value<string>(&searchStrategy), "The search strategy of the parser.")
      ;
  BPO::variables_map vm;
  cerr << "AgendaBuilder: Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
      cerr << "Option values of \"Agenda\"" << endl << endl;
      cerr << OptionsPrinter(config, vm) << endl;
  }

  if(agendaType == "") {
    THROW( "Error: the type of Agenda is not specified!" << endl);
  }

  agendaType = tolower(agendaType);
  searchStrategy = tolower(searchStrategy);

  if(agendaType == "lazy"){
    if(searchStrategy == "bottomup"){
      return new LazyAgenda<SmallestSizeFirstComparator>();
    } else if(searchStrategy == "bestfirst"){
      return new LazyAgenda<BestFirstComparator>();
    } else if(searchStrategy == "longestspan"){
      return new LazyAgenda<LongestSpanComparator>();
    } else {
      THROW( "Error: unknown search strategy specified!"<< endl);
    }
  }
  else if(agendaType == "compact"){
    if(searchStrategy == "bottomup"){
      return new CompactAgenda<SmallestSizeFirstComparator>();
    } else if(searchStrategy == "bestfirst"){
      return new CompactAgenda<BestFirstComparator>();
    } else if(searchStrategy == "longestspan"){
      return new CompactAgenda<LongestSpanComparator>();
    } else {
      THROW( "Error: unknown search strategy specified!"<< endl);
    }
  } else {
      THROW("Error: unknown agenda type."<< endl);
  }

}
