#include "AgendaSearchStrategy.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"
#include "AgendaBuilder.hpp"
#include <fstream>
#include "Exceptions.hpp"
#include "GPDefs.hpp"
#include "Path.hpp"

using namespace std;

AgendaSearchStrategy::AgendaSearchStrategy(const string &configFile) 
    : m_pAgenda(NULL) {

  // Things to look for in config
  string agendaConfigFile;

  // Compile the config file
  BPO::options_description config("Configuration");
  config.add_options()
    ("AgendaConfigFile",
     BPO::value<string>(&agendaConfigFile),
     "The agenda config file")
    ;
  BPO::variables_map vm;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(agendaConfigFile == "") {
    THROW("Error: agenda config file not specified");
  }

  agendaConfigFile = Path::maybe_compose(configFile, agendaConfigFile);

  // Build the Agenda
  m_pAgenda = AgendaBuilder::Instance()->build(agendaConfigFile);
  if(!m_pAgenda) {
    THROW("No agenda is built.");
  }
}

AgendaSearchStrategy::~AgendaSearchStrategy() {
  if(m_pAgenda) {
    delete m_pAgenda;
    m_pAgenda = NULL;
  }
}

void AgendaSearchStrategy::addItem(Item* item) {
  if(Dbug::isDebugAll(3)) {
    cerr << "Agenda " << m_uniqueID << " pushing: ";
    item->dump(cerr);
    cerr<<endl;
  }

  m_pAgenda->push(item);
}

Item* AgendaSearchStrategy::nextItem() {
  Item * pItem = m_pAgenda->top();
  m_pAgenda->pop();

  if(Dbug::isDebugAll(3)) {
    cerr <<  "Agenda " << m_uniqueID << " popped: ";
    pItem->dump(cerr);
    cerr<<endl;
  }

  return pItem;
}

void AgendaSearchStrategy::clear() {
  m_pAgenda->clear();
}

size_t AgendaSearchStrategy::size() const {
  return m_pAgenda->size();
}

bool AgendaSearchStrategy::empty() const {
  return m_pAgenda->empty();
}

void AgendaSearchStrategy::dump(ostream& out) const {
  m_pAgenda->dump(out);
}

bool AgendaSearchStrategy::exists(const Item &item) const {
  return m_pAgenda->exists(item);
}

