#ifndef ARCHIVE_H
#define ARCHIVE_H

#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include "HeadPMTGA.hpp"
#include "LexNT.hpp"
#include "LazyCD.hpp"
#include "ValencyEvent.hpp"
#include <iostream>
#include <boost/serialization/serialization.hpp>

class ArchiveMaker {
public:

  ArchiveMaker() {};

  void writeTextArchive(boost::archive::text_oarchive & oa, const LearnableGrammar* grammar) {
    oa <<(const vector<StringVocabulary*>)GlobalVocab::s_synCatSet;
    oa << (const vector<StringVocabulary*>)GlobalVocab::s_terminalSet;
    oa << GlobalVocab::s_noVocabsIn;
    oa << GlobalVocab::s_ioDIM;
    oa.register_type<HeadPMTGA>();
    oa.register_type<LexNT>();
    oa.register_type<LazyCD<ValencyEvent,
      ValencyEvent::FirstType,
      ValencyEvent::SecondType,
      boost::_mfi::cmf0<ValencyEvent::FirstType, ValencyEvent> > >();
    oa.register_type<LazyCD<HeirRoleVectorEvent,
      HeirRoleVectorEvent::FirstType,
      HeirRoleVectorEvent::SecondType,
      boost::_mfi::cmf0<HeirRoleVectorEvent::FirstType, HeirRoleVectorEvent> > >();
    oa.register_type<LazyCD<HeirEvent,
      HeirEvent::FirstType,
      HeirEvent::SecondType,
      boost::_mfi::cmf0<NTLink, HeirEvent> > >();
    oa.register_type<LazyCD<Dependency,
      Dependency::FirstType,
      Dependency::SecondType,
      boost::_mfi::cmf0<NTLink, Dependency> > >();
    oa.register_type<LazyCD<Dependency,
      Vector<unsigned short>,
      Vector<unsigned short>,
      boost::_bi::bind_t<Vector<unsigned short>,
        boost::_mfi::cmf1<Vector<unsigned short>, Dependency, unsigned short>,
        boost::_bi::list2<boost::arg<1>, boost::_bi::value<unsigned short> > >
      > >();
    oa.register_type<LazyCD<PAOrderEvent,
      PAOrderEvent::FirstType,
      PAOrderEvent::SecondType,
      boost::_mfi::cmf0<const NTPair&, ImDom<NTPair, PAV> > > >();
    oa << *grammar;
  }

  LearnableGrammar* readTextArchive(boost::archive::text_iarchive & ia, LearnableGrammar* grammar) {
    ia >> GlobalVocab::s_synCatSet;
    ia >> GlobalVocab::s_terminalSet;
    ia >> GlobalVocab::s_noVocabsIn;
    ia >> GlobalVocab::s_ioDIM;
    ia.register_type<HeadPMTGA>();
    ia.register_type<LexNT>();
    ia.register_type<LazyCD<ValencyEvent,
     ValencyEvent::FirstType,
     ValencyEvent::SecondType,
     boost::_mfi::cmf0<ValencyEvent::FirstType, ValencyEvent> > >();
   ia.register_type<LazyCD<HeirRoleVectorEvent,
     HeirRoleVectorEvent::FirstType,
     HeirRoleVectorEvent::SecondType,
     boost::_mfi::cmf0<HeirRoleVectorEvent::FirstType, HeirRoleVectorEvent> > >();
   ia.register_type<LazyCD<HeirEvent,
     HeirEvent::FirstType,
     HeirEvent::SecondType,
     boost::_mfi::cmf0<NTLink, HeirEvent> > >();
   ia.register_type<LazyCD<Dependency,
     Dependency::FirstType,
     Dependency::SecondType,
     boost::_mfi::cmf0<NTLink, Dependency> > >();
   ia.register_type<LazyCD<Dependency,
     Vector<unsigned short>,
     Vector<unsigned short>,
     boost::_bi::bind_t<Vector<unsigned short>,
       boost::_mfi::cmf1<Vector<unsigned short>, Dependency, unsigned short>,
       boost::_bi::list2<boost::arg<1>, boost::_bi::value<unsigned short> > >
     > >();
  ia.register_type<LazyCD<PAOrderEvent,
      PAOrderEvent::FirstType,
      PAOrderEvent::SecondType,
      boost::_mfi::cmf0<const NTPair&, ImDom<NTPair, PAV> > > >();
  ia >> grammar;
  return grammar;
  }
  void writeBinaryArchive(boost::archive::binary_oarchive & oa, const LearnableGrammar* grammar) {
    oa << (const vector<StringVocabulary*>)GlobalVocab::s_synCatSet;
    oa << (const vector<StringVocabulary*>)GlobalVocab::s_terminalSet;
    oa << GlobalVocab::s_noVocabsIn;
    oa << GlobalVocab::s_ioDIM;
    oa.register_type<HeadPMTGA>();
    oa.register_type<LexNT>();
    oa.register_type<LazyCD<ValencyEvent,
      ValencyEvent::FirstType,
      ValencyEvent::SecondType,
      boost::_mfi::cmf0<ValencyEvent::FirstType, ValencyEvent> > >();
    oa.register_type<LazyCD<HeirRoleVectorEvent,
      HeirRoleVectorEvent::FirstType,
      HeirRoleVectorEvent::SecondType,
      boost::_mfi::cmf0<HeirRoleVectorEvent::FirstType, HeirRoleVectorEvent> > >();
    oa.register_type<LazyCD<HeirEvent,
      HeirEvent::FirstType,
      HeirEvent::SecondType,
      boost::_mfi::cmf0<NTLink, HeirEvent> > >();
    oa.register_type<LazyCD<Dependency,
      Dependency::FirstType,
      Dependency::SecondType,
      boost::_mfi::cmf0<NTLink, Dependency> > >();
    oa.register_type<LazyCD<Dependency,
      Vector<unsigned short>,
      Vector<unsigned short>,
      boost::_bi::bind_t<Vector<unsigned short>,
        boost::_mfi::cmf1<Vector<unsigned short>, Dependency, unsigned short>,
        boost::_bi::list2<boost::arg<1>, boost::_bi::value<unsigned short> > >
      > >();
    oa.register_type<LazyCD<PAOrderEvent,
      PAOrderEvent::FirstType,
      PAOrderEvent::SecondType,
      boost::_mfi::cmf0<const NTPair&, ImDom<NTPair, PAV> > > >();
    oa << *grammar;
  }

  LearnableGrammar* readBinaryArchive(boost::archive::binary_iarchive & ia, LearnableGrammar* grammar) {
    ia >> GlobalVocab::s_synCatSet;
    ia >> GlobalVocab::s_terminalSet;
    ia >> GlobalVocab::s_noVocabsIn;
    ia >> GlobalVocab::s_ioDIM;
    ia.register_type<HeadPMTGA>();
    ia.register_type<LexNT>();
    ia.register_type<LazyCD<ValencyEvent,
     ValencyEvent::FirstType,
     ValencyEvent::SecondType,
     boost::_mfi::cmf0<ValencyEvent::FirstType, ValencyEvent> > >();
   ia.register_type<LazyCD<HeirRoleVectorEvent,
     HeirRoleVectorEvent::FirstType,
     HeirRoleVectorEvent::SecondType,
     boost::_mfi::cmf0<HeirRoleVectorEvent::FirstType, HeirRoleVectorEvent> > >();
   ia.register_type<LazyCD<HeirEvent,
     HeirEvent::FirstType,
     HeirEvent::SecondType,
     boost::_mfi::cmf0<NTLink, HeirEvent> > >();
   ia.register_type<LazyCD<Dependency,
     Dependency::FirstType,
     Dependency::SecondType,
     boost::_mfi::cmf0<NTLink, Dependency> > >();
   ia.register_type<LazyCD<Dependency,
     Vector<unsigned short>,
     Vector<unsigned short>,
     boost::_bi::bind_t<Vector<unsigned short>,
       boost::_mfi::cmf1<Vector<unsigned short>, Dependency, unsigned short>,
       boost::_bi::list2<boost::arg<1>, boost::_bi::value<unsigned short> > >
     > >();
  ia.register_type<LazyCD<PAOrderEvent,
      PAOrderEvent::FirstType,
      PAOrderEvent::SecondType,
      boost::_mfi::cmf0<const NTPair&, ImDom<NTPair, PAV> > > >();
  ia >> grammar;
  return grammar;
  }
};

#endif
