#include "BestFirstComparator.hpp"

// just use default constructor
// BestFirstComparator::BestFirstComparator(): {}

//! Compares two Items according to their scores
//! \return true means that b should be preferred over a
// false means a should be preferred over b.

//It is important to remember that this operator is a less
// then.
bool BestFirstComparator::operator()(const Item* a,
				     const Item* b) const {
  SCORE aH = a->getScore();
  SCORE bH = b->getScore();

  //cout << "aH: " << aH << " bH: " << bH << endl;

  // if the f-cost of items a and b are significantly different
  if(fabs(aH - bH) > TINYSCORE)
    // the lower score (cost) is the better one
    return (aH > bH);

  // Next compare by auxiliary cost (will be zeros if unused)
  SCORE as = a->getAuxScore();
  SCORE bs = b->getAuxScore();
  if(as < bs && bs - as > TINYSCORE) {
    return false;
  } else if(as > bs && as - bs > TINYSCORE) {
    return true;
  }

  //test number of non null dims.
  unsigned short nndA = a->numActiveDims();
  unsigned short nndB = b->numActiveDims();
  if(nndA != nndB) { // if different
    return (nndA < nndB);
  }

  //test volume
  size_t volA = a->volume();
  size_t volB = b->volume();
   if (volA != volB){ // if different
    return (volA < volB);
  }

  // return the younger item, i.e. the one with higher BatchID
  unsigned int aID = a->getBatchID();
  unsigned int bID = b->getBatchID();
  if(aID != bID)
    return (aID < bID);

  // compare inferences
  int ret;
  Inference* infOther = b->getInference().get();
  Inference* infThis = a->getInference().get();
  if ((ret = infThis->cmp(infOther)) != 0)
    return ret;

  //last, use comparator on item itself.
  int x = b->cmp(a) ;
  if(x < 0)
    return true;
  if(x > 0)
    return false;

  // return the younger item, i.e. the one with higher uniqueID
  aID = a->getUniqueID();
  bID = b->getUniqueID();
  if(aID != bID)
    return (aID < bID);

  cerr << "identical items being compared."
       << endl;
  a->dump(cerr);
  cerr << endl;
  b->dump(cerr);
  cerr << endl;

  assert(false);

  return false; // to avoid compiler warning

};
