/**
 * $Id$
 */

#ifndef _NGRAMPROBS_H_
#define _NGRAMPROBS_H_

#ifdef USE_KH_NGRAMS

#include <fstream>
#include "backoff.h"
#include "sym.h"
#include "utility.h"
#include "Exceptions.hpp"
#include "DebugMacros.hpp"
#include "boost/smart_ptr.hpp"

static const short NGRAM_SIZE = 2;

// sorry, these variables must be global, due to mysterious memory issue with member variables here
extern backoff::base<NGRAM_SIZE> khBigramBase; // base holds the training and heldout data
extern backoff::dist<NGRAM_SIZE>* keith_bigram_prob;
extern backoff::dist<NGRAM_SIZE>* keith_unigram_prob;

//! Provides access to ngram probabilities
class BigramProbs {

public:

  BigramProbs(){
    keith_bigram_prob = new backoff::dist<NGRAM_SIZE>(khBigramBase);
    keith_unigram_prob = NULL;
  }

  virtual ~BigramProbs(){}

  void loadModel(const string& modelFileStem);

  inline double getLogProb(const string& a, const string& givenB) const {
    return log(getProb(a, givenB));
  }

  inline double getLogUniProb(const string& a) const {
    return log(getUniProb(a));
  }

  inline double getProb(const string& a, const string& givenB) const {
    assert(keith_bigram_prob);
    symarray<NGRAM_SIZE>::Type dItem;
    dItem[0] = a;
    dItem[1] = givenB;
    double prob = (*keith_bigram_prob)(dItem);
    DEBUG(4, "prob(" << a << " | " << givenB << ") = " << prob << endl);
    return prob;
  }
  inline double getUniProb(const string& a) const {
    assert(keith_unigram_prob);
    symarray<NGRAM_SIZE>::Type dItem;
    dItem[0] = a;
    double prob = (*keith_unigram_prob)(dItem);
    DEBUG(4, "prob(" << a << " ) = " << prob << endl);
    return prob;
  }

};

#else

#include "DummyBigramProbs.hpp"

#endif // USE_KH_NGRAMS

#endif // _NGRAMPROBS_H_
