//! Utility functions related to the boost memory pool and to
//! the class objects of the parser.

#ifndef _BoostMemPool_H_
#define _BoostMemPool_H_

#include "boost/config.hpp"
#include "boost/pool/pool.hpp"
#include "boost/pool/object_pool.hpp"
#include "boost/pool/pool_alloc.hpp"
#include "decorated_singleton_pool.hpp"
#include "decorated_fast_pool_allocator.hpp"

#include "Hedge.hpp"
#include "COMPILER-DEPENDENT_HEADERS.hpp"

using namespace boost;

///////////////////////////////////////////////////////////////
// Memory Pools for Items. //
///////////////////////////////////////////////////////////////

//! Hedge pool, also used for memory management of objects
//! with the same size as Hedges.
typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
                       sizeof(Hedge) >
	HedgePool;

///////////////////////////////////////////////////////////////////
// Hashtable Node Pool.
///////////////////////////////////////////////////////////////////

typedef hash_set<Hedge,
                 HashNS::hash<Hedge>,
		 HashNS::equal_to<Hedge>,
		 boost ::decorated_fast_pool_allocator<Hedge> > :: iterator
		 hash_set_of_Hedges_with_boost_iterator;

//! The mem pool for the stl hash table node type, which wraps the
//! value type T.
#if 0
typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
                       sizeof(hash_set_of_Hedges_with_boost_iterator::_Node)
                       - sizeof(Hedge) + sizeof(Hedge(1))>
	pool_of_Hedges_1D_in_hash_set;

typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
                       sizeof(hash_set_of_Hedges_with_boost_iterator::_Node)
                       - sizeof(Hedge) + sizeof(Hedge(2))>
	pool_of_Hedges_2D_in_hash_set;
#endif
typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
                      sizeof(hash_set_of_Hedges_with_boost_iterator::_Node) >
	pool_of_Hedges_in_hash_set;

typedef hash_set<EngagedHedge,
                 HashNS::hash<EngagedHedge >,
		 HashNS::equal_to<EngagedHedge >,
		 boost ::decorated_fast_pool_allocator<EngagedHedge > > :: iterator
		 hash_set_of_EngagedHedges_with_boost_iterator;

//! The mem pool for the stl hash table node type, which wraps the
//! value type T.
typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
	       sizeof(hash_set_of_EngagedHedges_with_boost_iterator::_Node) >
	pool_of_EngagedHedges_in_hash_set;

typedef hash_set<Hhook,
                 HashNS::hash<Hhook >,
		 HashNS::equal_to<Hhook >,
		 boost ::decorated_fast_pool_allocator<Hhook > > :: iterator
		 hash_set_of_Hhooks_with_boost_iterator;

//! The mem pool for the stl hash table node type, which wraps the
//! value type T.
typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
	       sizeof(hash_set_of_Hhooks_with_boost_iterator::_Node) >
	pool_of_Hhooks_in_hash_set;

typedef decorated_singleton_pool<boost::decorated_fast_pool_allocator_tag,
	       sizeof(void*) > pool_of_pointers;
//! Clears the mempry pool
void clear_boost_pools();

#endif

