#ifndef _BottomUpTranslationLogic_H_
#define _BottomUpTranslationLogic_H_

#include "BottomUpLogic.hpp"
#include <boost/program_options.hpp>

using namespace std;

//! an extension of BottomUpLogic that adds Load inferences
class BottomUpTranslationLogic : public BottomUpLogic {
   typedef BottomUpLogic _super;

 private:

  //! adds the additional options that the configFile contains
  BSP<BPO::options_description> addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

 public:

  /// Constructor.
  BottomUpTranslationLogic
  (const string& configFile,
   BSP<BPO::options_description> priorOptions =
   BSP<BPO::options_description>(new BPO::options_description())
   );

  //! Destructor.
  virtual ~BottomUpTranslationLogic();

 protected: // methods

  //! Generates new inferences triggered by the input item.
  //! This method differs from the one in BottomUpLogic in that it also generates Loads.
  virtual void genNewInfs(const Item* triggerItem, vector<Item*>& itemList);

  //! Generates LOAD inferences.
  /// \return whether it succeeded (false if aborted)
  virtual bool genLoads(arity_t outDim, vector<Item*>& itemVec);

};

#endif
