#ifndef _CSPseudoMTG_H_
#define _CSPseudoMTG_H_

#include "Pair.hpp"

#include "PseudoMTG.hpp"
#include "Inference.hpp"
#include "Hedge.hpp"
#include "PAV.hpp"

using namespace std;

/// A PseudoMTG that uses non-context-free info for inferences
class CSPseudoMTG : public PseudoMTG {
  typedef PseudoMTG _super;
  typedef Grammar _grammar;

  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> optionsDescription);

public:
  CSPseudoMTG(arity_t arity, const string& configFile,
	      BSP<BPO::options_description> priorOptions = 
	      BSP<BPO::options_description>(new BPO::options_description()));

  virtual ~CSPseudoMTG();

  ostream& put(ostream& o) const;

  /// given two items, gets the consequent and returns an inference
  /// unlike the superclass, uses context-aware inferences
  virtual void getInferences(const Item*, const Item*, const Pair<bool, PAV>&, vector<Inference*>&);

  /// overrides superclass to get context-aware features of a consequent
  virtual void getFeats(Hedge* pConseq, WVector<GlobalVocab::FeatureType>& featVec);

  virtual SCORE getInfScore(Inference* pInf) const;

protected: // data
  
  /// from which dimension should features be extracted first?
  /// 0 means dim 0 first and dim 1 second; 1 means dim 1 first and dim 0 second
  arity_t m_featureDimPriority;

};
#endif
