#ifndef _CSTreeConstrainedMTG_H_
#define _CSTreeConstrainedMTG_H_

#include "TreeConstrainedMTG.hpp"

template<class TreeType>
class FeatureGenerator;

/// a TreeConstrainedMTG that can deal with context-aware features
class CSTreeConstrainedMTG: public TreeConstrainedMTG {
  typedef TreeConstrainedMTG _super;
  typedef Grammar _grammar;

  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> optionsDescription);

public:

  /**
   * Constructor.
   * @param configFile The config file of the grammar.
   */
  CSTreeConstrainedMTG(arity_t d, const string& configFile,
		       BSP<BPO::options_description> priorOptions =
		       BSP<BPO::options_description>(new BPO::options_description())
		       );

  virtual ~CSTreeConstrainedMTG();

  /**
   * Updates the content of the grammar.
   * @param tuple The sentence tuple.
   * @return If there is an empty parse read, then return false.
   */
  bool prepareFor(SentenceTuple& tuple);

  virtual void getFeats(Hedge* pConseq, WVector<GlobalVocab::FeatureType>& featVec);

protected: // data

  //! the feature generator
  FeatureGenerator<PositionTree>* m_pFeatGen;
};

#endif
