#ifdef USE_SERIALIZATION
#include "ArchiveMaker.hpp"
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#endif

#include "File.hpp"
#include "Exceptions.hpp"
#include "DebugMacros.hpp"
#include "OptionsPrinter.hpp"
#include <boost/program_options.hpp>
#include "ChartBuilder.hpp"
#include "Chart.hpp"
#include "GPDiscChart.hpp"
#include "HashChart.hpp"
//#include "dc_globals.hpp"
//#include "dc_chart.hpp"
//#include "HedgeDCInterface.hpp"

#ifdef USE_SERIALIZATION
#include <boost/serialization/serialization.hpp>
#endif

ChartBuilder* ChartBuilder::_instance = 0;

ChartBuilder* ChartBuilder::Instance(){
  if(_instance == 0){
    _instance = new ChartBuilder;
  }
  return _instance;
}

//! build grammar
Chart* ChartBuilder::build(const string& configFile,
                               BSP<BPO::options_description> priorOptions) {
  Chart* chart = NULL;

  string optChartType;
  unsigned int optDimensionality;
  unsigned int optInputArity = 0;
  int optFanout = DEFAULT_FANOUT;

  BPO::options_description config("Chart config file options");
  config.add_options()
    ("ChartType", BPO::value<string>(&optChartType), "")
    ("Dimensionality", BPO::value<unsigned int>(&optDimensionality), "") 
    ("InputArity", BPO::value<unsigned int>(&optInputArity), "")
    ("Fanout", BPO::value<int>(&optFanout))
    ;

  BPO::variables_map vm;

  if(configFile != ""){
    cerr << "ChartBuilder: Compiling config file " << configFile << endl;
    ifstream ifs(configFile.c_str());
    store(parse_config_file(ifs, config), vm);
  }

  notify(vm);

  if(debug(1)) {
    cerr << OptionsPrinter(config, vm) << endl;
  }

  if (optChartType == ""){
    THROW("Chart type not specified" << endl);
  }

  if (optInputArity == 0)
    optInputArity = optDimensionality;

  // construct the parser itself
  if (optChartType == "HashChart"){
    chart = new HashChart(optDimensionality);
  } else if (optChartType == "GPDiscChart"){
    chart = new GPDiscChart<Item>(optDimensionality, optInputArity, optFanout);
  } else {
    THROW (cerr << optChartType << " is not a recognized chart. " << endl);
  }

  return chart;

}

