#include "Compose.hpp"
#include "Exceptions.hpp"

Compose::
Compose(const Item *hedge1,
	const Item *hedge2,
        Hedge *hedgeCon,
	const Pair<bool,PAV> &arrg)
  : _super(arrg.second, arrg.first), m_pHedgeCon(hedgeCon) {

  m_pHedgeCon->setInference(this);

  // memoize the antecedents
  m_hedgeAnts.push_back(hedge1);
  m_hedgeAnts.push_back(hedge2);

  // always make the heir to be at the 0th element.
  role_t roleOfTrigger = m_headIsFirst ? 1 : 2;
  if(!m_PAV.isNormalFormForRole(roleOfTrigger)){
    // switch roles
    m_headIsFirst = not m_headIsFirst;
    swap(m_hedgeAnts[0], m_hedgeAnts[1]);
  }
}

void Compose::dump(ostream& out) const {
  _super::dump(out);
  out<<" :- ";
  for(unsigned int i = 0; i < getAnt().size(); i++){
    out<<" ";
    getAnt()[i]->dump(out);
  }
}

void Compose::dumpAnts(ostream& out) const {
  out << "Ant1 = " << m_hedgeAnts[0]->getUniqueID()
      << " and Ant2 = " << m_hedgeAnts[1]->getUniqueID();
}

//! Gets the antecedents.
vector<const Item*> Compose::getAnt() const {
  return m_hedgeAnts;
}

int Compose::typeRank() const { return 15; }

//! sets the consequent to item h. Note that this function
//! deletes the item that it is currently pointing to because
//! it is only used when we copy an item (into the chart).
void Compose::setConseq(const Item* h){
  delete m_pHedgeCon;
  m_pHedgeCon = (Hedge*)h;
}

//! returns true if inferences are equal.
bool Compose::equalTo(const Inference& inf) const {
  return _super::equalTo(inf);
}

//!compares two inferences using PAV and HeirIndices.
int Compose::cmp(const Inference* inf) const {
  if((dynamic_cast<const Compose*>(inf))){
    int ret = _super::cmp(inf);
    if(ret != 0) {
      return ret;
    }
    return 0;
  } else if(this->typeRank() < inf->typeRank()){
      return -1;
  } else if(this->typeRank() > inf->typeRank()){
    return 1;
  } else {
    cerr << "identical items being compared."
	 << endl;
    this->dump(cerr);
    cerr << endl;
    inf->dump(cerr);
    cerr << endl;
    THROW("Error: identical items being compared." << endl);
    
    return 0;
  }
}

