#ifndef _Compose_H_
#define _Compose_H_

#include "Inference.hpp"
#include "Item.hpp"
#include "Hedge.hpp"
#include "Grammar.hpp"
#include "ImDom.hpp"

//class Grammar;

//! The Compose inference rule.
//! It is a template class parameterized over span arrangement.

class Compose : public Inference {
    typedef Inference _super;

public:

//! Constructor.
  //! \param hedge1 : one of the antecedents.
  //! \param hedge2 : the other one of antecedents.
  //! \param hedgeCon : the consequent of this Item
  //! \param arrg : the pair of PAV and role indicating the role of the
  //! heir in the PAV.
  //! \param heirInds : the indices to the heirs in different components
  //! of the antecedents.
  Compose(const Item *hedge1,
	  const Item *hedge2,
	  Hedge *hedgeCon,
	  const Pair<bool,PAV> &arrg);

  //! Destructor.
  ~Compose() {}

  // returns a copy of this
  virtual Inference* clone() const{
    return new Compose(*this);
  };

  //! Language dimensionality.
  arity_t arity() const { return m_pHedgeCon->arity();}

  //! Gets the consequent.
  Item* getConseq() const { return static_cast<Item*>(m_pHedgeCon); }

  //! Gets the antecedents.
  vector<const Item*> getAnt() const;


  virtual vector<const Item*>::iterator begin(){
    vector<const Item*>::iterator i;
    i = m_hedgeAnts.begin();
    return i;
  }

  virtual vector<const Item*>::iterator end(){
    vector<const Item*>::iterator i;
    i = m_hedgeAnts.end();
    return i;
  }

  //! Return the rank of this inference rule, equal to the
  //! valency value of the consequent.
  unsigned int rank() const { return m_hedgeAnts.size();}

  void dump(ostream& out) const;
  void dumpAnts(ostream& out) const;

  size_t volume() const
  { return ((DSV*)(m_pHedgeCon->getDSV()))->getSize(); }

  //! Assigns a rank for this type of inference.
  int typeRank() const ;

  //! sets the consequent to item h. Note that this function
  //! deletes the item that it is currently pointing to because
  //! it is only used when we copy an item (into the chart).
  void setConseq(const Item* h);

  //! returns true if equal.
  bool equalTo(const Inference& inf) const;

  //! compare two inferences.
  int cmp(const Inference* inf) const;

protected: // data

  //! Antecedents. There are three cases about the order of antecedents.
  //! - Case 1, inference is Forward, then the heir is always stored
  //! as the first element.
  vector<const Item*> m_hedgeAnts;

  //! Consequent.
  Hedge* m_pHedgeCon;
};

#endif

