#ifndef _CompositePruningStrategy_H_
#define _CompositePruningStrategy_H_

#include "PruningStrategy.hpp"

// A composite of pruning strategies
class CompositePruningStrategy : public PruningStrategy {
  typedef PruningStrategy _super;
public:

  // constructor
  CompositePruningStrategy() {}

  // Destructor
  virtual ~CompositePruningStrategy() {
    vector<PruningStrategy*>::iterator it;
    for(it = m_strategies.begin(); it != m_strategies.end(); it++)
      delete *it;
  }

  // Add a substrategy
  void add(PruningStrategy* subStrategy) {
    m_strategies.push_back(subStrategy);
  }

  bool prunable(const Item* item) const {
    for (unsigned int i=0; i < m_strategies.size(); i++) {
      if (m_strategies[i]->prunable(item))
	return true;
    }
    return false;
  }

  /** Inserts a chart index into the item index table.*/
  virtual void memoize(const Item* item) {
    for (unsigned int i=0; i < m_strategies.size(); i++)
      m_strategies[i]->memoize(item);
  }

  //! how much threshold will be increased (negative decreased).
  virtual void lowerTh() {
    for (unsigned int i=0; i < m_strategies.size(); i++)
      m_strategies[i]->lowerTh();
  }

  virtual void setTh(SCORE th) {
    for (unsigned int i=0; i < m_strategies.size(); i++)
      m_strategies[i]->setTh(th);
  }

  virtual SCORE getOrigTh() const {
    return m_strategies[0]->getOrigTh();
  }

  virtual SCORE getCurTh() const {
    for (unsigned int i = 0; i < m_strategies.size(); i++) {
      if ( ( m_strategies[i]->getCurTh() != m_strategies[i]->getOrigTh() ) ||
	  (i == m_strategies.size() -1) )
	return m_strategies[i]->getCurTh();
    }
    return SCORE(1); // to avoid compiler warning
  }

  void reset() {
    for (unsigned int i=0; i < m_strategies.size(); i++)
      m_strategies[i]->reset();
  }

private:

  vector<PruningStrategy*> m_strategies;

};
#endif
