#include "CompositeWMTGA.hpp"
#include <vector>
#include "GPDefs.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"
#include "GrammarBuilder.hpp"
#include "Path.hpp"

using namespace std;

CompositeWMTGA::CompositeWMTGA(arity_t dim, const string& configFile,
			       BSP<BPO::options_description> priorOptions)
  : Grammar(dim), GrammarMix(dim, configFile), WMTG(dim, false),
    m_pPrimaryGrammar(NULL), m_pSecondaryGrammar(NULL) {
  init();
}

CompositeWMTGA::CompositeWMTGA(arity_t dim, MixtureType mType)
  : Grammar(dim), GrammarMix(dim, mType), WMTG(dim, false),
    m_pPrimaryGrammar(NULL), m_pSecondaryGrammar(NULL) {
  init();
}

void CompositeWMTGA::init(){
  // our base class constructor could already have added some grammars
  if(m_grammars.size() != 2)
    THROW("dunno what to do with non-binary mixture");

  m_pPrimaryGrammar = dynamic_cast<WMTG*>(m_grammars[0]);
  if(!(m_pPrimaryGrammar)) THROW("First grammar must be a WMTG");

  m_pSecondaryGrammar = dynamic_cast<WMTG*>(m_grammars[1]);
  if(m_pSecondaryGrammar == 0) THROW("Second grammar must be a WMTG");
}

void CompositeWMTGA::clear(){
  _mixbase::clear(); // disambiguate among possible superclasses
}

void CompositeWMTGA::getPossibleConsHeaded(const Item* heir, const Item* depend,
					  const Pair<bool,PAV>& arrangement, vector<HRVIpair >& parents){

  // delegate
  m_pPrimaryGrammar->getHeadedBinaryCons(heir, depend, arrangement, parents);
  // post-process
  m_pSecondaryGrammar->getHeadedBinaryCons(heir, depend, arrangement, parents);
  DEBUG(5, "get cons for " << *heir << " and " << *depend << endl);
}

void CompositeWMTGA::getPossibleConsForTLink ( const TLink& link, const DSV& dsv, vector<Item*>& vItem ) const {

  // delegate
  m_pPrimaryGrammar->getUnaryCons(link, dsv, vItem);
  // post-process
  m_pSecondaryGrammar->getUnaryCons(link, dsv, vItem);
}

/// Compile w2w model from the productions, assuming GCNF.  Currently,
/// this ignores links involving a null in the 0th component.
BSP<W2WDictionary> CompositeWMTGA::getW2WDict() const {
  BSP<W2WDictionary> ret;
  const HasW2WDict* hw2wd = dynamic_cast<const HasW2WDict*>(m_pPrimaryGrammar);
  if (hw2wd)
    ret = hw2wd->getW2WDict();

  return ret;
}


BSP<BPO::options_description> CompositeWMTGA::addOptionDescriptionsTo(BSP<BPO::options_description> optionsDescription){
  //optionsDescription->add_options()
  // ("mixture-type", BPO::value<string>(), "The mixture type")
  // ("grammarbuilder-config", BPO::value<string>(), "The grammars to be mixed")
  // ;
  return optionsDescription;
}
