#ifndef _ConjunctCompositeTerminationTest_H_
#define _ConjunctCompositeTerminationTest_H_

#include "TerminationTest.hpp"

/// A composite of parsing goals. The goal is achieved
/// if all are achieved.
class ConjunctCompositeTerminationTest : public TerminationTest {
  typedef TerminationTest _super;
public:
  typedef _super::HowAchieved HowAchieved;
  typedef _super::GoalID GoalID;

  /// constructor.
  ConjunctCompositeTerminationTest() {}

  //Destructor.
  virtual ~ConjunctCompositeTerminationTest() {
      vector<TerminationTest*>::iterator it;
      for(it = m_goals.begin(); it != m_goals.end(); it++)
      delete *it;
  }

  /// add a subgl, returning a ID for this sub-goal.
  GoalID add(TerminationTest* subGoal) {
    m_goals.push_back(subGoal);
    return (GoalID)(m_goals.size() - 1);
  }

  //! reset the goals
  virtual void reset(){
    TerminationTest::reset();
    GoalID i;
    for(i = 0; i < (GoalID) m_goals.size(); ++i)
      m_goals[i]->reset();
  }

  //! reset the goals
  virtual void reset(DSV& span){
    TerminationTest::reset(span);
    GoalID i;
    for(i = 0; i < (GoalID) m_goals.size(); ++i)
      m_goals[i]->reset(span);
  }

  virtual void redefine(NTLink label) {
    TerminationTest::redefine(label);
    GoalID i;
    for(i = 0; i < (GoalID) m_goals.size(); ++i)
      m_goals[i]->redefine(label);
}

  /// returns true is this goal is achieved.
  /// \param item : the item just derived.
  /// The trace() will return the first goal that is not
  /// achieved or ALL.
  /// \param which : which subgoal is reached. If one goal is NOT
  /// reached, then return its GoalID, otherwise, ALL.
  bool achieved(const Item*item, GoalID& which) {
    GoalID i;
    for(i = 0; i < (GoalID) m_goals.size(); ++i){
      if(!m_goals[i]->achieved(item, which)){
	which = i;
	setGoal(i);
	return false;
      }
    }

    which = ALL;
    setGoal(ALL);
    return true;
  }

  /// Checks whether the goal is reached. Used after
  /// the above method is called.
  bool achieved() const {
    if(allAchieved()){
      return true;
    } else {
      return false;
    }
  }

  /// assignment.
  ConjunctCompositeTerminationTest&
  operator=(const ConjunctCompositeTerminationTest& other){
    _super::operator=(other);
    m_goals = other.m_goals;
    return *this;
  }

private:

  vector<TerminationTest*> m_goals;
};
#endif

