#include "DSV.hpp"
#include "PAV.hpp"
#include "Pair.hpp"
#include "Corner.hpp"
#include "Dbug.hpp"
#include "DebugMacros.hpp"

void CornerNS::getCornersForInsert(const DSV& dsv, const PAV& pav, bool takeFirstRole, Corners& corners){

  short theRole = takeFirstRole ? 1 : 2;
  arity_t dim = pav.arity();
  short other = (theRole == 1) ? 2 : 1;
  FanoutVect fov = pav.getFanoutV();

  if (Dbug::isDebugAll(5))
    cerr << endl << "getCorners for DSV " << dsv << endl;

  corners.resize(dim);

  // for each dimension d
  for(arity_t d = 0; d < dim; d++){
    unsigned int mySpanCount = 0;
    unsigned int otherSpanCount = 0;

    // DEBUG(5, "INSERT: fanoutV(" << (int)d << ") = " << dsv.getFanoutV()[d] << endl);

    if(fov[d] > 0){
      // span of "me" item is (X,Y)
      // find the relative ordering of (me,other) in PAV[d]
      const PrecArray& currPA = pav.getPA(d);
      for(unsigned int i = 0; i < currPA.size(); i++) {
	if(currPA[i] == theRole) {
	  if(i > 0 && currPA[i-1] == other) {
	    // PAV[d] = [other me]
	    // create boundary (X,UP)
	    if(d < dsv.arity() && dsv[d].size() > 0) {
	      short position = dsv[d][mySpanCount].first;  // if there's a trigger to my left	
	      corners[d].push_back(Boundary(position, UP));
	    }
	  }
	  if(i < currPA.size() - 1 && currPA[i+1] == other) {
	    // PAV[d] = [me other]
	    // create boundary (Y,DOWN)
	    if(d < dsv.arity() && dsv[d].size() > 0) {
	      short position = dsv[d][mySpanCount].second;   // if there's a trigger to my right
	      corners[d].push_back(Boundary(position, DOWN));
	    }
	  }
	  mySpanCount++;
	} else if(currPA[i] == other){
	  otherSpanCount ++;
	}
      }
      // if other was not seen
      if (otherSpanCount == 0) {
	// PAV[d] = [me]
	// create boundary (?,?)
	// doesn't matter whether the second parameter is UP or DOWN; just matters that the first is DONT_CARE
	corners[d].push_back(Boundary(DONT_CARE, UP));
      }
      // if me was not seen
      if (mySpanCount == 0) {
	// PAV[d] = other
	// create empty boundary (,)
	// doesn't matter whether the second parameter is UP or DOWN; just matters that the first is BLANK
	corners[d].push_back(Boundary(BLANK, UP));
      }
    }
    // else PAV[d] = []
    else {
      // create empty boundary (,)
      // doesn't matter whether the second parameter is UP or DOWN; just matters that the first is BLANK
      corners[d].push_back(Boundary(BLANK, UP));
    }
  }

  if (Dbug::isDebugAll(5)) {
    cerr<<"getCorners: returning insertion corners ";
    for(arity_t d2 = 0; d2 < corners.size(); ++d2)
      cerr<< corners[d2] << " | ";
    cerr << endl;
  }
}

void CornerNS::getCornersForMatch(const DSV& dsv, const PAV& pav, bool takeFirstRole, Corners& corners) {
  short theRole = takeFirstRole ? 1 : 2;
  short other = (theRole == 1) ? 2 : 1;
  FanoutVect fov = pav.getFanoutV();

  if (Dbug::isDebugAll(5))
    cerr << endl << "getCorners for DSV " << dsv << endl;

  arity_t dim = pav.arity();
  corners.resize(dim);

  // for each dimension d
  for(arity_t d = 0; d < dim; d++) {
    unsigned int mySpanCount = 0;
    unsigned int otherSpanCount= 0;

    // DEBUG(5, "MATCH: fanoutV(" << (int)d << ") = " << dsv.getFanoutV()[d] << endl);

    // if the PAV has an element in dimension d
    if(fov[d] > 0){
      // span of "me" item is (X,Y)
      // find the relative ordering of (me,other) in PAV[d]
      const PrecArray& currPA = pav.getPA(d);
      for(unsigned int i = 0; i < currPA.size(); i++) {
	if(currPA[i] == theRole) {
	  if(i > 0 && currPA[i-1] == other) {  // if there's a matcher to my left
	    // PAV[d] = [other me]
	    // create boundary (X,DOWN)
	    if(d < dsv.arity() && dsv[d].size() > 0) {
	      short position = dsv[d][mySpanCount].first;
	      corners[d].push_back(Boundary(position, DOWN));
	    }
	  }
	  if(i < currPA.size() - 1 && currPA[i+1] == other) { // if there's a matcher to my right
	    // PAV[d] = [me other]
	    // create boundary (Y,UP)
	    if(d < dsv.arity() && dsv[d].size() > 0) {
	      short position = dsv[d][mySpanCount].second;
	      corners[d].push_back(Boundary(position, UP));
	    }
	  }
	  ++mySpanCount;
	} else if(currPA[i] == other){
	  ++otherSpanCount;
	}
      }
      // if other was not seen
      if (otherSpanCount == 0) {
	// PAV[d] = [me]
	// create empty boundary (,)
	// doesn't matter whether the second parameter is UP or DOWN; just matters that the first is BLANK
	corners[d].push_back(Boundary(BLANK, UP));
      }
      // if me was not seen
      if (mySpanCount == 0) {
	// PAV[d] = [other]
	// create boundary (?,?)
	// doesn't matter whether the second parameter is UP or DOWN; just matters that the first is DONT_CARE
	corners[d].push_back(Boundary(DONT_CARE, UP));
      }
    }
    // else if item has no elements in dimension d
    else {
      // PAV[d] = []
      // create boundary (BLANK,BLANK)
      // doesn't matter whether the second parameter is UP or DOWN; just matters that the first is DONT_CARE
      corners[d].push_back(Boundary(BLANK, UP));
    }
  }
  if (Dbug::isDebugAll(5)) {
    cerr<<"getCorners: returning insertion corners ";
    for(arity_t d2 = 0; d2 < corners.size(); ++d2)
      cerr<< corners[d2] << " | ";
    cerr << endl;
  }
}
