#ifndef _CORNERFUNCTIONS_H_
#define _CORNERFUNCTIONS_H_

#include "DSV.hpp"
#include "PAV.hpp"
#include "Pair.hpp"
#include "Dbug.hpp"
#include "DebugMacros.hpp"
#include "Exceptions.hpp"

//! Directed 1-D boundary
//! First refers to the boundary position; second refers to the direction,
//! with true indicating RIGHT, and false indicating LEFT.
typedef Pair<short, bool> Boundary;

// vector of vectors of boundaries; inner vector being for a single dimension
// The entire Corners structure represents a single (conjunctive) match ad
typedef Vector<Vector<Boundary> > Corners;

//! Contains static helper functions for computing corners etc
namespace CornerNS {

  // N.B.: The following conventions are from the p.o.v. of the
  // trigger (current or future).

  // UP and DOWN denote directions in the coordinates for each boundary pair
  // (X,DOWN) can match (0,X), (1,X), (2,X), ... (X-1,X)
  // (X,UP) can match (X,X+1), (X,X+2), ... (X,N)
  static const bool DOWN = 0;
  static const bool UP = 1;

  // (DONT_CARE, 1) denotes a boundary which can match anything
  // (BLANK, 1) denotes the presence of nothing, i.e., an empty span in
  // that dimension
  static const int DONT_CARE = -1;
  static const int BLANK = -2;

  // INSERTION:
  // when inserting the item "me" into the chart,
  // for each dimension d in which "me" has a non-empty span, (fanoutV[d] = 1),
  // there are 5 cases for PAV[d]:
  // PAV[d] = [me other], [other me], [me], [other], or []
  // if PAV[d] = [me] :
  // if me.span == (X,Y), then create boundary (DONT_CARE,DONT_CARE) to insert
  // else if PAV[d] = [me other]:
  // if me.span == (X,Y), then create boundary (Y,DOWN) to insert
  // else if PAV[d] = [other me]:
  // if me.span == (X,Y), then create boundary (X,UP) to insert
  // else if PAV[d] = [other]
  // create boundary (BLANK, BLANK) to insert
  // else if PAV[d] = []
  // create boundary (BLANK,BLANK) to insert
  void getCornersForInsert(const DSV& dsv, const PAV& pav, bool takeFirstRole, Corners& corners);

  // MATCH:
  // when matching the item "me" in the chart,
  // for each dimension d in which "me" has a non-empty span, (fanoutV[d] = 1),
  // there are 5 cases for PAV[d]:
  // PAV[d] = [me other], [other me], [me], [other], or []
  // if PAV[d] = [me] :
  // if me.span == (X,Y), then create boundary (BLANK,BLANK) to match
  // else if PAV[d] = [me other]:
  // if me.span == (X,Y), then create boundary (Y,UP) to match
  // else if PAV[d] = [other me]:
  // if me.span == (X,Y), then create boundary (X,DOWN) to match
  // else if PAV[d] = [other]
  // create boundary (DONT_CARE,DONT_CARE) to match
  // else if PAV[d] = []
  // create boundary (BLANK,BLANK) to match
  void getCornersForMatch(const DSV& dsv, const PAV& pav, bool takeFirstRole, Corners& corners);
};

#endif // _CORNERFUNCTIONS_H_
