#include "DSpan.hpp"

//! Empty constructor.
DSpan::DSpan() : m_DSpan() { }

//! Construct a DSpan from a vector of Spans.
DSpan::DSpan(Vector< Span > pairs) {
  for (unsigned int i = 0; i < pairs.size(); i++)
    m_DSpan.push_back(pairs[i]);
}

//! Destructor.
DSpan::~DSpan() { }

//! Equality test.
bool DSpan::operator==(const DSpan& other) const {
  if(m_DSpan.size() != (other.m_DSpan).size()) return false;

  Vector<Span> :: const_iterator it1, it2;
  for(it1 = m_DSpan.begin(), it2 = (other.m_DSpan).begin();
      it1 != m_DSpan.end(), it2 != (other.m_DSpan).end();
      it1++, it2++) {
    if(it1->first != it2->first ||
       it1->second != it2->second) return false;
  }
  return true;
}

//! Inequality test.
bool DSpan::operator!=(const DSpan& other) const {
  if ((*this) == other) return false;
  return true;
}

//! Less than test.
bool DSpan::operator<(const DSpan& other) const {
  if(cmp(*this, other)<0) return true;
  return false;
}

//! Greater than test.
bool DSpan::operator>(const DSpan& other) const {
  return !operator<(other);
}

//! For hashing.
size_t DSpan::hash() const {
  size_t temp = 1;
  unsigned int sz = m_DSpan.size();
  for(unsigned int i = 0; i < sz; i++){
    temp += m_DSpan[i].first + m_DSpan[i].second * 1061;
    if(i != 0)
      temp*= (1061*i);
  }
  return temp;
}

//! Assignment
DSpan& DSpan::operator=(const DSpan& other) {
  m_DSpan = other.m_DSpan;
  return *this;
}

//! Judge whether it is empty
bool DSpan::empty() const {
  if (m_DSpan.size() > 0) return false;
  return true;
}

//! output.
ostream& operator<<(ostream& out, const DSpan& span) {
  // out<<"[";
  Vector<DSpan::Span> :: const_iterator it;
  for(it = span.begin(); it != span.end(); it++){
    out<<" " << (DSpan::Span)(*it);
  }
  out << " ";
  // out<<"] ";
  return out;
}

//! Input.
// Format may be [ 0 1 3 4 ] or 0 1 3 4 | but must end in | or ]
istream& operator>>(istream& in, DSpan& span) {

  // Clear the span
  span.clear();

  while(true){
    if (in.peek() == '[') {
      char ch;
      in >> ch;
    }
    in >> ws;
    while(in.peek() != '|' && in.peek() != ']'){
      DSpan::Span bd;
      in >> bd;
      in >> ws;
      span.push_back(bd);
    }
    if(in.peek() == '|' || in.peek() == ']')
      break;
  }
  return in;
}

//! Judges whether the DSpan has gap.
bool DSpan::hasGap() const {
  if (m_DSpan.size() > 1) return true;
  return false;
}

//! Returns the gaps in the dspan.
DSpan DSpan::getGaps() const {
  DSpan gaps;
  if (m_DSpan.size() <= 1) {
    (gaps.m_DSpan).clear();
    return gaps;
  }

  Vector<Span> ::const_iterator it_cur, it_aft;

  it_cur = m_DSpan.begin();
  it_aft =it_cur; it_aft++;
  while(it_aft != m_DSpan.end()){
    Span bund;
    bund.first = it_cur->second;
    bund.second = it_aft->first;
    (gaps.m_DSpan).push_back(bund);
    it_cur = it_aft;
    it_aft++;
  }
  return gaps;
}

//! Returns the gaps in the dspan including the
//! gaps at the start and end of the sentence
DSpan DSpan::getGaps(const int indfirst, const int indlast) const {
  DSpan gaps;

  if(m_DSpan.size() == 0) {
    (gaps.m_DSpan).push_back(Span(indfirst, indlast+1));
    return gaps;
  }

  gaps = getGaps();

  Span bund;
  Vector<Span> ::const_iterator it_first = m_DSpan.begin();
  Vector<Span> ::const_iterator it_last = m_DSpan.end();
  it_last--;

  //insert the boundary from the beginning of the sentence
  if(it_first->first > indfirst) {
    (gaps.m_DSpan).insert((gaps.m_DSpan).begin(), Span(indfirst, it_first->first));
  }

  if(it_last->second < indlast+1) {
    (gaps.m_DSpan).push_back( Span(it_last->second, indlast+1));
  }

  return gaps;
}

//! Judge whether this span contains another span.
bool DSpan::contains(const DSpan& other) const {
  span_contain<Span> con; // defined in DSpan.hpp

  Vector<Span>::const_iterator it, it1;
  for(it = (other.m_DSpan).begin(); it != (other.m_DSpan).end(); ++it) {
    bool contained = false;
    for(it1 = ((*this).m_DSpan).begin(); it1 != ((*this).m_DSpan).end(); ++it1) {
      if(con(*it1, *it)) {
	contained = true;
	break;
      }
    }
    if(!contained) {
      return false;
    }
  }
  return true;
}

//! Judge whether this span is contained in the other 1D span
bool DSpan::containedIn(const DSpan& other) const {
  // empty span is always contained in anything
  if (empty())
    return true;

  span_contain<Span> con;

  Vector<Span>::const_iterator it, it1;
  for(it = (other.m_DSpan).begin(); it != (other.m_DSpan).end(); ++it){
    bool contained = false;
    for(it1 = (m_DSpan).begin(); it1 != (m_DSpan).end(); ++it1){
      if(con(*it, *it1)){
	contained = true;
	break;
      }
    }
    if(!contained) {
      return false;
    }
  }

  return true;
}

/** Judge whether two DSpans are overlapped. Note that an empty DSpan is
    never overlapped with another span, even if it is also an empty
    span. */
bool DSpan::overlapped(const DSpan& other) const {
  Vector<Span>::const_iterator it2, it1;
  for(it2 = m_DSpan.begin(); it2 != m_DSpan.end(); it2++){
    for(it1 = (other.m_DSpan).begin(); it1 != (other.m_DSpan).end(); it1++){
      if(((*it1).second > (*it2).first) && ((*it1).first < (*it2).second))
	return true;
      if(((*it2).second > (*it1).first) && ((*it2).first < (*it1).second))
	return true;
    }
  }

  return false;
}

// //! Decides the relationship between two DSpans.
// DSpan::SpanRelationType DSpan::relation(const DSpan& other) const {
//   if(this->contains(other) || other.contains(*this)){
//     return CONTAINING;
//   } else if(this->overlapped(other)){
//     return OVERLAPPED;
//   } else {
//     return GAP;
//   }
// }

//! "Other" is at the direction of "this", in other word,
//! the direction is from "this" to "other".
//! Compoares leftmost/lowest boundary of first Span of each DSpan
DSpan::Direction DSpan::to(const DSpan& otherSpan) const {
  if(size() == 0 || otherSpan.size() == 0){ // 0-0|0-1|1-0 (this-other)
    return UNDEF;
  } else { // 1-1
    if((*this) == otherSpan)
      return EQUAL;
    else if (this->overlapped(otherSpan))
      return OVERLAP;
    else if(front().first < otherSpan.front().first)
      return RIGHT;
    else if(front().first > otherSpan.front().first)
      return LEFT;
    else {
      cerr<<"error in DSpan::to()"<<endl;
      assert(0);
    }
  }
  return UNDEF; // duck compiler warning
}

//! Clears the data inside
void DSpan::clear() {
  m_DSpan.clear();
}

//! Constructor. Create DSpan from Span
DSpan::DSpan(Span bpair) {
  m_DSpan.clear();
  m_DSpan.push_back(bpair);
};

//! Calculate the size of a dspan (number of distinct pieces)
unsigned int DSpan::size() const { return m_DSpan.size(); }

//! Calculate the length of a dspan (total number of words)
size_t DSpan::length() const {
  size_t len = 0;
  Vector<Span>::const_iterator it;

  for(it = m_DSpan.begin(); it != m_DSpan.end(); ++it){
    len += it->second - it->first; }
  return len;
}

// for simple access from other classes, vector properties on the m_DSpan copied to DSpan.
DSpan::Span DSpan::back() const { return m_DSpan.back(); }

DSpan::Span DSpan::front() const { return m_DSpan.front(); }

void DSpan::push_back(Span bp) { m_DSpan.push_back(bp); }

Vector<DSpan::Span> :: const_iterator DSpan::begin() const {
  return m_DSpan.begin();
};

Vector<DSpan::Span> :: const_iterator DSpan::end() const {
  return m_DSpan.end();
};

int DSpan::cmp(const DSpan& one, const DSpan &other) const {
  return cmp(one.m_DSpan, other.m_DSpan);
}

//! Access the data of a dimension.
DSpan::Span& DSpan::operator[](int d) {
  return m_DSpan[d];
}

const DSpan::Span & DSpan::operator[](unsigned int d) const {
  return m_DSpan[d];
}

size_t DSpan::gapSize() const {
  size_t gapSize = 0;
  for(unsigned int j = 1; j < m_DSpan.size(); j++){
    size_t cur_gap = m_DSpan[j].first - m_DSpan[j-1].second;
    gapSize = (cur_gap > gapSize) ? cur_gap : gapSize; }
  return gapSize;
}

DSpan DSpan::operator+(const DSpan& other) const {
  DSpan boundsList = *this;
  bool overlap = false;

  Vector<Span>::iterator it2;
  Vector<Span>::const_iterator it1;

  for(it1 = (other.m_DSpan).begin(); it1 != (other.m_DSpan).end(); it1++){
    for(it2 = (boundsList.m_DSpan).begin(); it2 != (boundsList.m_DSpan).end(); it2++){
      if(it1->first >= it2->second)
	continue;
      else if(it1->second <= it2->first) 
	break;
      else {
	overlap = true; 
	break;
      }
    }
    if(overlap) {
      boundsList.clear();
      return boundsList;
    }
    else {
      it2 = (boundsList.m_DSpan).insert(it2, *it1);
    }
  }

  size_t count = boundsList.size();

  Vector<Span> ::iterator it3;

  if(count > 1) {
    it2 = (boundsList.m_DSpan).begin();
    it3 = (boundsList.m_DSpan).begin(); it3++;

    for(unsigned int i = 0; i < count-1; i++){
      if(it2->second == it3->first){
	it3->first = it2->first;
	it2 = (boundsList.m_DSpan).erase(it2);
	it3 = it2;
	it3++;
      }
      else {
	it2++;
	it3++;
      }
    }
  }
  return boundsList;
}

//! Returns the dejoin if "one" and "other".
//! We can see the dejoinability from the lengths of "one",
//! "other" and the returned value.
DSpan DSpan::operator-(const DSpan& other) const {
  DSpan retBound = *this;
  Vector<Span> :: iterator it;
  Vector<Span> :: const_iterator it_other;

  for(it_other = other.m_DSpan.begin(); it_other != other.m_DSpan.end(); it_other++){
    for(it = retBound.m_DSpan.begin(); it != retBound.m_DSpan.end(); it++){
      if(it->first<= it_other->first &&
	 it->second >= it_other->second) break;
    }

    if( it == retBound.m_DSpan.end()){
      retBound.clear();
      break;
    }
    else {
      Span val = *it;
      it = retBound.m_DSpan.erase(it);

      if(it_other->second != val.second)
	it = retBound.m_DSpan.insert(it, Span(it_other->second, val.second));

      if(it_other->first != val.first)
	it = retBound.m_DSpan.insert(it, Span(val.first, it_other->first));

    }
  }
  //cout << " result: " << retBound<<endl;
  return retBound;
}

//given this vector, split the DSpan into many Dspans, where new borders 
// are inserted at each integer in the vector.
Vector<DSpan> DSpan::split(vector<int> vec){
  Vector<DSpan> ret;
  DSpan ds;


  unsigned int cnt = 0;
  for(unsigned int j = 0; j < m_DSpan.size(); j++){
    if(m_DSpan[j].second <= vec[cnt]){
      ds.push_back(m_DSpan[j]);
    }
    else{
      int last = m_DSpan[j].first;
      while(m_DSpan[j].second >= vec[cnt]){
	if(vec[cnt] > m_DSpan[j].first){
	  Span bp(last,vec[cnt]);
	  ds.push_back(bp);
	  ret.push_back(ds);
	  ds.clear();
	  last = vec[cnt];
	  
	  cnt++;
	  if((cnt >= vec.size()) || (m_DSpan[j].second == vec[cnt])){
	    Span bp(last,m_DSpan[j].second);
	    ds.push_back(bp);
	    ret.push_back(ds);
	    ds.clear();
	    break;
	  }
	}
	else
	  cnt++;
      }
    }
  }

  return ret;
}

      

