
#include "Dependency.hpp"
#include "NTLink.hpp"
#include "Vector.hpp"

Dependency::
Dependency(const FirstType& ft, const SecondType& st) :
  _super(ft.size()) {
  setParent(ft); setDepend(st);
}

size_t Dependency:: hash () const {
    size_t ret = 0;

    for(arity_t d = 0; d < arity(); ++d){
	ret *= 10;
	ret += (*this)[d].hash();
    }
    return ret;
#if 0
  Link<NTLink> lk(getParent(), getDepend());
  return lk.hash();
#endif
}

//! return a vector of shorts which represent parts of the dependecy.
Vector<unsigned short> Dependency::getFromDesc(unsigned short td)const{

  NTLink lnt = getParent();
  //get a vector of unsigned ints which represents the nonterminal.
  Vector<unsigned short> ve;

  for(unsigned int i = 0; i < lnt.size(); i++){
    vector<unsigned short>tmpV(lnt.getNT(i)->vectorDecompose());
    vector<unsigned short>::iterator it;
    for(it = tmpV.begin(); it != tmpV.end(); it++){
      ve.push_back(*it);
    }
  }

  //the input value tells us which child to return.
  if(td == 0){
    ve.push_back(getDepend().getNT(0)->nt());
  }
  else if (td == 1){
    ve.push_back(getDepend().getNT(1)->nt());
  }
  else
    THROW("Dependency::getFromDesc not implemented");

  return ve;
}

NTLink Dependency::getParent() const {
  NTLink ft(size());
  for(arity_t d = 0; d < arity(); d++) ft.setNT(d,(*this)[d].parent()->clone());
  return ft;
}

void Dependency::setParent(const NTLink& ft) {
if(this->arity() != ft.arity())
  setDimity(ft.arity());
 for(arity_t d = 0; d < arity(); d++){
   if((*this)[d].parent() != NULL)
     delete (*this)[d].parent();
   (*this)[d].parent() = ft.getNT(d)->clone();
 }
}

Dependency::SecondType Dependency::getDepend() const {
  SecondType st(arity());
  for(arity_t d = 0; d < arity(); d++) st.setNT(d,(*this)[d].child()->clone());
  return st;
}

void Dependency:: setDepend(const SecondType& st) {
  if(this->arity() != st.arity())
    setDimity(st.arity());
  for(arity_t d = 0; d < arity(); d++) {
    if((*this)[d].child() != NULL)
      delete (*this)[d].child();
    (*this)[d].child() = st.getNT(d)->clone();
  }
}

#if 0
deque<Dependency::element_type> Dependency::serialize() const {
  deque<element_type> elements;
  deque<element_type> dq;
  dq = getParent().serialize();
  elements.insert(elements.end(), dq.begin(), dq.end());

  dq = getDepend().serialize();
  elements.insert(elements.end(), dq.begin(), dq.end());

  return elements;
}
#endif

//! Decategorize the dependency using DummyNT.
Dependency& Dependency::decat() {
    arity_t d;
    for(d = 0; d < arity(); ++d){
	(*this)[d].parent()->nt() = DummyNT;
	(*this)[d].child()->nt() = DummyNT;
    }
    return *this;
}

//!This function treats the child terminal link as a binary value...
//! A dependency is a 0, and an heir is 1.
//! So if there is a dependency of the form HD, it is 0, DH is 1.
//! (D is dependency, H is Heir)

int Dependency::getDistributionNumber()const{
  //if its not lexicalized
  if(!dynamic_cast<const LexNT*>(this->getDepend().getNT(0))){
    return -1;
  }
  int d;
  int ret = 0;
  unsigned int binVal = 1;
  for(d = arity() -1; d>=0; d--){
    if((static_cast<const LexNT*>(this->getDepend().getNT(d)))->lex() == HeirMark)
      ret+= binVal;
    binVal *= 2;
  }
  return ret-1;

}

//! given a distribution number (as seen above), this function returns the
//! size of the conditioned event. This changes because we condition over the
//! parents and all the heirs. The more Heirs in the event, the more we are
//! conditioning on. This is of course a hack and should be updated to work
//! with all dimension sizes
unsigned int Dependency::getLHSsizefromNumber(unsigned int num, arity_t d){
  unsigned int sizes[14]={1,1,2,1,2,2,3,1,2,2,3,2,3,3};
  if (d > 3){
    cerr << "unsupported size in dependency: " << d << endl;
    throw NotImplemented();
  }
  else
    return (d + sizes[num]) * 2 ;
}

