#ifndef GDEPENDENCY_H
#define GDEPENDENCY_H

#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "GPDefs.hpp"
#include "Vector.hpp"
#include "NTLink.hpp"
#include "MoreAlgorithms.hpp"
// #include "MazMemPool.hpp"
#include "ImDom.hpp"
#include <boost/serialization/serialization.hpp>

using namespace std;

//! Dependency model event. Implemented via a Vector.
class Dependency :
  // public MazMemPool ,
  public Vector< ImDom<Nonterminal*, Nonterminal*> > {
 public:

  typedef Nonterminal* element_type;
  typedef Vector< ImDom<element_type, element_type> > _super;
  typedef _super::const_iterator const_iterator;
  typedef NTLink SecondType;
  typedef NTLink FirstType;

  //! Empty constructor.
  Dependency() {}

  //! Default constructor.
  Dependency(const arity_t& dim) : _super(dim) {}

  //! Constructor from two NTLink.
  Dependency(const FirstType& ft, const SecondType& st);

  virtual ~Dependency() {};

  arity_t arity() const { return (arity_t)size(); }

  bool operator<(const Dependency& other) const {
    return (_super::operator<(other));
  }

  void setDimity(const arity_t& d) { resize(d);}

  //! For hashing.
  virtual size_t hash () const;

  //! returns part of the event, depending on the short passed in.
  //! This is used with split heirs.
  Vector<unsigned short> getFromDesc(unsigned short)const;

  //! Returns the parent NTLink.
  NTLink getParent() const;

  //! Sets the parent to be the input NTLink.
  void setParent(const NTLink& pa);

  element_type getParent(const arity_t& d) const
  { return (*this)[d].parent(); }

  element_type& getParent(const arity_t& d)
  { return (*this)[d].parent(); }

  element_type getDepend(const arity_t& d) const
  { return (*this)[d].child(); }

  element_type& getDepend(const arity_t& d)
  { return (*this)[d].child(); }

  //! Returns the dependent.
  SecondType getDepend() const;

  //! Sets the dependent to be the input NTLink.
  void setDepend(const SecondType& dp);

  //! Serializes the content into deque.
  //deque<element_type> serialize() const;

  //! Input line by line.
  friend istream& operator >> (istream& in, Dependency& dep) {
    // arity_t d = dep.arity();
    NTLink ft;
    Dependency::SecondType st;

    in >> ft;
    in >> st;

    dep.setParent(ft);
    dep.setDepend(st);
    return in;
  }

  //! Output.
  friend ostream& operator << (ostream& out, const Dependency& dep)
  { out << dep.getParent()<<" "<<dep.getDepend(); return out; }

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    // ar & boost::serialization::base_object<MazMemPool>(*this);
    ar & boost::serialization::base_object<_super >(*this);
  };

  //! Decategorize the dependency using DummyNT.
  Dependency& decat();

  /// Given the layout of heirs and dependencies in this event, a
  /// unique number is assigned to it.  This tells which distribution
  /// to use. (HD is 0, DH, is 1 in 2D)
  int getDistributionNumber() const;

  //Give the size of the vector of pointer events representing the RHS of this event.
  static unsigned int getLHSsizefromNumber(unsigned int number, arity_t d);

};

namespace HashNS {
  //! Template specialization of STL's hash object.
  template<> struct hash<Dependency> {
    size_t operator()(const Dependency& dep) const
    { return dep.hash();}
  };
}

#endif
