#ifndef _DimSplitEstimator_H_
#define _DimSplitEstimator_H_

#include "OutsideCostEstimator.hpp"
#include "GPDefs.hpp"
#include "Grammar.hpp"
#include "NaiveWMTG.hpp"
#include "Dbug.hpp"
#include "MoreAlgorithms.hpp"
#include "ParseStats.hpp"
#include "HeadPMTGA.hpp"
#include "Parser.hpp"

using namespace std;

//! Computes the outside cost by combining estimates from 1-D parses.
/*!
 * Monolingual parsing is much faster than multilingual parsing.  Ideally,
 * the monolingual parse trees should be similar to their corresponding
 * multi-parse trees.  This estimator uses these facts to try and produce
 * a good estimate.
 */
class DimSplitEstimator : public OutsideCostEstimator {
  typedef OutsideCostEstimator _super;
public:

  //! Constructor.
  DimSplitEstimator(const string& configFile);

  //! Destructor.
  virtual ~DimSplitEstimator();

  //! Precompute grammar information
  //! \param grammar : the grammar model from which we precompute 
  //!                  values.  Note that this model should
  //!                  be estimated from the following treebank
  //!                  grammar BEFORE this method is called.
  virtual void precompute(NaiveWMTG *grammar);

  void initialize(SentenceTuple &sent);

  //! Returns the estimate to the outside score (logprob) of the item.
  //! \param sntT : the sentence tuple, the subtuple of which the item
  //!               concerned is covering.
  //! \param item : the item, for which we are computing the estimate to
  //!               its outside score.
  virtual SCORE operator()(SentenceTuple &sent, const Item* item);

  //! Resets the object.
  virtual void reset();

protected:
  //! The single dimension grammars
  vector<IndexedWMTG*> m_grammars;

  //! The single dimension parsers
  vector<Parser*> m_parsers;

  //! The scores for the 1-D parses
  vector<SCORE> m_scores;

  //! The config file for building the parsers
  string m_parserConfigFile;

  //! The config file for building the gramars
  string m_grammarConfigFile;

  //! Return the 1d outside cost
  SCORE get1DScore(Parser* parser1D, const Hedge *hedge1D, SCORE totalScore);
};

#endif
