//! \brief discriminative learner for synchronous grammars

#ifndef _DiscrimLearner_H_
#define _DiscrimLearner_H_

#include <string>
#include "Timer.hpp"
#include "Grammar.hpp"
#include "Logic.hpp"
#include "Exceptions.hpp"
#include "SearchStrategy.hpp"
#include "Parser.hpp"
#include "LearnableWMTG.hpp"

class Logic;

using namespace std;

/// discriminative learner for synchronous grammars
class DiscrimLearner : public Parser {
  typedef Parser _super;
private:

  //! adds the additional options that the configFile contains
  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

public:

  /// constructor from a config file
  DiscrimLearner(const string& configFile, 
		 Grammar* pGrammar,
		 BSP<BPO::options_description> optionsDescription =
		 BSP<BPO::options_description>(new BPO::options_description()));

  /// does not delete the grammars
  virtual ~DiscrimLearner();

  /// the key method
  virtual bool learn();

  /// initialize the learner for new input
  virtual void initialize(SentenceTuple&);

  //! Clears the learner by clearing the contents of its components.
  //! The components are still there, not being deleted.
  //! The learning grammar is not cleared.
  virtual void clear();

  //! Set the learning grammar
  virtual void setGrammar(Grammar* pGrammar);

 protected: // methods

  // no default constructor from outside
  DiscrimLearner() {}

  void updateWeights(BSP<Inference> pCorrectPrediction, 
		     list<BSP<Inference> >& errorSet,
		     bool forceCorrectUpdate = false);

 protected: // components

  /// the grammar guiding the learner.  Should probably be a PseudoMTG
  /// (even though it'll be used with a TranslationLogic!)
  Grammar* m_pSupervisingGrammar;

  /// the Logic used for generating "correct" guiding inferences,
  /// using the supervising grammar.  Should probably be a TranslationLogic
  /// so that it can properly supervise Loads.
  Logic* m_pSupervisingLogic;

  //! the SearchStrategy used for ordering guiding inferences
  SearchStrategy* m_pSupervisingSearchStrategy;

  //! the grammar being induced, for now assuming LearnableWMTG
  //! typically this will point to the base class's grammar, but with a downcast
  LearnableWMTG* m_pLearningGrammar;


};

#endif
