#include "File.hpp"
#include "IOUtil.hpp"
#include "GPDefs.hpp"
#include "DummyParser.hpp"
#include "AbstractParser.hpp"
#include "Path.hpp"

#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"

// Constructor with a grammar and a configure file.
DummyParser::DummyParser(const string& configFile, 
	       Grammar* pGrammar,
	       BSP<BPO::options_description> optionsDescription)
  : m_pTreeStream(NULL) {

  string treeStreamFile;
  /*
   * Compile the config file
   */
BPO::options_description config("Configuration");
  config.add_options()
    ("TreeFile", BPO::value<string>(&treeStreamFile),
"File containing the supervisory parses");
  
  BPO::variables_map vm;
  cerr << "Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
      cerr << "Option values of \"DummyParser\"" << endl << endl;
      cerr << OptionsPrinter(config, vm) << endl;
  }

  if (treeStreamFile == "") {
    THROW("Tree stream file required");
  }
  m_pTreeStream = new iFile(treeStreamFile);

}

DummyParser::~DummyParser() {
  delete m_pTreeStream;
}

WTreeVec DummyParser:: getParse(SentenceTuple& sntT) {
  WTreeVec parse;
  SCORE treeScore = UNITYSCORE;
  string line;
  
  // read the current line, if eof has been reached, then start over
  // from the beginning 
  getline(*m_pTreeStream, line);
  if (m_pTreeStream->eof()){
    rewind(*m_pTreeStream);
    getline(*m_pTreeStream, line);
  }

  // read in all subtrees for the current tree 
  vector<string> strvec = split(line, "\t");
  for (size_t i = 0; i < strvec.size(); ++i){
    istringstream ist(strvec[i]);
    try {
      HeadMTree* tr = new HeadMTree();
      ist>>*tr;
      parse.push_back(pair<HeadMTree*, SCORE>(tr,treeScore));
    }  catch (std::runtime_error){
      cerr << "caught runtime exception reading tree: " << line << endl;
      cerr << "continuing. ";
    }
      
  }
  return parse;
}

void DummyParser::clear() {}

