//! \brief A Parser by Lookup.
/**
*  This parser is an interface for a treestream.  It assumes that 
* the sentenceTuple being passed in will match the next item on the
* stream and simply returns it.
*/

#ifndef _DummyParser_H_
#define _DummyParser_H_

#include "AbstractParser.hpp"
#include "Grammar.hpp"
#include "Exceptions.hpp"
#include <stdio.h>
#include "HeadMTree.hpp"
#include "SentenceTuple.hpp"

using namespace std;

/// The Dummy Parser.  Instead of "parsing" an input, this parser
// will read parses from an input stream and feed return them as if 
// it was parsing.
class DummyParser : public AbstractParser {

public:

  // constructor from a config file
  DummyParser(const string& configFile, 
	 Grammar* pGrammar = NULL,
	 BSP<BPO::options_description> priorOptions =
	 BSP<BPO::options_description>(new BPO::options_description()));

  // the grammar is not deleted. You need to delete the
  // grammar outside Parser.
  virtual ~DummyParser();

  //! compute the parse and return the resulting tree
  virtual WTreeVec getParse(SentenceTuple&);

  //! Clears the parser by clearing the contents of its components.
  //! The components are still there, not being deleted.
  //! The grammar is not cleared.
  virtual void clear();

 private:

  /// This method kept private to avoid use/redefinition.
  void parse() {
    THROW("Please use either getParse() or silentParse() to avoid confusion." << endl);
  }

  //! add options from the configFile
  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

 protected: // components

  //! The reference to the tree stream
  iFile* m_pTreeStream;

  //! true if we are using a bigram grammar
  bool m_bigram;
  
};

#endif
