#ifndef _FTreeTransducer_H_
#define _FTreeTransducer_H_

#include "GPDefs.hpp"
#include "FeatureWMTG.hpp"
#include "PositionTree.hpp"
#include "HedgeTree.hpp"

#include <boost/program_options.hpp>

class PositionTree;
template<class TreeType>
class FeatureGenerator;
class CSHeadCompose;

//! Bottom-up (frontier-to-root, or "F") tree transducer
class FTreeTransducer : public FeatureWMTG {

  typedef FeatureWMTG _super;
  typedef Grammar _grammar;
public:

  // for serialization only
  FTreeTransducer() {};
  BSP<BPO::options_description> addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

  /// constructor from config file
  FTreeTransducer(arity_t dm, const string& configFile,
	    char** overrideArgV = NULL, size_t overrideArgC = 0,
	    BSP<BPO::options_description> priorOptions 
	    = BSP<BPO::options_description>(new BPO::options_description())
	    );

  //! destructor
  virtual ~FTreeTransducer();

  virtual bool prepareFor(SentenceTuple& tuple);
  virtual bool prepareFor(const Vector<HeadMTree*>& treeVec, vector<unsigned int>& startingPosns);

  //! create an inference from the given tree node and add it to the set
  virtual const ScoreSet<GlobalVocab::FeatureType>&
  treeNode2feats(const HeadMTree::iterator&,
		 const HeadMTree::iterator& hmtAntNode1, const HeadMTree::iterator& hmtAntNode2,
		 const HeirRoleVector& hrv, 
		 const DSV& conseqDSV, const DSV& antDSV1 = DSV(), const DSV& antDSV2 = DSV());
 
  virtual const ScoreSet<GlobalVocab::FeatureType>&
  inf2featsHelper(const Hedge* pConH, const Hedge* pAnt1, const Hedge* pAnt2,
		  const HeirRoleVector& hrv, const DSV& ant1DSV, const DSV& ant2DSV) const;

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super>(*this);
    ar & m_feats;
  };

protected:
  //!1d tree
  PositionTree* m_pRestrictingTree;

  //! stream of 1d trees
  iFile* m_pMonoTBStream;

  //! the feature generator
  FeatureGenerator<PositionTree>* m_pPosFeatGen;

};

#endif
