#include "FactoredItemIndex.hpp"

SCORE FactoredItemIndex::
retrieve(const DSV& aDSV, arity_t d, vector<Item*>& ret) const {
  // select the d-th span.
  DSpan theSpan = aDSV[d];

  SCORE min = POSINF;
  hash_map<DSpan, pair<vector<Item*>, SCORE> >::const_iterator it;

  // locate the index entry using the d-th dimensional span.
  it = m_factoredIndex[d].find(theSpan);
  // if there are items indexed to this entry,
  if(it != m_factoredIndex[d].end()){
    // return the set of index items.
    ret = it->second.first;
    // also the minimum f-value (cost) over these item.
    min = it->second.second;
  } else {
    // if there are no items indexed to this entry, we return nothing.
    ret.clear();
  }
  return min;
}

void FactoredItemIndex::insert(const Item* item) {
  arity_t d;
  const DSV* aDSV = item->getDSV();
  arity_t ar = aDSV->size();
  for(d = 0; d < ar; ++d){
    DSpan theSpan = (*aDSV)[d];
    
    pair<vector<Item*>, SCORE>& theCell = m_factoredIndex[d][theSpan];
    theCell.first.push_back(const_cast<Item*>(item));
    // update the minimum of the scores of inserted items.
    // if there is only one item that has been inserted so far,
    // the minimum score will be the score of the inserted item.
    SCORE currScore = item->getScore();
    if(theCell.first.size() == 1){
      theCell.second = currScore;
    } else {
      //otherwise, update the minimum score.
      SCORE ss = theCell.second;
      if(ss > currScore){
	theCell.second = currScore;
      }
    }
  }
}

// clear all indexes
void FactoredItemIndex::clear(){
  vector<hash_map<DSpan, pair<vector<Item*>, SCORE> > >::iterator i;
  for(i = m_factoredIndex.begin(); i != m_factoredIndex.end(); i++){
    i->clear();
  }
}
