#ifndef _FactoredItemIndex_H_
#define _FactoredItemIndex_H_
#include "ItemIndex.hpp"
#include "DSV.hpp"

//! The index to edges in Chart.
class FactoredItemIndex : public ItemIndex {
public:

  FactoredItemIndex(arity_t d) : m_factoredIndex(d) {}
  virtual ~FactoredItemIndex() {}

  //! Retrieves all the edges covering the d-th component
  //! of the input span.
  //! \param span : the span whose d-th dimensional span will
  //! be used for the retrieval operation.
  //! \param d : the dimension on which span will be projected.
  //! \param ret : the set of items covering the d-th
  //! dimension of span.
  //! \return the minimum cost over all items in ret
  SCORE retrieve(const DSV& span,
		  arity_t d,
		  vector<Item*>& ret) const;

  /// Insert an item in the index.
  void insert(const Item*);

  //! How many factored indexes are there?
  size_t numFactors() const { return m_factoredIndex.size();}

  //! clear all the hash_maps in M_factoredIndex.
  void clear();

protected:

  //! Factored index to the items. Stored dimension by
  //! dimension. Each element is a map from a 1D span
  //! to a pair of a set of items covering the span and
  //! the minimum cost of these items.
  vector<hash_map<DSpan,
		  pair<vector<Item*>, SCORE> > > m_factoredIndex;
};
#endif
