#ifndef _FanoutVect_H_
#define _FanoutVect_H_

#include <numeric>
#include "Vector.hpp"
#include "GPDefs.hpp"
#include <boost/serialization/serialization.hpp>

//! Fanout Vector: Contains the number of discontinuous spans in each dimension.
class FanoutVect : public Vector<short> {
public:

  //! Empty Constructor.
  FanoutVect() {}

  //! Constructor. From dimensionality.
  FanoutVect(const arity_t& dim) : Vector<short>(dim) {}

  FanoutVect(const arity_t& dim, const short v) :
      Vector<short>((const unsigned int)dim, v)
  {}

  //! Initialized from a vector of uint.
  FanoutVect(const Vector<unsigned int>& ve) {
    resize(ve.size());
    for(unsigned int d = 0; d < ve.size(); d++){
      (*this)[d] = (short)ve[d];
    }
  }

  //! Initialized from a vector of short.
  FanoutVect(const Vector<short>& ve) {
    Vector<short>::operator=(ve);
  }

  arity_t arity() const { return (arity_t)size();}

  //! Random access.
  short operator[](arity_t d) const {
    return Vector<short>::operator[](d);
   }

  short& operator[] (arity_t d) {
    return Vector<short>::operator[](d);
  }

  //! Equality test.
  bool operator == (const FanoutVect& other) const{
    return Vector<short>::operator==(other);
  }

  //! Assignment.
  FanoutVect& operator=(const FanoutVect& other){
    Vector<short>::operator=(other);
    return *this;
  }

  //! Returns the overall fanout.
  short overallFanout() const{
    return accumulate(begin(), end(), 0);
  }

  //! Returns the overall fanout.
  short dimensionality() const{
    short ret = 0;
    for(arity_t j = 0; j < size(); j++){
      if ((*this)[j] > 0)
	ret++;
    }
    return ret;
  }

  //! For hashing.
  size_t hash();

  //! Output.
  friend ostream& operator<<(ostream& out, const FanoutVect& fov){
    out<<(const Vector<short>&)fov;
    return out;
  }

  //! Input.
  friend istream& operator>>(istream& in, FanoutVect& fov){
    in >> (Vector<short>&)fov;
    return in;
  }

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<Vector<short> >(*this);
  }

};

namespace HashNS {
  template<> struct hash<FanoutVect > : hash<Vector<short> >{};
}

#endif
