#ifndef _FasterBottomUpTranslationLogic_H_
#define _FasterBottomUpTranslationLogic_H_

#include "BottomUpTranslationLogic.hpp"
#include <boost/program_options.hpp>

using namespace std;

//! an extension of BottomUpLogic that adds LoadCompose inferences
class FasterBottomUpTranslationLogic : public BottomUpTranslationLogic {
  typedef BottomUpTranslationLogic _super;

private:

  //! adds the additional options that the configFile contains
  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

public:

  /// Constructor.
  FasterBottomUpTranslationLogic
  (const string& configFile,
   BSP<BPO::options_description> priorOptions =
   BSP<BPO::options_description>(new BPO::options_description())
   );

  //! Destructor.
  virtual ~FasterBottomUpTranslationLogic();
 
protected: // methods

  //! Generates LOAD inferences.
  virtual bool genLoads(arity_t outDim, vector<Item*>& itemVec);

  /// copy unseen input words to the output, by generating all relevant inferences
  /** If the m_copyUnseen is true, then genLoads will save unseen
     input words in a separate container.  The words are passed by
     position, because this method will need to create their DSV.  This
     method does the following for each unseen input word.
     1. send it to genScansHelper
     2. for each Scan conseq, memoize it immediately, to obtain a valid pointer to a chart item
     3. copy the terminal to the output side, and send it to genScansHelper separately
     4. for each pair in the cross-product of results, send to WMTG::makePretermInference
     5. put the conseq of that inference on the itemVec to be returned
     6. for each Load conseq, insert into the chart without indexing
  */
  void copyUnseenInputs(const vector<size_t>& unseenInputPosns, vector<Item*>& ret);

protected: // data

  BSP<W2WDictionary> m_spW2WModel;
  double m_minTranConf;
  bool m_copyUnseen;
  unsigned int m_maxFertility;
};

#endif
