#ifndef _FeatureGrammar_H_
#define _FeatureGrammar_H_

#include <set>
#include "Dbug.hpp"
#include "GPDefs.hpp"
#include "MoreExceptions.hpp"

#include "LearnableGrammar.hpp"
#include "HeadMTree.hpp"

#include <boost/serialization/serialization.hpp>

using namespace std;

class UpdateRule;

//! A grammar that evaluates synchronous parse structures based on a
//linear mixture of features
class FeatureGrammar : public LearnableGrammar {
  typedef LearnableGrammar _super;

  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> optionsDescription);

  // for serialization only
  FeatureGrammar() {}

public:
  FeatureGrammar(arity_t arity, const string& configFile,
		 BSP<BPO::options_description> priorOptions = 
		 BSP<BPO::options_description>(new BPO::options_description()));

  //! destructor
  virtual ~FeatureGrammar() {}

  // Updates the content of the grammar.
  virtual bool prepareFor(SentenceTuple& tuple);

  //! Decompose a multitree in the way specified by the grammar and learn from it.
  virtual void increment(HeadMTree&, SCORE = UNITYSCORE);

  //! create an inference from the given tree node and add it to the set
  virtual void treeNode2infs(HeadMTree::iterator&, DSV&, SentenceTuple&, Vector<Inference*>&);

  /// overrides superclass to get context-aware features of a consequent
  virtual void getFeats(Hedge* pConseq, Vector<Weighted<GlobalVocab::FeatureType> >& featVec);

  //! Increment features from another grammar
  virtual void increment(const Grammar* );

  //! return the sum of the scores of all features
  virtual SCORE mass();

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    cerr << "SER FEATURE_G" << endl;
    ar & boost::serialization::base_object<_super>(*this);
  }

  /// call-back method to simulate virtual template parameter for UpdateRule::trees2gradient()
  /// computes the margin and return its total cost
  virtual SCORE trees2gradient(const vector<HeadMTree::iterator>& supervisorMargin, 
			       const vector<HeadMTree::iterator>& learnerMargin,
			       const vector<HeadMTree::iterator>& intersectionMargin,
			       Grammar* pCurrModel,
			       const UpdateRule* uRule);

  protected // data

  // memoize current sentence tuple
  SentenceTuple* m_pSentTuple;

};

#endif
