
#ifndef _GPDiscChart_H_
#define _GPDiscChart_H_

#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include <list>

#include <queue>
#include "Exceptions.hpp"
#include "DebugMacros.hpp"

//SONJIA
#include "DiscChartGlobals.hpp"
#include "GPDiscChartInterface.hpp"
#include "DiscChart.hpp"
#include "Hedge.hpp"
#include "ItemSet.hpp"

static const short DEFAULT_FANOUT = 2;

using namespace std;
using namespace boost;
using namespace disc_chart;

//! GenPar::Chart adapter for DiscChart
template<class ItemT = Item, class ItemInterface = GPDiscChartInterface>
class GPDiscChart : public ItemSt {
  typedef ItemSt _super;
  typedef DiscChart<ItemT,ItemInterface> DiscChartType;
  typedef typename DiscChartType::ItemList  ItemList;
  typedef typename DiscChartType::ItemListIter ItemListIter;
  typedef typename DiscChartType::RangeIterator RangeIter;
public:

  //! Constructors
  GPDiscChart(const arity_t& dim, const arity_t& inputArity, 
	      const int fanout = DEFAULT_FANOUT);

  // Clone a HashChart without copying the content.
  GPDiscChart*  memorylessClone() const;

  //! Destructor.
  ~GPDiscChart();


  //! Inserts an item, returns the pointer to the inserted item in chart.
  //! \param item : the item to be inserted.
  //! \param memoRes : the result of the insertion.
  //!            - INSERT: the item inserted  by this method is new;
  //!            - UPDATE: there is an item with the same signature already
  //!                      in chart, and the inside score of the inserted
  //!                      item is larger than the old one;
  //!            - NOEFFECT: there is an item with the same signature
  //!                      already in chart, and has larger inside score;
  //!            - IGNORE: this item is ignored. I.e. because of having
  //!                      minimum infinity score.
  //!            - ERROR : unrecognized item type.
  //! \param arments : the arrangements compatible with this item, according to the grammar.
  Item* insert(Item *item, MemoResult& memoRes, const set< Pair<bool,PAV> >& arments);

  //! Finds the matched items for the input item
  void matchItem(const Item& item, const Pair<bool,PAV>& arment, vector<Item*>& matchedItems);

  //! Clears the content.
  virtual void clear();

  //! Prints the statistics of this class.
  virtual void printStats(ostream& out) const;

  // test if chart iterator has at least k items, sets iterator to kth
  // item if return value is true, or to the end otherwise
  bool hasMin(const RangeIter & it, unsigned int k) {
    unsigned int i = 0;
    while (it.has_next() && i<k) {
      ++it;
      i++;
    }
    return (i>=k);
  }

  // test if hyperrectange contains k or more items
  bool hasMin(const HRect & hr, unsigned int k) {
    RangeIter it = queryHRect(hr);
    return hasMin(it,k);
  }

protected: // methods

  // query chart for a hyperrectange, return chart iterator
  RangeIter queryHRect(const HRect & hr)
  { return m_discChart.query(hr); }

  // assumption: there is only one item per label per position
  static Item* findItem( ItemList* pitem_list, const Item* trigger) {
    for(ItemListIter it = pitem_list->begin(); it!=pitem_list->end(); it++)
      if((*it)->equal_to(trigger))
        return *it;
    
    return NULL;
  }

  unsigned short maxSpansPerDim() const {
    return m_discChart.C;
  }

protected: // data

  DiscChartType m_discChart;

};

#include "GPDiscChart.cpp"

#endif

