#include "GlobalVocab.hpp"
#include "GPDefs.hpp"
#include "StringVocabulary.hpp"

// GLOBAL INITIALIZATIONS
//! Terminal sets
StringVocabulary* GlobalVocab::s_terminalSet = initTermVocab();

//! SynCat sets
StringVocabulary* GlobalVocab::s_synCatSet = initNTVocab();

//! feature set
StringVocabulary* GlobalVocab::s_featSet = new StringVocabulary();

//! global static which tells if we are using vocabs for input.
// it it is true, we are not using vocabs.
bool GlobalVocab::s_noVocabsIn = false;

//! global static which tells if we are using vocabs for output.
// it it is true, we are not using vocabs.
bool GlobalVocab::s_noVocabsOut = false;

//! global static that tells which dimension i/o is happening on.
// this is usful for 1D objects that exist in different dimensions.
arity_t GlobalVocab::s_ioDIM = 0;

arity_t GlobalVocab::s_arity = 2;


// public methods

void GlobalVocab::addPreDefinedSynCat(StringVocabulary& vcb) {
    vcb.addWord("NULL");
    vcb.addWord("ROOT");
    vcb.addWord("EOSH");
    vcb.addWord("DUM");
    vcb.addWord("UNK");
    vcb.addWord("-OOB-");
    vcb.addWord("UNKPT");
    vcb.addWord("UNDEF");
    vcb.addWord("HEIRMARK");
    // the next nonterminal starts from 11.
    //    vcb._curIndex = 11;
}

void GlobalVocab::addPreDefinedTerminal(StringVocabulary& vcb) {
    vcb.addWord("null");
    vcb.addWord("root");
    vcb.addWord("eosh");
    vcb.addWord("dummy");
    vcb.addWord("unk");
    vcb.addWord("-oob-");
    vcb.addWord("unkprtm");
    vcb.addWord("undef");
    vcb.addWord("heirmark");
    // the next nonterminal starts from 11.
    //    vcb._curIndex = 11;
}

StringVocabulary* GlobalVocab::initNTVocab(){
  StringVocabulary* vcb = new StringVocabulary;
  addPreDefinedSynCat(*vcb);
  return vcb;
}

StringVocabulary* GlobalVocab::initTermVocab(){
  StringVocabulary* vcb = new StringVocabulary;
  addPreDefinedTerminal(*vcb);
  return vcb;
}

// //! output operator for vocabdefs
// ostream& operator<<(ostream& out, const GlobalVocab& vcbd) {
//   return GlobalVocab::put(out); 
// }

ostream& GlobalVocab::put(ostream& out) {
  out << "dumping terminal vocab " << endl;
  out << *(s_terminalSet);
  out << "dumping nonterminal vocab" << endl;
  out << *(s_synCatSet);
  return out;
}
