/*! \file GrammarBuilder.hpp
 * \brief GrammarBuilder class.
 * created Wed Mar 2 21:13:50 EST 2005
 * header by mkh script
 */

#ifndef GRAMMARBUILDER_H
#define GRAMMARBUILDER_H

#include "Dbug.hpp"
#include <vector>

class Grammar;
class StringVocabulary;

using namespace std;

class GrammarBuilder : public Dbug {

 public:
  //! Returns a pointer to the instance of this class.
  static GrammarBuilder* Instance();

  /**
  * Builds the grammar based on a configure file. If the grammar
  * file name is specified in the config file, then load the grammar.
  *
  * @param configFile The config file of the grammar.
  * @param overrideArgV The argument vector used to override
  * the config file.
  * @param overrideArgC The size of the override argv.
  */
  Grammar* build(const string& configFile,
		 StringVocabulary* synCatVcb = NULL,
		 StringVocabulary* tvcb = NULL,
		 char** overrideArgV = NULL,
		 size_t overrideArgC = 0);

  Grammar* buildOne(const string& prunerType, arity_t arity, string& configFile,
		    char** overrideArgV, size_t overrideArgC,const string& origConfig);

 protected:
  GrammarBuilder(){}

 private:
  static GrammarBuilder* _instance;

};

#endif
