#include "MoreMath.hpp"
#include "GrammarMix.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"
#include "GrammarBuilder.hpp"
#include "Path.hpp"
#include "File.hpp"
#include "DebugMacros.hpp"

//! static storage of mixture type strings
vector<string> sm_MixTypeNames;

//! lazy initialize the mixture type names (for output)
vector<string>& mixtTypeNames(void) {
  if (sm_MixTypeNames.empty()) {
    sm_MixTypeNames[(int)GrammarMix::UNIFORM] = string("Uniform");
    sm_MixTypeNames[(int)GrammarMix::GREEDY] = string("Greedy");
    sm_MixTypeNames[(int)GrammarMix::MixtureTypeMax] = string("UNDEFINED");
  }
  return sm_MixTypeNames;
}

//! constructors
//! mixture grammars set WMTG::m_inGCNF false as the mixture itself is
//! not in GCNF even if components of the mixture are.
GrammarMix::GrammarMix(arity_t dim,
		       GrammarMix::MixtureType mType)
  : _super(dim), m_mtype(mType)
{}

GrammarMix::GrammarMix(arity_t dim, const string& mainConfigFile) :
  _super(dim), m_mtype(UNIFORM) {
  vector<string> grammarBuilderConfigFiles;
  string mTypeString;
  BPO::options_description opt("GrammarMix options");
  opt.add_options()
    ("mixture-type", BPO::value<string>(&mTypeString), "The mixture type")
    ("grammarbuilder-config", BPO::value< vector<string> >(&grammarBuilderConfigFiles), "The grammars to be mixed")
    ;
  BPO::variables_map vm;
  cerr << "Compiling config file " << mainConfigFile << endl;
  iFile ifs(mainConfigFile.c_str());
  store(parse_config_file(ifs, opt), vm);
  notify(vm);
  if(mTypeString == "uniform"){
    m_mtype = UNIFORM;
  }else if(mTypeString == "greedy"){
    m_mtype = GREEDY;
  }else{
    // default
  }
  // Process config files for the different grammar builders
  size_t mixtureSize = 0;
  for(vector<string>::const_iterator it = grammarBuilderConfigFiles.begin();
      it != grammarBuilderConfigFiles.end(); ++it){
    string grammarBuilderConfigFile = Path::maybe_compose(mainConfigFile, *it);
    Grammar* grammar = GrammarBuilder::Instance()->build(grammarBuilderConfigFile);
    mixtureSize = add(grammar);
    DEBUG(5, "Added grammar" << endl);
  }
}

//! destructor
GrammarMix::~GrammarMix() {
  m_grammars.clear();
}

void GrammarMix::clear(){
  for(_GrammarVector::size_type i = 0; i < m_grammars.size(); ++i){
    m_grammars[i]->clear();
  }
}

//! Read.
//! Just read the mixture type for now
istream& GrammarMix::get(istream& is) {
  THROW("GrammarMix::get not implemented");
  int mType;
  is >> mType >> ws;
  m_mtype = static_cast<MixtureType>(mType);
  /*
      is >> numGrammars;
      for (int i; i < numGrammars; i++) {
      // need to create new grammars...
      }
  */
  return is;
}

//! Write
ostream& GrammarMix::put(ostream& os) const {
  os << static_cast<unsigned int>(m_mtype) << endl;
  if (debug(6)) {
    os << "GrammarMix:" << endl;
    os << m_grammars.size();
    _GrammarVector::const_iterator cGram = m_grammars.begin(); 
    for (;cGram != m_grammars.end(); cGram++) {
      os << *cGram;
    }
  }
  return os;
}

