#ifndef _GrammarMix_H_
#define _GrammarMix_H_

#include <string>
#include "ScoreDefs.hpp"
#include "StringVocabulary.hpp"
#include "Grammar.hpp"

using namespace std;

//! Grammar Mixture Class
//! An abstract base class for mixing multiple grammars.
class GrammarMix : virtual public Grammar {

  typedef Grammar _super;

public:
  enum MixtureType {
    UNIFORM = 0,
    GREEDY = 1,
    MixtureTypeMax = 2};

  GrammarMix(arity_t dim,
	     MixtureType mType = UNIFORM);

  GrammarMix(arity_t dim, const string& configFile);

  //! destructor
  virtual ~GrammarMix();

  // notify the grammar that the end is nigh, in case it wants to save any state
  virtual void clear();

  //! add a new Grammar to the mixture
  //! \param grammar Pointer to a Grammar object
  //! \return Number of grammars in the mixture
  virtual size_t add(Grammar* grammar) {
    m_grammars.push_back(grammar);
    return m_grammars.size();
  }

  //! return a string of this mixture type
  string mixtureTypeStr(void) const {
    assert(m_mtype < MixtureTypeMax);
    return mixTypeNames()[m_mtype];
  }

  //! Read
  virtual istream& get(istream&);

  //! Write
  virtual ostream& put(ostream&) const;

  //! Adapter of read
  friend istream& operator >> (istream& in, GrammarMix& gr)
  { return gr.get(in); }

  //! Adapter of write.
  friend ostream& operator << (ostream& out, const GrammarMix& gr)
  { return gr.put(out); }

private:
  static vector<string>& mixTypeNames(void);
  static vector<string> sm_MixTypeNames;

protected:

  typedef vector<Grammar*> _GrammarVector;
  _GrammarVector m_grammars; // should be in the inherited classes
  MixtureType m_mtype;
};

#endif
