#ifndef _HashChart_C_
#define _HashChart_C_

#include "HashChart.hpp"
#include "DebugMacros.hpp"

static const size_t ITEM_NUMBER_ESTIMATE = 10000;

// Constructor to reset the content.
HashChart::
HashChart(const arity_t& dim)
  : _super(dim) {

}

// Clone a Chart of the same type as mine without copying the content.

HashChart*
HashChart:: memorylessClone() const {
  const HashChart* newChart =
    new HashChart(arity());
  if(typeid(newChart) == typeid(this)){
    return (HashChart*) newChart;
  } else {
    throw NotImplemented();
    return NULL;
  }
}

// Destructor
HashChart::~HashChart() {}

//! Begin iterator for the iteration over the items contained.


// Reset the content.

void HashChart :: clear() {
  if (debug(7)) {
    matchingIndex_t::iterator mii;
    cell_t::iterator cci;
    cerr << "Deleting matching index " << m_uniqueID 
	 << " of " << m_matchingIndex.size() << " cells:";
    for(mii = m_matchingIndex.begin(); mii != m_matchingIndex.end(); ++mii) {
      cerr << "Corners: " << mii->first << " with "
	   << mii->second.size() << " items: " << endl;
      cell_t& cell = mii->second;
      while(not cell.empty()) {
	cerr << "deleting " << cell.back();
	cell.pop_back();
      }
      m_matchingIndex.erase(mii);
    }
  }

  m_matchingIndex.clear();
  
  _super::clear();
}


//! Inserts an item, returns the pointer to the inserted item in chart.
//! \param item : the item to be inserted.
//! \param memoRes : the result of the insertion.
//! \param arrgs: the compatible arrangements for this item, according to the grammar.
Item* HashChart ::
insert(Item * inItem, MemoResult &memoRes, const set< Pair<bool,PAV> >& arrgs) {
  if (debug(4))
    cerr << "HashChart " << m_uniqueID << " ::insert " << *inItem;

  ItemPtr spItem( inItem);
  item_table_type :: iterator iter;
  // if it's not already in the chart
  if((iter = m_closedList.find(spItem)) == m_closedList.end()){

    // insert it here and update the inference to this consequent
    iter = m_closedList.insert(spItem).first;
    BSP<Inference> spInf = spItem->getInference();
    // fuseInference also deletes its previous consequent!
    spItem->fuseInference(spInf);

    // index the input item...
    DSV* key = spItem->getDSV();
    set< Pair<bool,PAV> >::iterator k;
    // note that this loop won't run if arrgs is empty, e.g. for the root
    for(k = arrgs.begin(); k != arrgs.end(); k++){
      Corners corners;
      CornerNS::getCornersForInsert(*key, k->second, k->first, corners);
      if (debug(5))
	cerr << "HC inserting with Corners: " << corners 
	     << " item " << spItem;
      m_matchingIndex[corners].push_back(spItem);
    }
    memoRes = INSERT;
  } else {
    memoRes = updateScoreAndInference( iter->get() /*previous*/,
					  inItem /*current */);
  }

  if (debug(6)) {
    cerr << "HashChart::insert ";
    printStats(cerr);
  }

  // swap (new) chart item for input item, since old item might have
  // been deleted by fuseInference()
  return spItem.get();
}




void HashChart ::
printStats(ostream& out) const {

  // If we're on really high debug level, print the matching index.
  // Otherwise just dump the closed list.
  if (debug(7)) {
    matchingIndex_t::const_iterator mii;
    cell_t::const_iterator cci;
    cerr << "Dumping matching index " << m_uniqueID << " of " << m_matchingIndex.size() << " cells:" << endl;
    for(mii = m_matchingIndex.begin(); mii != m_matchingIndex.end(); ++mii) {
      cerr << "Corners: " << mii->first << " with "
	   << mii->second.size() << " items: " << endl;
      const cell_t& cell = mii->second;
      for(cci = cell.begin(); cci != cell.end(); ++cci)
	cerr << **cci;
    }
  }

  _super::printStats(out);
}


void HashChart::matchItem(const Item & trigger, const Pair<bool,PAV>& arrg, vector<Item*> & ret) {

  // Cast information of the input item
  const DSV* key = trigger.getDSV();
  assert(key);

  Corners corners;
  CornerNS::getCornersForMatch(*key, arrg.second, arrg.first, corners);

  DEBUG(5, "matchItem: trigger item with span=" << *key
	<< "looking for corners: " << corners << endl);

  matchingIndex_t::iterator found = m_matchingIndex.find(corners);
  if(found != m_matchingIndex.end()){
    cell_t& resultSet = found->second;
    cell_t::const_iterator cci;
    for(cci = resultSet.begin(); cci != resultSet.end(); cci++) {
      ret.push_back(cci->get());
      if (debug(5))
	dout() << "MATCHED ITEM: " << *(cci->get()) << endl;
    }
  }
}

/// export all items in the chart
void HashChart::exportAll(Vector<const Item*>& ret) const {
  for(iterator itItem = begin(); itItem != end(); ++itItem)
    ret.push_back( &(*itItem->get()));
};

#endif
