#include "HeadCompose.hpp"
#include "Exceptions.hpp"

HeadCompose::
HeadCompose(const Item *hedge1,
	    const Item *hedge2,
	    Hedge *hedgeCon,
	    const Pair<bool,PAV> &arrg,
	    HeirRoleVector heirIndices)
  :_super(hedge1,hedge2,hedgeCon,arrg), m_heirIndices(heirIndices) {
  // always make the heir to be at the 0th element.
  PAV pav = arrg.second;
  role_t roleOfTrigger = arrg.first ? 1 : 2;

  if(!pav.isNormalFormForRole(roleOfTrigger)){
    m_heirIndices = m_heirIndices.invert();
  }
}


//! returns true if inferences are equal.
bool HeadCompose::equalTo(const Inference& inf) const {
  if(const HeadCompose* r = dynamic_cast<const HeadCompose*>(&inf)){
    if(!(getPAV() == r->getPAV()))
      return false;
    if(!(heirIndices() == r->heirIndices()))
      return false;
    return true;
  }
  else
    return false;
}

//!compares two inferences using PAV and HeirIndices.
int HeadCompose::cmp(const Inference* inf) const {
  int ret;
  if(const HeadCompose* comp = (dynamic_cast<const HeadCompose*>(inf))){

    if((ret = getPAV().cmp(comp->getPAV())) != 0){
      return ret;
    }
    if((ret = heirIndices().cmp(comp->heirIndices())) != 0){
      return ret;
    }
    return 0;
  }
  else
    if(this->typeRank() < inf->typeRank()){
      return -1;
    }
    else if(this->typeRank() > inf->typeRank()){
      return 1;
    }

    else {
      cerr << "identical items being compared."
	   << endl;
      this->dump(cerr);
      cerr << endl;
      inf->dump(cerr);
      cerr << endl;
      THROW("Error: identical items being compared." << endl);

      return 0;
    }
}

void HeadCompose::dump(ostream& out) const {
  Inference::dump(out);
  out << " Heirs: " << heirIndices();
  out<<" :- ";
  for(unsigned int i = 0; i < getAnt().size(); i++){
    out<<" ";
    getAnt()[i]->dump(out);
  }
}

//! Checks whether there is head split in this inference.
bool HeadCompose::headSplit() const {
  return m_heirIndices.headSplit();
}

// find the smallest descendant containing this item's head
Item* HeadCompose::headItem(arity_t dim) const {
  if (m_hedgeAnts.empty())
    return m_pHedgeCon;
  unsigned short heir = m_heirIndices.getRole(dim)-1;
  return m_hedgeAnts[heir]->headItem(dim);
};
