#ifndef _HeadCompose_H_
#define _HeadCompose_H_

#include "Compose.hpp"
#include "Inference.hpp"
#include "Item.hpp"
#include "Hedge.hpp"
#include "Grammar.hpp"
#include "HeirRoleVector.hpp"
#include "ImDom.hpp"

//! The Compose inference rule which includes heads.
//! It is a template class parameterized over span arrangement.

class HeadCompose : public Compose {
  typedef Compose _super;

public:

  //! Constructor.
  //! \param hedge1 : one of the antecedents.
  //! \param hedge2 : the other one of antecedents.
  //! \param hedgeCon : the consequent of this Item
  //! \param arrg : the pair of PAV and role indicating the role of the
  //! heir in the PAV.
  //! \param heirInds : the indices to the heirs in different components
  //! of the antecedents.
  HeadCompose(const Item *hedge1,
	      const Item *hedge2,
	      Hedge *hedgeCon,
	      const Pair<bool,PAV> &arrg,
	      HeirRoleVector heirInds);

  //! Destructor.
  ~HeadCompose() {}

  // returns a copy of this
  virtual Inference* clone() const{
    return new HeadCompose(*this);
  };

  //! returns true if equal.
  bool equalTo(const Inference& inf) const;

  //! compare two inferences.
  int cmp(const Inference* inf) const;

  //! Returns the heirIndices.
  HeirRoleVector heirIndices() const { return m_heirIndices;}

  //! Returns the heirIndices.
  void setHeirIndices(const HeirRoleVector& hrv) {m_heirIndices = hrv;}

  void dump(ostream& out) const;

  //! Returns true if this inference involves head splitting.
  bool headSplit() const;

  // find the smallest descendant containing this item's head on the given dimension
  virtual Item* headItem(arity_t dim) const;

protected: // data

  //! Indices to heirs in different components.
  HeirRoleVector m_heirIndices;

};

#endif
