#include "HeadMTree.hpp"
#include "TLink.hpp"
#include "HeirRoleVector.hpp"
#include "Exceptions.hpp"
#include <map>

//input a Multitree
istream& operator>>(istream& in, HeadMTree& tr) {
  tr.clear();
  // HeadMTree bmt;
  tr.readHeadMTree(in, tr.begin());
  return in;
}

/*
 * Input of HeadMTree.
 */
void HeadMTree::readHeadMTree(istream& in, iterator it) {
  char ch; bool isHeir = false;
  HeadMTreeNode theNode(s_arity);

  if(in.eof()) return;
  in >> ws;
  in>>ch; if(in.eof()) return ;
  if(ch != '('){
    THROW("In HeadMTree input, expected a '(', got "<<ch<<endl);
  }


  iterator it1;
  short num;
  queue<short> d;
  in >> ws;
  while(in.peek() == '*'){
    in >> ch;
    if(isspace(in.peek())){
      in >> ws;
      if(isdigit(in.peek())){
	in >> num;
 	in >> ws;
	d.push(num);
	isHeir = true;
      }
      else{
        in.putback(' ');
	in.putback(' ');
	in.putback('*');
	break;
      }
    }
    else{
      in.putback('*');
      break;
    }
  }

  in >> ws;
  in >> theNode;
  it1 = append_child(it, theNode);
  //if its an heir, set it as the heir in the appropriate dimensions.
  //these dimensions are found in d, and are read from the input.
  if(isHeir){
    HeirTree<tree<HeadMTreeNode> >::setHeirQueue(it1, d);
  }
  in >> ws;
  //  cerr << "Here we see: " << in.peek() << " ";
  while(in.peek() != ')' && in.peek() != '\n' && not in.eof()) {
    //    cerr << "Here we see: " << in.peek() << " ";
    readHeadMTree(in, it1);
  }
  if(in.peek() == '\n' || in.eof()) THROW("Malformed multitree!") ;
  in >> ws;
  in >> ch; // for right paren
  in >> ws;
  string buffer; 
  if(in.peek() == '\n') 
    getline(in, buffer, '\n');
}

//MTree output
ostream& operator<<(ostream& out, const HeadMTree& mTree) {
  const HeadMTree* pT = addressof(mTree);
  pT->print(out, pT, mTree.begin());
  return out;
}
ostream& operator<<(ostream& out, HeadMTree& mTree) {
  const HeadMTree* pT = addressof(mTree);
  pT->print(out, pT, mTree.begin());
  return out;
}
/*
 * Output of HeadMTree.
 */
void HeadMTree::print(ostream& out, const MTree* pT, iterator it) const {
  const HeadMTree* pHMT = dynamic_cast<const HeadMTree*>(pT);
  
  HeadMTree::sibling_iterator sib_it;

  out<<" ( ";
  if(pHMT->is_valid(it) && it != pHMT->end()){
    if(pHMT->isHeir(it) && it != pHMT->begin() ) {
      //if it is an heir, find out in what dimensions.
      queue<short> d = pHMT->getHeirQueue(it);
      //output the appropriate dimensions for the heir.
      while(!d.empty()){
	out<<" * ";
	out << ((short)d.front());

	d.pop();
      }
      out << " ";
    }
    out<<*it;
    for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
      print(out, pT, sib_it);
  }

  out<<" )";
  return ;
}

/*
 * Indent printing.
 * @param out The output stream.
 * @param tr The parse tree to be output.
 * @param ntset The global set of nonterminal vocabs
 * @param terminalSet The global set of terminal vocabs
 * @param ptbII If true, the output form is consistent with the PTB
 * format. Note that the parse tree must be monolingual when ptbII
 * is set true.
 * @param newline whether to break lines
 */
void HeadMTree::indentPrint(ostream& out,
			    StringVocabulary* ntset,
			    StringVocabulary* terminalSet,
			    bool ptbII, bool newline) {
  indentPrint_helper(out, begin(), ntset, terminalSet,
		     0, ptbII, newline);
}

/*print num spaces.*/
void HeadMTree::indent(ostream& out, size_t numsp) {
  for(size_t z = 0; z < numsp; ++z) { out<<" ";}
}

/**
 * Print a subtree begined at iterator it. Arrangement 'ind' is the number
 * of spaces for indent printing.
 */
void HeadMTree::indentPrint_helper(ostream& out,
				   iterator it,
				   StringVocabulary* ntset,
				   StringVocabulary* terminalSet,
				   int ind, bool ptbII, bool newline) {

  static const short INDENT_SIZE = 2; /*2 spaces*/

  // whether we need to break the line before we print the
  // current node.
  if(ptbII){
    // if the parent of the current node is parent, we
    // donut break the line.
    if(parent(it) == begin()){
      // do thing.
    }
    // if the current node is not begin, then it must have a parent.
    else if(it != begin()){
      iterator parI = parent(it);

      // we don't break the line before printing the current
      // node if the current node is the first child of its
      // parent AND is at the pre-terminal level.
      if(it == parI.begin() && !it.number_of_children()){
	// do nothing.
      } else {

	bool allTerminal = true; // all the children of node parI
	// are at the pre-terminal level.
	for(HeadMTree::sibling_iterator i = parI.begin();
	    i != parI.end(); ++i) {
	  if(i.number_of_children()) {
	    allTerminal = false;
	    break;
	  }
	}

	// We break the line before printing the current node if
	// not all the children of its parent are at the
	// pre-terminal level, and (2) it is not true that the current
	// node is the first child of its parent and it is at the
	// pre-terminal level.
	if(!allTerminal){
	  if(newline){
	    out<<endl;
	    indent(out, ind);
	  }
	}
      }
    }

  } else {
    if(newline){
      out<<endl;
      indent(out, ind);
    }
  }

  // left bracketing.
  out<<" (";

  HeadMTree::sibling_iterator sib_it;
  if(is_valid(it) && it != end()){
    // print out the heir marker and heir vector.
    // If the PTB format is desired, we don't print the heir information.
    if(!ptbII && isHeir(it) && it != begin() ) {
      //if we have an heir, find out what dimensions its an heir in.
      queue<short> d = getHeirQueue(it);
      //output the appropriate dimensions that this is an heir in.
      while(!d.empty()){
	out<<"*" << d.front();
	d.pop();
      }
      out << " ";
    }

    /*print the node*/
    it->print(out, ntset, terminalSet, ptbII);

    //out<<endl;

    /*print siblings*/
    for(sib_it = it.begin(); sib_it != it.end(); ++sib_it){
      indentPrint_helper(out, sib_it, ntset, terminalSet,
			 ind+INDENT_SIZE, ptbII, newline);
    }
  }

  //indent(out, ind);
  out<<" )";

}

//! Compares two trees in terms of constituent labels and heirs.
bool HeadMTree::compare(const HeadMTree& tr1, const HeadMTree& tr2,
			vector<StringVocabulary*> ntset,
			vector<StringVocabulary*> tset) {
  return HeadMTree::compare(tr1, tr1.begin(), tr2, tr2.begin(), ntset, tset);
}
//! Helper
bool HeadMTree::compare(const HeadMTree& tr1, const iterator i,
			const HeadMTree& tr2, const iterator j,
			vector<StringVocabulary*> ntset,
			vector<StringVocabulary*> tset) {
  if((tr1.isHeir(i) && ! tr2.isHeir(j)) ||
     (!tr1.isHeir(i) && tr2.isHeir(j))) {
    return false;
  }

  /*compare label*/
  NTLink lab1 = BGET_NT_LABEL(*i);
  NTLink lab2 = BGET_NT_LABEL(*j);
  assert(lab1.arity() == 1);

  bool exists;
  string s1 = "";
  string s2 = "";

  /*nonterminal*/
  s1 += *(ntset[0]->getWord(lab1.getNT(0)->nt(), exists));
  assert(exists);

  s2 += *(ntset[1]->getWord(lab2.getNT(0)->nt(), exists));
  assert(exists);

  if(!(s1 == s2)) {
    return false;
  }

  if(dynamic_cast<const LexNT*>(lab1.getNT(0))){
    s1 = "";
    s2 = "";
    /*terminal*/
    s1 += *(tset[0]->getWord((static_cast<const LexNT*>(lab1.getNT(0)))->lex(), exists));

    s2 += *(tset[1]->getWord((static_cast<const LexNT*>(lab2.getNT(0)))->lex(), exists));

    if(!(s1 == s2)) {
      return false;
    }
  }
  if(!(BGET_CONST_PAV(*i) == BGET_CONST_PAV(*j))){
    return false;
  }

  /*children*/
  if(i.number_of_children() != j.number_of_children()){
    return false;
  }

  HeadMTree::sibling_iterator k, l;
  for(k = i.begin(), l = j.begin(); k != i.end(); ++k, ++l){
    if(!compare(tr1, k, tr2, l, ntset, tset)) { return false;}
  }
  return true;
}

//! Propagate the POS upward until the begin
void HeadMTree::propogatePOS(iterator it) {
  HeadMTree::sibling_iterator heirIter = it.end();
  for(HeadMTree::sibling_iterator j = it.begin(); j != it.end(); ++j){
    propogatePOS(j);
    if( isHeir(j) ) {
      heirIter = j;
    }
  }

  // if found
  if(heirIter != it.end()){
    // copy the POS from the heir
    BGET_NT_LABEL(*it) = BGET_NT_LABEL(*heirIter);
  }
}

//! Transforms the input lexicalized constituent tree
//! to a dependency tree. This is done by deleting all
//! the internal heir nodes and mask all the non-terminals
//! (excluding the part-of-speech tags).
//! This function is applicable to multi-dimensional trees.
void HeadMTree::extractDependencyTree(iterator it) {
  /*transform all the children*/
  for(HeadMTree::sibling_iterator k = it.begin(); k != it.end(); ++k){
    extractDependencyTree(k);
  }

  /*mask the nonterminals if it is not a POS*/
  if(it.number_of_children()){
    NTLink lab = BGET_NT_LABEL(*it);
    for(size_t d = 0; d < lab.arity(); ++d){
      Nonterminal* nt = BGET_NT_LABEL(*it).getNT(d)->clone();
      nt->nt() = UNKNT;
      BGET_NT_LABEL(*it).setNT(d,nt);
    }
  }

  /*if this node is an internal heir node, then
   * delete it*/
  if(it.number_of_children() && isHeir(it) && it != begin()){
    /*find the heir of 'it'*/
    HeadMTree::sibling_iterator j;
    for(j = it.begin(); j != it.end(); ++j){
      if(isHeir(j)) { break;}
    }
    assert(j != it.end());
    iterator pa_i = parent(it);
    reparent(pa_i, it.begin(), it.end());
    setHeir(j);
    erase(it);

  }

  /*
   * If the node is leaf heir node, but its parent has no dependent,
   * then delete it.
   */
  iterator itPa = parent(it);
  if(is_valid(itPa)){
    if(it != begin() && !it.number_of_children() &&
       itPa.number_of_children() == 1){
      BGET_NT_LABEL(*itPa) = BGET_NT_LABEL(*it);
      erase(it);
      for(unsigned int i = 0; i < BGET_HEIRVEC(*itPa).size(); i++)
	BGET_HEIRVEC(*itPa)[i] = 0;
    }
  }
}

//! Besides the work done by extract dependency tree,
//! also propagate the POS from leaves upward to begin.
void HeadMTree::toDependencyTree( ) {
  extractDependencyTree(begin());
  propogatePOS(begin());
}

/**
 * Assignment of trees.
 */
HeadMTree& HeadMTree::copyTree(const HeadMTree& source) {
  clear();
  if(source.begin() == source.end()) { return *this;}

  HeadMTreeNode n(s_arity);

  iterator begin = source.begin();

  /*label*/
  BGET_NT_LABEL(n) = BGET_NT_LABEL(*begin);
  /*PAV*/
  BGET_PAV(n) = BGET_PAV(*begin);

  /*insert the new begin.*/
  append_child(this->begin(), n);

  copyTree_helper(this->begin(), source, source.begin());

  // return dest;
  return *this;

}
void HeadMTree::copyTree_helper(iterator i,
				const HeadMTree& source, iterator j) {
  HeadMTree::sibling_iterator k;
  for(k = j.begin(); k != j.end(); ++k){
    HeadMTreeNode n(s_arity);

    /*label*/
    BGET_EITHER_LABEL(n) = BGET_EITHER_LABEL(*k);

    /*PAV*/
    BGET_PAV(n) = BGET_PAV(*k);

    iterator childIt = append_child(i, n);

    /*heir*/
    if(source.isHeir(k)){
      queue<short> q = source.getHeirQueue(k);
      setHeirQueue(childIt, q);
    }

    /*children*/
    copyTree_helper(childIt, source, k);
  }
}


/** flatten a the subtree begined at the iterator. */
void HeadMTree::flatten(iterator it, HeadMTree& tr) {
  for(HeadMTree::sibling_iterator itsib = it.begin(); itsib != it.end();
      ++itsib){
    flatten(itsib, tr);

    if(itsib.number_of_children()){
      HeadMTree::sibling_iterator k;
      bool isheir = tr.isHeir(itsib);
      if(isheir){
	for(k = itsib.begin(); k != itsib.end(); ++k){
	  if(tr.isHeir(k)){
	    break;
	  }
	}
      }

      tr.reparent(it, itsib.begin(), itsib.end());
      if(isheir){
	tr.setHeir(k);
      }
      tr.erase(itsib);
    }
  }
}

//Converts a positionTree to HeadMTree
// void HeadMTree::positionTree2LexTree(HeadMTree& tr, const SentenceTuple& snt ) {
//   for(iterator it = tr.begin(); it != tr.end(); ++it){
//     NTLink lab = BGET_NT_LABEL(*it);
//     for(arity_t d = 0; d < lab.arity(); ++d){
//       if( BGET_NT_LABEL(*it)[d]->nt() != NULLNT){
// 	if(dynamic_cast<LexNT*>(BGET_NT_LABEL(*it)[0]))
// 	  (static_cast<LexNT*>(BGET_NT_LABEL(*it)[d])->lex() =snt[d][(static_cast<LexNT*>(BGET_NT_LABEL(*it)[d])->lex() -1 ];

//       }
//     }
//   }
// }
/** Obtain a word-to-tag mapping from a MONOLINGUAL tree. */
// void HeadMTree::word2TagMap(hash_map<Terminal, set<SynCat> >& mapping,
// 			     arity_t d)
// {
// iterator i;
// for(i = begin(); i != end(); ++i){
// if(!i.number_of_children()){
// mapping[(static_cast<LexNT*>(BGET_NT_LABEL(*i)[d])->lex()].
// 	insert((static_cast<LexNT*>(BGET_NT_LABEL(*i)[d])->nt());
// }
// }
// }

// /** Obtain a vector of word-to-tag mapping from a multitree.*/
// void HeadMTree::word2TagMap(vector<hash_map<Terminal, set<SynCat> > >& mapping)
// {
// iterator i;
// for(i = begin(); i != end(); ++i){
// if(!i.number_of_children()){
// NTLink lab = BGET_NT_LABEL(*i);
// arity_t d;

// if(mapping.size() < lab.arity()){
// 	mapping.resize(lab.arity());
// }

// for(d = 0; d < lab.arity(); ++d){
// 	mapping[d][(static_cast<LexNT*>(lab[d])->lex()].insert((static_cast<LexNT*>(lab[d])->nt());
// }
// }
// }
// }

/** obtain the PoS tags from the tree.*/
// void HeadMTree::tags(hash_set<SynCat>& tagset, arity_t d)
// {
// iterator i;
// for(i = begin(); i != end(); ++i){
// if(i.number_of_children() == 0){
// tagset.insert((static_cast<LexNT*>(BGET_NT_LABEL(*i)[d])->nt());
// }
// }
// }

// reverse the dimensions.
void HeadMTree::flipTree( ) {
  iterator it = begin();
  HeadMTree::flipTree_helper(it);
}
void HeadMTree::flipTree_helper(iterator it) {
  HeadMTree tr;

  BGET_PAV(*it) = PAV::reverse(BGET_PAV(*it));
  BGET_HEIRVEC(*it).reverse();

  if ( boost::get<NTLink>( &BGET_EITHER_LABEL(*it) ) ){
    BGET_NT_LABEL(*it).reverse();
  }

  else if ( boost::get<TLink>( &BGET_EITHER_LABEL(*it) ) ){
    BGET_TERM_LABEL(*it).reverse();
  }

  else if ( boost::get<Pair<TLink,DSV> >( &BGET_EITHER_LABEL(*it) ) ){
    BGET_TERM_SPAN_LABEL(*it).first.reverse();
    BGET_TERM_SPAN_LABEL(*it).second.reverse();
  }

  HeadMTree::sibling_iterator sib_it;

  if(tr.is_valid(it) && it != tr.end()){
    for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
      flipTree_helper(sib_it);
  }

  return ;
}

void HeadMTree::normalize() {
  iterator it = begin();
  HeadMTree::normalizeHelper(it);
}
void HeadMTree::normalizeHelper(iterator it) {
  HeadMTree tr;
  //if it's a leaf node.
  if ( boost::get<TLink>( &BGET_EITHER_LABEL(*it) ) ){
    return;
  }
  if ( boost::get<Pair<TLink,DSV> >( &BGET_EITHER_LABEL(*it) ) ){
    return;
  }
  // if it's not a leaf node.
  else{
    PAV pav = PAV::normalize(BGET_PAV(*it));
    bool pavChanged = not( pav == BGET_PAV(*it));
    BGET_PAV(*it) = pav;
    vector<sibling_iterator> vecPre;
    HeadMTree::sibling_iterator sib_it;
    if(tr.is_valid(it) && it != it.end()){
      sib_it = it.begin();
      if(pavChanged)
	swap(sib_it);
      for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
	normalizeHelper(sib_it);
    }
    return;
  }
}


void HeadMTree::extractTLinks(Histogram<TLink>& w2wHist) {
  iterator it = begin();
  HeadMTree::extractTLinks_helper(it, w2wHist);
}
//helper class
void HeadMTree::extractTLinks_helper(iterator it, Histogram<TLink>& w2wHist) {
  HeadMTree tr;

  //if it's a leaf node.
  if ( boost::get<TLink>( &BGET_EITHER_LABEL(*it) ) ){
    return;
  }
  // if it's not a leaf node.
  else{
    HeadMTree::sibling_iterator sib_it;
    vector<HeadMTree::sibling_iterator> itVec;
    if(tr.is_valid(it) && it != tr.end()){
      for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
	itVec.push_back(sib_it);
    }

    // if node does not have 2 children, exit
    if(itVec.size() != 2)
      return;

    //get the childrens TLinks
    TLink antTLink1;
    if(boost::get<TLink>( &BGET_EITHER_LABEL(*(itVec[0]))))
      antTLink1 = *boost::get<TLink>( &BGET_EITHER_LABEL(*(itVec[0])));
    else
      antTLink1 = BGET_NT_LABEL(*(itVec[0])).getTLink();

    TLink antTLink2;
    if(boost::get<TLink>( &BGET_EITHER_LABEL(*(itVec[1]))))
      antTLink2 = *(boost::get<TLink>( &BGET_EITHER_LABEL(*(itVec[1]))));
    else
      antTLink2 = BGET_NT_LABEL(*(itVec[1])).getTLink();

    // set up the HeirVector
    HeirRoleVector hv(2);

    if(tr.isHeir(itVec[0],0))
      hv.setRole(0,1);
    else if(tr.isHeir(itVec[0],1))
      hv.setRole(0,2);

    if(tr.isHeir(itVec[1],0))
      hv.setRole(1,1);
    else if(tr.isHeir(itVec[1],1))
      hv.setRole(1,2);

    int deltaPos;
    TLink tl(2); // THIS IS THE TLINK WE ARE TESTING

    switch(getHeirLayout(antTLink1,antTLink2,deltaPos)){
    case NODELTAS:
      //no deltas in this production.
      //A B/ A B (no deltas)
      // no prob score needed.
      // normally, we would add antecedents

      //CASE A DELTA / DELTA A (deltaPos = 1)
      //or DELTA A / A DELTA (deltaPos = 0)
      break;
    case CROSSWAYS:
      if(deltaPos == 0){
	tl[0] = antTLink2[0];
	tl[1] = antTLink1[1];
      }
      else{
	tl[0] = antTLink1[0];
	tl[1] = antTLink2[1];
      }
      w2wHist.increment(tl,1);

      break;
      // A DELTA / B DELTA
      // or DELTA A / DELTA B
    case WHOLEDIM:
      if(hv.getRole(1) == 0){
	tl[0] = antTLink2[0];
	tl[1] = antTLink2[1];
      }
      else{
	tl[0] = antTLink1[0];
	tl[1] = antTLink1[1];
      }
      w2wHist.increment(tl,1);

      break;

      //DELTA A / B B
      //A DELTA / B B
      //A A / B DELTA
      //A A / DELTA B
    case ONEDELTA:
      if(deltaPos == 0 || deltaPos == 1){
	tl[0] = antTLink1[0];
	tl[1] = antTLink1[1];
      }
      else if (deltaPos == 2 || deltaPos == 3){
	tl[0] = antTLink2[0];
	tl[1] = antTLink2[1];
      }
      w2wHist.increment(tl,1);

    }

    if(tr.is_valid(it) && it != tr.end()){
      for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
	extractTLinks_helper(sib_it, w2wHist);
    }
    return ;
  }
}

// this function enumerates different heir layouts.
// It should be combined with the similar function in
// PseudoMTG.
HeadMTree::HeirLayout HeadMTree::getHeirLayout(TLink& antTLink1, TLink& antTLink2, int& posDelta){

  if(antTLink1.size() != 2){
    THROW( "PseudoMTG::getHeirLayout for dim != 2 not implemented");
  }

  //CASE 0: A B/ A B (no deltas)
  if(antTLink1[0] != UNDEF_T &&
     antTLink1[1] != UNDEF_T &&
     antTLink2[0] != UNDEF_T &&
     antTLink2[1] != UNDEF_T ){
    // there is no w2w to apply here it does not effect the probability.
    return NODELTAS;
  }

  //CASE 1: A DELTA / DELTA A
  if(antTLink1[0] != UNDEF_T &&
     antTLink1[1] == UNDEF_T &&
     antTLink2[0] == UNDEF_T &&
     antTLink2[1] != UNDEF_T ){
    posDelta = 1;
    return CROSSWAYS;
  }

  //CASE 1:DELTA A / A DELTA
  if(antTLink1[0] == UNDEF_T &&
     antTLink1[1] != UNDEF_T &&
     antTLink2[0] != UNDEF_T &&
     antTLink2[1] == UNDEF_T ){
    posDelta = 0;
    return CROSSWAYS;
  }

  //CASE 2: A DELTA / B DELTA
  if(antTLink1[0] != UNDEF_T &&
     antTLink1[1] == UNDEF_T &&
     antTLink2[0] != UNDEF_T &&
     antTLink2[1] == UNDEF_T ){
    return WHOLEDIM;
  }

  //CASE 2: DELTA A / DELTA B
  if(antTLink1[0] == UNDEF_T &&
     antTLink1[1] != UNDEF_T &&
     antTLink2[0] == UNDEF_T &&
     antTLink2[1] != UNDEF_T ){
    return WHOLEDIM;
  }

  //CASE 3:DELTA A / B B
  if(antTLink1[0] == UNDEF_T){
    posDelta = 0;
    return ONEDELTA;
  }
  //CASE 3:A DELTA / B B
  else if(antTLink1[1] == UNDEF_T){
    posDelta = 1;
    return ONEDELTA;
  }
  //CASE 3:A A / DELTA B
  else if(antTLink2[0] == UNDEF_T){
    posDelta = 2;
    return ONEDELTA;
  }
  //CASE 3: A A/ B DELTA
  else if(antTLink2[1] == UNDEF_T){
    posDelta = 3;
    return ONEDELTA;
  }
  else{
    THROW("HeadMTree::getHeirLayout : invalid blue print" << endl);
  }
  //will never reach here:
  return ONEDELTA;
}

void HeadMTree::twoSidedDiff(const WTreeVec & vecGold, const WTreeVec & vecCandidate, 
			     vector<iterator>& goldNotCandidate,
			     vector<iterator>& candidateNotGold,
			     vector<iterator>& intersectionVec,
			     bool returnW2WNodes, bool filterNoise){
  vector<int> splitsG;
  vector<unsigned int> length(2,0);
  vector<unsigned int> length2(2,0);

  //first extract the hash of the gold standard side
  hash_map< Pair<DSV,Vector<int> > , iterator > hmapGold;
  hash_map<DSV,Vector<int> >  hmapGold2;
  hash_map<Pair<DSV,Vector<int> >, unsigned int> hmapGoldCnts;
  hash_set<Vector<int> > yieldGold;
  for(unsigned int i = 0; i < vecGold.size(); i++){
    static_cast<const HeadMTree*>(vecGold[i].first)->exportToHash(hmapGold,hmapGold2,hmapGoldCnts,yieldGold,length);
    //keep track of where the gaps are in the gold standard
    if(i+1 != vecGold.size())
      splitsG.push_back(length[0]);
  }

  //next extract the hash of the learned side.
  hash_map< Pair<DSV,Vector<int> > , iterator > hmapCandidate;
  hash_map<DSV,Vector<int> >  hmapCandidate2;
  hash_map<Pair<DSV,Vector<int> >, unsigned int> hmapCandCnts;
  hash_set<Vector<int> > yieldCandidate;
  for(unsigned int i = 0; i < vecCandidate.size(); i++){
    static_cast<const HeadMTree*>(vecCandidate[i].first)->exportToHash(hmapCandidate,hmapCandidate2, hmapCandCnts,yieldCandidate,length2);    
  }
  
  //now its time to compare the two hashs of each.
  hash_map< Pair<DSV,Vector<int> > , iterator >::iterator i;
  hash_map<DSV,Vector<int> >::iterator i2;
  hash_map< Pair<DSV,Vector<int> > , unsigned int>::iterator j;
  hash_map<DSV,Vector<int> >::iterator j2;

  //if any pair of input-span/output string exists in the gold standard, 
  //but does not exist in the other, we keep that node.
  for(i = hmapGold.begin(); i != hmapGold.end(); i++){
    j = hmapCandCnts.find(i->first);
    if(j == hmapCandCnts.end() || j->second == 0){
      //we only want to increment the learned if it's not noise.  
      //its noise if the yield exists in the candidate side. 
      if(!filterNoise 
	 || i->first.second.size() == 0 ||  
	 ( yieldCandidate.find(i->first.second) == yieldCandidate.end())){
	goldNotCandidate.push_back(i->second);
      }else{
	cerr << "FOUND ONE!!!" << "IT IS : " << i->first << endl;
      }
    }else{
      intersectionVec.push_back(i->second);
      j->second--;
    }
  }
  
  //The other direction is more complicated. 
  //If a pair of input-span/output-string exist in the guess, then
  //either it must exist in the gold standard, or there must be n
  //such pairs such that when you stitch them together, they create the
  //correct output.  If such a set of pairs does not exist, we mark
  //that node as wrong. 
  for(i = hmapCandidate.begin(); i != hmapCandidate.end(); i++){
    j = hmapGoldCnts.find(i->first);
    //if the original pair is not found
    if(j == hmapGoldCnts.end() || j->second == 0){
      //if there are no gaps in the gold standard, then we add it
      if(vecGold.size())
	candidateNotGold.push_back(i->second);
      else if(returnW2WNodes) 
	candidateNotGold.push_back(i->second);
      else if(dummyTree.parent(i->second) != NULL )
	candidateNotGold.push_back(i->second);
      else {
	PAV pav = BGET_PAV(*(i->second));
	if(pav.getPA(0).size() != 1 || pav.getPA(1).size() != 1){
	  candidateNotGold.push_back(i->second);
	} else {
	//we double check to see if there are multiple pieces 
	//that can be joined to make what we are looking for.
       
	  DSV dsv = i->first.first;
	  //split the Span into subspans that match 
	  //the gaps in the gold standard
	  Vector<DSpan> vec = dsv[0].split(splitsG);

	  //will hold stithced output
	  Vector<int> total;
       
	  //loop over each sub span
	  for(unsigned int k = 0; k < vec.size(); k++){
	    DSV dsv(2);
	    dsv[0] = vec[k];
	 
	    //look up the span in the gold.
	    j2 = hmapGold2.find(dsv);
	    if(j2 != hmapGold2.end()){
	      //if its found, append the target output to total.
	      total.insert(total.end(), j2->second.begin(), j2->second.end());
	    }
	    else{
	      //if its not found, give up and add the node.
	      break;
	    }
	  }
	  //if the two strings don't match, the node is wrong 
	  if(total != i->first.second){
	      candidateNotGold.push_back(i->second);
	  }
	}
      }
    }
    else{
      j->second--;
      //match found
    }
  }
}

void HeadMTree::exportToHash(hash_map< Pair<DSV,Vector<int> > , 
			     iterator >& hmap,
			     hash_map<DSV,Vector<int> >& hmap2,  
			     hash_map<Pair<DSV,Vector<int> >, unsigned int>& hmapCnts, 
			     hash_set<Vector<int> > & yields,
			     vector<unsigned int>& length){
  iterator it = begin();
  //append spans first.  They are needed.
  this->appendSpans(length);
  Vector<Vector<int> > vec(2);
  DSV dsv(2);

  PAV pav = BGET_PAV(*it);
  
  HeadMTree::exportToHashHelper(it, hmap,hmap2,hmapCnts,yields,vec,dsv);
  
}

void HeadMTree::exportToHashHelper(iterator& it,
				   hash_map< Pair<DSV,Vector<int> > , 
				   iterator >& hmap, 
				   hash_map< DSV,Vector<int> >& hmap2, 
				   hash_map<Pair<DSV,Vector<int> >, unsigned int>& hmapCnts,
				   hash_set<Vector<int> >& yields,
				   Vector<Vector<int> > & yieldVec, 
				   DSV& dsvRet)const{
  
  static HeadMTree tr;
  if(!(tr.is_valid(it)))
    return;
  
  //If it's a leaf
  if(boost::get<Pair<TLink,DSV> >(&BGET_EITHER_LABEL(*it))){
    Pair<TLink,DSV> p = BGET_TERM_SPAN_LABEL(*it);
    //add to yieldVec
    if(p.first[0] != NULLT){
      yieldVec[0].push_back(p.first[0]);
    }
    
    if(p.first[1] != NULLT){
      yieldVec[1].push_back(p.first[1]);
    }
    //get DSV and return it.
    dsvRet = p.second;
    return;
  }
  //else, it's an internal node.
  else{
    //this vector stores the children in order.
    vector<iterator> mtVector;
    for(HeadMTree::sibling_iterator k = it.begin(); k != it.end(); ++k){
      mtVector.push_back(k);
    }
    //if we have two children
    if(mtVector.size() == 2){
      //stucs for child1
      Vector< Vector<int> > yieldVec1(2);
      DSV dsvRet1(2);
      //recurse down for child 1.
      tr.exportToHashHelper(mtVector[0],hmap,hmap2,hmapCnts,yields, yieldVec1,dsvRet1);

      //stucs for child2
      Vector<Vector<int> > yieldVec2(2);
      DSV dsvRet2(2);
      //recure down for child 2
      tr.exportToHashHelper(mtVector[1],hmap,hmap2,hmapCnts,yields, yieldVec2,dsvRet2);

      //strucs for combination of 1 and 2.
      Vector<Vector<int> > yieldVecFin(2);
      DSV dsvRetFin(2);
      
      PAV pav = BGET_PAV(*it);   
      
      //we know that the yields can be combined in left to right order
      //for dim 1.
      yieldVecFin[0].insert(yieldVecFin[0].end(), yieldVec1[0].begin(), yieldVec1[0].end());
      yieldVecFin[0].insert(yieldVecFin[0].end(), yieldVec2[0].begin(), yieldVec2[0].end());
      
      //check to see combination order in dim2.
      if((pav.getPA(1))[0] == 2){
	//backwards order
	yieldVecFin[1].insert(yieldVecFin[1].end(), yieldVec2[1].begin(), yieldVec2[1].end());
	yieldVecFin[1].insert(yieldVecFin[1].end(), yieldVec1[1].begin(), yieldVec1[1].end());
      }
      else{
	//forward order
	yieldVecFin[1].insert(yieldVecFin[1].end(), yieldVec1[1].begin(), yieldVec1[1].end());
	yieldVecFin[1].insert(yieldVecFin[1].end(), yieldVec2[1].begin(), yieldVec2[1].end());
      }
      
      //combine DSVs
      dsvRetFin = dsvRet1 + dsvRet2;
      //empty out second dim of DSVs
      dsvRetFin[1].clear();
      
      //return these stucs
      dsvRet = dsvRetFin;
      yieldVec = yieldVecFin;
      
      //build pair to hash
      Pair<DSV,Vector<int> > pair(dsvRetFin, yieldVecFin[1]);
      if(hmapCnts.find(pair) == hmapCnts.end())
	hmapCnts[pair] = 1;
      else
	hmapCnts[pair] = hmapCnts[pair]+1;
      hmap[pair] = it;
      hmap2[dsvRetFin] = yieldVecFin[1];
      yields.insert(yieldVecFin[1]);
    }
    //else we only have 1 child.
    else{
      tr.exportToHashHelper(mtVector[0],hmap,hmap2,hmapCnts,yields,yieldVec,dsvRet);
      dsvRet[1].clear();
      Pair<DSV,Vector<int> > pair(dsvRet, yieldVec[1]);
      if(hmapCnts.find(pair) == hmapCnts.end())
	hmapCnts[pair] = 1;
      else
	hmapCnts[pair] = hmapCnts[pair]+1;
      hmap[pair] = it;
      hmap2[dsvRet] = yieldVec[1];
      yields.insert(yieldVec[1]);
      
    }
  }
}

// return the position of the headword of a node
// assumes that spans have already been appended
// zero means unknown or nonexistent
short HeadMTree::headPosn(const iterator& it, arity_t dim) {
  if (it.number_of_children() == 0) {
    //    const RHSElem& theRHSE = BGET_EITHER_LABEL(*it);
    DSV theDSV = BGET_TERM_SPAN_LABEL(*it).second;
    if (theDSV.empty(dim))
      return 0;
    else
      return theDSV[dim][0].second; // right boundary of 0th DSpan on relevant dim
  } else if (it.number_of_children() == 1) {
    return headPosn( it.begin(), dim);
  } else {
    // recurse to the node which is the heir on that dim
    const iterator heirIt = findHeir(it, dim);
    if(heirIt == dummyTree.end(it)){
      THROW("No Heir Found in HeadMTree::headPosn");
    }
    return headPosn( heirIt, dim);
  }
};

