#include "HeadPMTG.hpp"

#include <boost/program_options.hpp>

// added for serialization
HeadPMTG::HeadPMTG():_super() {};

/**
 * @param configFile The config file of the grammar.
 */
HeadPMTG::HeadPMTG(arity_t dm,
		   const string& configFile,
		   char** overrideArgV ,
		   size_t overrideArgC,
		   BSP<BPO::options_description> priorOptions )
  :_super(dm,configFile,overrideArgV,overrideArgC,priorOptions){};

// Returns the cost of a MTG production.
SCORE HeadPMTG::getProdScore(Production* rule) const {
  SCORE sum = 0;

  SPVector<FeatureTypes> features = getFeatureBag(rule);
  SPVector<FeatureTypes>::const_iterator it;
  for(it = features.begin(); it != features.end(); ++it){
    sum += getFeatureScore(*(it->get()));
  }
  //  size_t featCount = features.size();
  return sum;
}

/** Increment {\it binary/unary} MTG rules.
    @param score is the increment size
    If the fanout of this rule is greater than the fanout limit, then
    this rule is rejected. The fanout of a production is the
    fanout of the involved PAV */
void HeadPMTG:: increment(Production* pProd, SCORE score) {
  if(debug(3))
    cerr << "HeadWMTG " << m_uniqueID << "::incr by" << score << *pProd << endl;
  // Only valid for productions with rank at most 2.
  assert(pProd->rhsPerm().rank() <= 2);

  // The fanout limit test.
  if(pProd->fanout() > m_fanoutLimit){
    return;
  }
  FanoutVect fanoutv(pProd->rhsPerm().getFanoutV(1));
  FanoutVect fanoutv1(pProd->rhsPerm().getFanoutV(2));

  if( fanoutv.overallFanout() > m_fanoutLimit 
      || fanoutv1.overallFanout() > m_fanoutLimit){
    return;
  }

  // Increment the features into the Grammar.
  SPVector<FeatureTypes> features;
  features = getFeatureBag(pProd);
  //  SCORE updateSize = score / features.size();
  SPVector<FeatureTypes>::const_iterator it;
  for(it = features.begin(); it != features.end(); ++it){
    updateFeatureScore(it->get(), score);
  }

  // remember the whole production (this might eventually disappear)
  IndexedWMTG::updateFeatureScore(pProd, score);
}
