#ifndef _HeadPMTGB_H_
#define _HeadPMTGB_H_

#include "MultiMap.hpp"
#include "Set.hpp"
#include "MultinomialDistribution.hpp"

#include "StringVocabulary.hpp"
#include "Distribution.hpp"

#include "HeadPMTG.hpp"
#include <boost/program_options.hpp>
#include <boost/serialization/serialization.hpp>

class HeadPMTGB : public HeadPMTG {

  typedef HeadPMTG _super;
  typedef Grammar _grammar;
  typedef IndexedWMTG _indexed;

private:
  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

public:

  typedef _super::FeatureTypes FeatureTypes;

  /**
   * Constructor.
   * @param configFile The config file of the grammar.
   */
  HeadPMTGB(arity_t dm, const string& configFile,
	    char** overrideArgV = NULL, size_t overrideArgC = 0,
	    BSP<BPO::options_description> priorOptions =
	    BSP<BPO::options_description>(new BPO::options_description())
	    );

  //! Destructor.
  virtual ~HeadPMTGB();

  //! make it empty
  virtual void clear();

  //! Estimate the probability based on the incremented counts.
  virtual void estimate();

  //! Normalize model parameters or feature scores
  virtual void normalize();

//   //! The cost of having this item as a root
//   virtual SCORE rootCost(const Item*)const;
  
  //! Read.
  virtual istream& get(istream&);

  //! Write
  virtual ostream& put(ostream&) const;

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super>(*this);
    ar & m_depDistUsed;
    ar & m_depHeirDistribs;
    ar & m_pValDistrib;
    ar & m_pDepDistrib;
    ar & m_pHeirDistrib;
    ar & m_pHeirRoleVectorDistrib;
    ar & m_pPermDistrib;
  };

protected: // methods

  //! Returns the cost of a MTG production.
  virtual SCORE getProdScore(Production*) const;

  // test whether a feature exists in the model
  virtual bool knows(FeatureTypes* pFT) const {
    THROW("Not implemented");
  };

  //! Returns the cost of a model event.
  virtual SCORE getFeatureScore(const FeatureTypes&) const;

  //! increment events. Note that score is not log.
  void increment(const FeatureTypes&, SCORE score);

  /// Decompose a production into the feature types of this grammar
  virtual SPVector<FeatureTypes> getFeatureBag(Production* prod) const;

  //!given a dependency event, this gives a pointer to the relevant distribution
  //! for this type of event.
  Distribution<Dependency>* getRelevantDistribution(const Dependency& d, int& i) const;

protected: // data members

  //! descriptors for dep/heir distributions
  bool m_depDistUsed[20];

  Distribution<ValencyEvent>* m_pValDistrib;

  Distribution<Dependency>* m_pDepDistrib;

  Distribution<HeirEvent>* m_pHeirDistrib;

  Distribution<HeirRoleVectorEvent>* m_pHeirRoleVectorDistrib;

  //this holds mixed depend events which depend on heirs
  vector<Distribution<Dependency>*> m_depHeirDistribs;

  /**
   * container for the permutation events.
   */
  Distribution<PAVOrderEvent>* m_pPermDistrib;

};

#endif
