#ifndef _HeadProduction_H_
#define _HeadProduction_H_

#include "Production.hpp"
/*
#include "HeadMTree.hpp"
#include "NTLink.hpp"
#include "PAV.hpp"
#include "Vocabulary.hpp"
*/
#include "HeirRoleVector.hpp"
#include "Bigram.hpp"
/*
#include "ProdRHSElem.hpp"
#include "Exceptions.hpp"
#include "boost/variant.hpp"
#include <boost/serialization/serialization.hpp>
*/

class HeirRoleVector;

//! MTG production with heir information.
class HeadProduction : public Production {
  typedef Production _super;
 public:

  //! default constructor
  HeadProduction() {}

  //! copy constructor
  HeadProduction(const HeadProduction& other);

  //! constructor
  HeadProduction(const NTLink& lhs,
		 const Vector<ProdRHSElem>& rhsLinks,
		 const PAV& pav,
		 const HeirRoleVector& heirIndex);

  //! constructor from inference
  HeadProduction(const Inference* pInf);

  //! Constructs from a HeadMTree iterator.
  HeadProduction(const HeadMTree::iterator&);

  virtual ~HeadProduction(){};

  //! Assignment.
  HeadProduction& operator=(const HeadProduction&);

  //! Equality test.
  bool operator==(const HeadProduction&) const;

  // comparator
  bool less(const HeadProduction& other) const;

  // two-sided comparator
  int cmp(const HeadProduction& other) const;

  //! Returns the link index (1-started) of the heir.
  //! Note that the name of this method is misleading,
  //! since there might be no role for the heir link in
  //! the PAV. Rename it later.
  const HeirRoleVector heirRole() const {
    return m_heirRole;
  }
  HeirRoleVector heirRole() { return m_heirRole;}
 
  //! set the heir role.
  void setHeirRole(const HeirRoleVector& hr){
    m_heirRole = hr;
  }

  //! what do you think this does?
  void clear();

  //! For hashing.
  size_t hash() const;

  //! Remove a RHS element by removing both the link and the role in the
  //! PAV.
  //! \param r : the 1-started link index.

  // makes a copy of this Production
  virtual Production* clone() const{
    return new HeadProduction(*this);
  };

  ///////////////////////////////////////////////////////////
  // Input / Output //
  ///////////////////////////////////////////////////////////

  //!Output a production.
  friend ostream& operator << (ostream&, const HeadProduction&);
  //!Output a production. (needed for polymorphism)
  virtual ostream& put(ostream&) const;

  //!Input a production.
  friend istream& operator >> (istream&, HeadProduction&);
  //!Input a production. (needed for polymorphism)
  virtual istream& get(istream&);

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    Production::serialize(ar, version);
  };
  
  //! Return the data simply as a vector of ints.
  Vector<int> makeVec()const;

  //! get terminal link from production
  bool getTLink(W2WLink& lk) const;

private:

  HeirRoleVector m_heirRole;

};

namespace HashNS {
  //! Template specialization of STL's hash object.
  template<>
  class hash<HeadProduction> {
  public:
    size_t operator()(const HeadProduction& obj) const
    { return obj.hash(); }
  };

  template<>
  class hash<HeadProduction*> {
  public:
    size_t operator()(const HeadProduction* obj) const
    { return obj->hash(); }
  };

  template<>
  class hash<BSP<HeadProduction> > {
  public:
    size_t operator()(const BSP<HeadProduction> obj) const
    { return obj->hash(); }
  };
};

#endif
