#ifndef _HeadWMTGB_H_
#define _HeadWMTGB_H_

#include "MultiMap.hpp"
#include "Set.hpp"
#include "MultinomialDistribution.hpp"

#include "StringVocabulary.hpp"
#include "ScoreSet.hpp"

#include "HeadWMTG.hpp"
#include <boost/program_options.hpp>
#include <boost/serialization/serialization.hpp>

class HeadWMTGB : public HeadWMTG {

  typedef HeadWMTG _super;
  typedef Grammar _grammar;
  typedef IndexedWMTG _indexed;

private:
  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> priorOptions);

public:

  typedef _super::FeatureTypes FeatureTypes;

  /**
   * Constructor.
   * @param configFile The config file of the grammar.
   */
  HeadWMTGB(arity_t dm, const string& configFile,
	    char** overrideArgV = NULL, size_t overrideArgC = 0,
	    BSP<BPO::options_description> priorOptions =
	    BSP<BPO::options_description>(new BPO::options_description())
	    );

  //! Destructor.
  virtual ~HeadWMTGB();

  //! make it empty
  virtual void clear();

  // return the sum of the scores of all features
  virtual SCORE mass();

  //! Increment features from a grammar
  virtual void increment(const Grammar* );

  //! Read.
  virtual istream& get(istream&);

  //! Write
  virtual ostream& put(ostream&) const;

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super>(*this);
    ar & m_depHeirDistribs;
    ar & m_pValDistrib;
    ar & m_pDepDistrib;
    ar & m_pHeirDistrib;
    ar & m_pHeirRoleVectorDistrib;
    ar & m_pPermDistrib;
  };

protected: // methods

  // test whether a feature exists in the model
  virtual bool knows(FeatureTypes* pFT) const;

  //! Returns the cost of a model event.
  virtual SCORE getFeatureScore(const FeatureTypes&) const;

  //! increment events. Note that score is not log.
  void increment(const FeatureTypes&, SCORE score);

  /// Decompose a production into the feature types of this grammar
  virtual SPVector<FeatureTypes> getFeatureBag(Production* prod) const;

  //!given a dependency event, this gives a pointer to the relevant distribution
  //! for this type of event.
  ScoreSet<Dependency>* getRelevantScoreSet(const Dependency& d, int& i) const;

protected: // data members

  ScoreSet<ValencyEvent>* m_pValDistrib;

  ScoreSet<Dependency>* m_pDepDistrib;

  ScoreSet<HeirEvent>* m_pHeirDistrib;

  ScoreSet<HeirRoleVectorEvent>* m_pHeirRoleVectorDistrib;

  // container for the permutation events.
  ScoreSet<PAVOrderEvent>* m_pPermDistrib;

  //this holds mixed depend events which depend on heirs
  vector<ScoreSet<Dependency>*> m_depHeirDistribs;

};

#endif
