#include "HedgeTree.hpp"

#include "Hedge.hpp"
#include "HeadCompose.hpp"

// count the number of antecedents
unsigned int HedgeTree::number_of_children(const iterator& pHedge) {
  BSP<Inference> spInf = pHedge->getInference();
  vector<const Item*> antVec = spInf->getAnt();
  return antVec.size();
}

// count the number of antecedents
unsigned int HedgeTree::headPosn(const iterator& pHedge, arity_t dim) {
  return pHedge->headPosn(dim);
}
unsigned int HedgeTree::headPosn(const sibling_iterator& itemIter, arity_t dim) {
  return static_cast<const Hedge*>(*itemIter)->headPosn(dim);
}

const SynCat HedgeTree::pretermLabel(const iterator& pHedge, arity_t dim) {
  return pHedge->pretermLabel(dim);
}

HedgeTree::iterator HedgeTree::parent(const iterator&) {
  return NULL;
}

//! Judges whether the child node is the heir of the parent node.
bool HedgeTree::isHeir(const sibling_iterator& child, const iterator& parent) {
  return isHeir(static_cast<const Hedge*>(*child), parent);
}
bool HedgeTree::isHeir(const iterator& child, const iterator& parent) {
  BSP<Inference> spInf = parent->getInference();
  vector<const Item*> antVec = spInf->getAnt();
  size_t numAnts = antVec.size();
  unsigned short antNumber; // the child is which antecedent?
  bool found = false;
  for (size_t ant = 0; ant < numAnts; ++ant) {
    if (child == antVec[ant]) { // intentional pointer comparison
      antNumber = ant + 1; // +1 cuz heirRoles are 1-indexed
      found = true;
      break;
    }
  }
  if (! found)
    THROW("couldn't find child among antecedents");

  BSP<HeadCompose> spHC = dynamic_pointer_cast<HeadCompose>(spInf);
  if (!spHC)
    THROW("incompatible inference type");
  const HeirRoleVector& theHRV = spHC->heirIndices();
  arity_t sentinel = parent->arity();
  for (arity_t dim = 0; dim < sentinel; ++dim) {
    role_t theHeir = theHRV.getRole(dim);
    if (theHeir != antNumber)
      return false;
  }
  return true;
};

//! Judges whether the child node is the heir of the parent node.
bool HedgeTree::isHeir(const sibling_iterator& child, const iterator& parent, const arity_t dim) {
  return isHeir(static_cast<const Hedge*>(*child), parent, dim);
}
bool HedgeTree::isHeir(const iterator& child, const iterator& parent, const arity_t dim) {
  BSP<Inference> spInf = parent->getInference();
  vector<const Item*> antVec = spInf->getAnt();
  size_t numAnts = antVec.size();
  unsigned short antNumber; // the child is which antecedent?
  bool found = false;
  for (size_t ant = 0; ant < numAnts; ++ant) {
    if (child == antVec[ant]) { // intentional pointer comparison
      antNumber = ant + 1; // +1 cuz heirRoles are 1-indexed
      found = true;
      break;
    }
  }
  if (! found)
    THROW("couldn't find child among antecedents");

  BSP<HeadCompose> spHC = dynamic_pointer_cast<HeadCompose>(spInf);
  if (!spHC)
    THROW("incompatible inference type");
  const HeirRoleVector& theHRV = spHC->heirIndices();
  role_t theHeir = theHRV.getRole(dim);
  if (theHeir == antNumber)
    return true;
  
  return false;
};

HedgeTree::iterator HedgeTree::maximalProjection(const iterator&) {
  return NULL;
}

bool HedgeTree::is_valid(iterator& pHedge) {
  return (pHedge != NULL);
}

vector<const Item*>::iterator HedgeTree::begin(const iterator& pHedge) {
  return pHedge->getInference()->begin();
};
vector<const Item*>::iterator HedgeTree::end(const iterator& pHedge) {
  return pHedge->getInference()->end();
};

HedgeTree HedgeTree::dummyTree = HedgeTree();
