#ifndef HEDGETREE_H
#define HEDGETREE_H

#include <vector>
#include <boost/iterator/iterator_facade.hpp>
#include "GPDefs.hpp"
class Item;
class Hedge;
class HeadCompose;

/// Facade for treating the antecedent/consequent relationship among hedges as a tree
/// some of the naming conventions follow tree.hpp for now
class HedgeTree {
public:
  typedef const Hedge* iterator;
  typedef vector<const Item*>::iterator sibling_iterator;
  static HedgeTree dummyTree;

  // count the number of antecedents
  static unsigned int number_of_children(const iterator& pHedge);

  // count the number of antecedents
  static unsigned int headPosn(const iterator& pHedge, arity_t dim);

  static unsigned int headPosn(const sibling_iterator& itemIter, arity_t dim);

  static const SynCat pretermLabel(const iterator& pHedge, arity_t dim);

  static iterator parent(const iterator&);

  //! Judges whether a node pointed to by the input iterator
  //! is the heir of the parent.
  static bool isHeir(const sibling_iterator& child, const iterator& parent);
  static bool isHeir(const iterator& child, const iterator& parent);
  static bool isHeir(const sibling_iterator& child, const iterator& parent, const arity_t dim);
  static bool isHeir(const iterator& child, const iterator& parent, const arity_t dim);

  static iterator maximalProjection(const iterator&);

  static bool is_valid(iterator& pHedge);

  vector<const Item*>::iterator begin(const iterator& pHedge);
  vector<const Item*>::iterator end(const iterator& pHedge);
};

#endif
