#include "HedgeWithExtrema.hpp"

//! constructor
HedgeWithExtrema::
HedgeWithExtrema(const Hedge* pH, const Terminal& leftEx, const Terminal& rightEx) 
  : _super(*pH), m_leftEx(leftEx), m_rightEx(rightEx) 
{};

//! constructor
HedgeWithExtrema::
HedgeWithExtrema(const NTLink& label, const DSV& span, const Terminal& leftEx, const Terminal& rightEx)
  : _super(label, span), m_leftEx(leftEx), m_rightEx(rightEx)
{}

//copy constructor
HedgeWithExtrema::HedgeWithExtrema(const HedgeWithExtrema& other) 
  : _super(other), m_leftEx(other.m_leftEx), m_rightEx(other.m_rightEx)
{}

//! Assignment
HedgeWithExtrema& HedgeWithExtrema::operator = (const HedgeWithExtrema& hwe) {
  _super::operator=(hwe);
  m_leftEx = hwe.m_leftEx;
  m_rightEx = hwe.m_rightEx;
  return *this;
}

int HedgeWithExtrema::cmp(const Item* item, bool sameType) const {
  const HedgeWithExtrema* pHedge = dynamic_cast<const HedgeWithExtrema*>(item);
  int ret;
  // fanout vector
  if((ret = MC::cmp(getFanoutV(), pHedge->getFanoutV())) != 0) return ret;
  // constituent label
  NTLink label = pHedge->getNTLink();
  if((ret = m_label.cmp(label)) != 0) return ret;
  // d-span
  if((ret = m_span.cmp((DSV&)(*(pHedge->getDSV())))) != 0) return ret;

  return 0;
}

//! equality test
bool HedgeWithExtrema::operator == (const HedgeWithExtrema& other) const {
  return ( _super::operator==(other)
	   && m_leftEx == other.m_leftEx
	   && m_rightEx == other.m_rightEx);
}

//! Compares whether two items are equal in content and type.
//! Compared with operator==, this method is virtual.
bool HedgeWithExtrema::equal_to(const Item& other) const {
  const HedgeWithExtrema* pOther = dynamic_cast<const HedgeWithExtrema*>(&other);
  if(pOther){
    return operator==(*pOther);
  } else {
    return false;
  }
}

/// output
void HedgeWithExtrema::dump(ostream& o) const{
  _super::dump(o);
  o << " {";
  o << m_leftEx << " : " << m_rightEx;
  o << " }";
}
