/*
 * Class HedgeWithExtrema is an Hedge that stores its bounding
 * terminals in the output dimension
 */

#ifndef _HedgeWithExtrema_H_
#define _HedgeWithExtrema_H_

#include "Hedge.hpp"
#include <vector>

/**
 * Class HedgeWithExtrema is an Hedge item containing
 * an iterator to a tree node.
 */
class HedgeWithExtrema : public Hedge {
  typedef Hedge _super;
 public:

  //! constructors
  HedgeWithExtrema(const Hedge* pH, const Terminal& leftEx, const Terminal& rightEx);
  HedgeWithExtrema(const NTLink& labels, const DSV& spans, const Terminal& leftEx, const Terminal& rightEx);
  HedgeWithExtrema(const HedgeWithExtrema& nhr);
  HedgeWithExtrema();

  //! Destructor.
  virtual ~HedgeWithExtrema() {};

  // returns a copy of this
  virtual Item* clone() const {
    return new HedgeWithExtrema(*this);
  }

  //! Assignment
  HedgeWithExtrema& operator = (const HedgeWithExtrema& hedge);

  //! Equality test by comparing signature
  bool operator == (const HedgeWithExtrema& hedge) const;

  //! Compares whether two items are equal in content and type.
  //! Compared with operator==, this method is virtual.
  virtual bool equal_to(const Item& item) const;

  //! Output Adaptor.
  //  friend ostream& operator<<(ostream& o, const HedgeWithExtrema& h);
  virtual void dump(ostream& o) const;

  int cmp(const Item* item, bool sameType) const;

  const Terminal& getLeftEx() const {
    return m_leftEx;
  }

  const Terminal& getRightEx() const {
    return m_rightEx;
  }

private:
  
  Terminal m_leftEx, m_rightEx;
};

namespace HashNS {
  //! Template specialization of STL's hash object.
  template<> struct hash< HedgeWithExtrema >
  { size_t operator() (const HedgeWithExtrema& item) const {return item.hash();} };

  template<> struct hash< HedgeWithExtrema* >
  { size_t operator() (const HedgeWithExtrema* pitem) const {return pitem->hash();} };

  template<> struct hash< BSP<HedgeWithExtrema> >
  { size_t operator() (const BSP<HedgeWithExtrema> pitem) const {return pitem->hash();} };

  template<> struct hash< BSP<const HedgeWithExtrema> >
  { size_t operator() (const BSP<const HedgeWithExtrema> pitem) const {return pitem->hash();} };
}

#endif
