#include "HeirEvent.hpp"
#include "TLink.hpp"

HeirEvent:: HeirEvent(const FirstType& parent, const SecondType& heir)

  : _super(parent.size(), ImDom<Nonterminal*, Nonterminal*>(NULL, NULL))
{ setParent(parent); setHeirNTLink(heir); }

void HeirEvent::setHeirNTLink(const NTLink& heir) {
  if(this->size() != heir.arity())
    setDimity(heir.arity());

  for(arity_t d = 0; d < arity(); d++){
    if((*this)[d].child() != NULL)
      delete (*this)[d].child();
    (*this)[d].child() = heir.getNT(d)->clone();
  }
}

HeirEvent::FirstType HeirEvent::getParent() const {
  assert(arity());
  FirstType parent(arity());
  for(arity_t d = 0; d < arity(); d++) parent.setNT(d, (*this)[d].parent()->clone());
  return parent;
}

void HeirEvent::setParent(const FirstType& parent) {
  if(this->size() != parent.arity())
    setDimity(parent.arity());
  for(arity_t d = 0; d < arity(); d++) {
    if((*this)[d].parent() != NULL)
      delete (*this)[d].parent();
    (*this)[d].parent() = (parent.getNT(d))->clone();
  }
}

HeirEvent::FirstType HeirEvent::getHeir() const {
   assert(arity());
   FirstType heir(arity());
   for(arity_t d = 0; d < arity(); d++) {
     heir.setNT(d,(*this)[d].child()->clone());
   }
   return heir;
}

int HeirEvent::cmp(const HeirEvent& otherHeir) const {
  int ret;
  if((ret = getParent().cmp(otherHeir.getParent())) != 0) return ret;
  for(arity_t i = 0; i < arity(); i++){
    const Nonterminal* nt = otherHeir.getHeir().getNT(i);
    if((ret = (*this)[i].child()->cmp(nt) != 0)) return ret;
  }
  return 0;
}

//! Reads line by line.
istream& operator >> (istream& in, HeirEvent& heirEvent) {

  HeirEvent::FirstType parent;
  NTLink child;
  in >> parent;
  in >> child;

  heirEvent.setParent(parent);
  heirEvent.setHeirNTLink(child);

  return in;
}
