#ifndef _HeirEvent_H_
#define _HeirEvent_H_

#include <list>
#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "GPDefs.hpp"
#include "ImDom.hpp"
#include "Vector.hpp"
#include "NTLink.hpp"
#include <boost/serialization/serialization.hpp>

class NTLink;
class TLink;

//! An heir model event.
class HeirEvent :
  public Vector<ImDom<Nonterminal*, Nonterminal*> > {
public:
  typedef Vector<ImDom<Nonterminal*, Nonterminal*> > _super;
  typedef _super::const_iterator const_iterator;
  typedef _super::iterator iterator;
  typedef NTLink FirstType;
  typedef NTLink SecondType;
  //! Used for serialization.
  typedef Nonterminal* element_type;

  //! Empty constructor.
  HeirEvent() { }

  //! Constructor.
  HeirEvent(const arity_t& dim) : _super(dim) {
  }

  //! Constructor.
  HeirEvent(const HeirEvent& he) : _super(he) {
  }

  //! Constructor (from a Vector).
  HeirEvent(const _super& heirEventVector) : _super(heirEventVector) {}

  //! Constructor (from parent NTLink and heir NTLink)
  HeirEvent(const FirstType& parent, const SecondType& heir );

  virtual ~HeirEvent(){}

  arity_t arity() const { return (arity_t)size();}

  //! Sets language dimensionality.
  void setDimity(const arity_t& d) {
    resize(d);
  }

  //! Returns the parent of the lang'th dimension.
  Nonterminal* getParent1D(const arity_t& d ) const
  { return (*this)[d].parent(); }

  //! Returns the parent.
  FirstType getParent() const;

  //! Set the parent.
  void setParent(const FirstType& parent);

  //! Returns the heir.
  FirstType getHeir() const;

  //! Sets the heir to be the input NTLink.
  void setHeirNTLink(const NTLink& heir);

  //! Sets the lang'th heir to be the input nonterminal.
  void setHeirNTLink(const arity_t& d, const Nonterminal* nonterminal)
  { (*this)[d].child() = nonterminal->clone();}

  int cmp(const HeirEvent& other) const;

  bool operator<(const HeirEvent& other) const{
    return _super::operator<(other);
  }

  //! For hashing.
  size_t hash() const {
    size_t ret = 0;

    for(arity_t d = 0; d < arity(); ++d){
      ret *= 10;
      ret += (*this)[d].hash();
    }
    return ret;
  }

  //! Output.
  friend ostream& operator << (ostream& out, const HeirEvent& heirEvent)
  { out<< heirEvent.getParent()<<" "<<heirEvent.getHeir(); return out;}

  //! Reads line by line.
  friend istream& operator >> (istream& in, HeirEvent& heirEvent);

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super >(*this);
  };

};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
namespace HashNS {
  //! Template specialization of STL's hash object (a hash function) for
  //! HeirEvents.
  template<> struct hash<HeirEvent > {
    size_t operator()(const HeirEvent& heirEvent) const { return heirEvent.hash();}
  };
}

#endif
