#include <map>
#include <set>
#include "HeirRoleVector.hpp"
#include "RawHeirRoleVector.hpp"
#include "GPDefs.hpp"

using namespace std;

static const int SERIALIZE_DIM = 100000;
static const short HASH_VALUE = 17;

Vocabulary<RawHeirRoleVector> HeirRoleVector::sm_voc;

HeirRoleVector::HeirRoleVector() : m_index(sm_voc.addWord(_super())){}

 //! Constructor.
HeirRoleVector::HeirRoleVector(const arity_t& dim) 
  : m_index(sm_voc.addWord( _super(dim))) 
{}
HeirRoleVector::HeirRoleVector(const RawHeirRoleVector& raw) : 
  m_index(sm_voc.addWord(_super(raw)))
{}

//! Constructor.
HeirRoleVector::HeirRoleVector(const arity_t& dim, short s) 
  : m_index(sm_voc.addWord( _super(dim, s)))
{}

/// constructor from super
HeirRoleVector::HeirRoleVector(const Vector<short>& other) 
  : m_index(sm_voc.addWord( _super(other)))
{}

bool HeirRoleVector::empty() const {
  return sm_voc.getWord(m_index)->empty();
}

size_t HeirRoleVector::hash() const {
 return m_index;
}

HeirRoleVector HeirRoleVector::invert() const{
  RawHeirRoleVector raw = sm_voc.getWord(m_index)->invert();
  return HeirRoleVector(raw);
}

HeirRoleVector HeirRoleVector::get1DItem(arity_t d)const{
  RawHeirRoleVector raw = sm_voc.getWord(m_index)->get1DItem(d);
  return HeirRoleVector(raw);
}

ostream& operator<<(ostream& out, const HeirRoleVector& hv) {
  out << *(HeirRoleVector::sm_voc.getWord(hv.m_index));
  return out;
}

istream& operator>>(istream& in, HeirRoleVector& hv) {
  RawHeirRoleVector s;
  in >> s;
  hv = HeirRoleVector(s);
  return in;
}

ostream& HeirRoleVector::put(ostream& out) const{
  return  sm_voc.getWord(m_index)->put(out);
}

int HeirRoleVector::cmp(const HeirRoleVector& other) const {
  return MC::cmp((int)m_index, (int)(other.m_index));
}

//! Returns the set of roles in this PAV.
set<short> HeirRoleVector::roleSet() const {
  return  sm_voc.getWord(m_index)->roleSet();
}

/**
 * Return the maximum role. This HeirRoleVector do not have to be normalized.
 */
short HeirRoleVector::maxRole() const {
  return  sm_voc.getWord(m_index)->maxRole();
}

ostream& HeirRoleVector::serialize(ostream& out) const {
  return  sm_voc.getWord(m_index)->serialize(out);
}

void HeirRoleVector::setRole(arity_t a, short s){
  RawHeirRoleVector raw =  *sm_voc.getWord(m_index);
  raw[a] = s;
  *this = HeirRoleVector(raw);
}

short HeirRoleVector::getRole(arity_t a)const{
  return   ((*sm_voc.getWord(m_index))[a]);
}

//! (make sure first position is 1)
HeirRoleVector HeirRoleVector::normal() const{
  RawHeirRoleVector raw = sm_voc.getWord(m_index)->normal();
  return HeirRoleVector(raw);
}

//! returns true if this heir vector is head split.
bool HeirRoleVector::headSplit() const{
  return  sm_voc.getWord(m_index)->headSplit();
};

//!Normalized this HeirRoleVector
//! (make sure first position is 1)
//! N.B. this should ONLY be used when being made into an event.
//! Once in a distribution, we only care about normalized HVs, but
//! when an HV is in a production, we can't normalize it unless we
//! switch around all the other parts of the production to match.
void HeirRoleVector::normalize(){
  RawHeirRoleVector raw = *sm_voc.getWord(m_index);
  raw.normalize();
  HeirRoleVector x(raw);
  *this = x;
}

//! detects whether the HeirRoleVector is normalized.
bool HeirRoleVector::isNormal()const{
  return  sm_voc.getWord(m_index)->isNormal();
}

short HeirRoleVector::rank() const {
  return  sm_voc.getWord(m_index)->rank();
}

//! Returns true is there is any information about heirs.
bool HeirRoleVector::containsHeir() const{
  return  sm_voc.getWord(m_index)->containsHeir();
}

//! Return true if the role is in the heirVector
bool HeirRoleVector::contains(short val)const{
  return  sm_voc.getWord(m_index)->contains(val);
}

//We are passed in a Vector which is the RHS,and
// we export an Heir, given the this HeirRoleVector.
NTLink* HeirRoleVector::getHeirEvent(const Vector<ProdRHSElem>& d) const{
  return  sm_voc.getWord(m_index)->getHeirEvent(d);
}

//We are passed in a NTLink1 and 2.
// we export an HeirEvent, given the this is the HeirRoleVector.
NTLink* HeirRoleVector::getHeirEvent(const NTLink& link1, const NTLink& link2) const{
  return  sm_voc.getWord(m_index)->getHeirEvent(link1,link2);
}

//! We are given and NTLink and we create a dependency from it.
NTLink* HeirRoleVector::getDependency(const NTLink& ntl, short role) const{
  return  sm_voc.getWord(m_index)->getDependency(ntl,role);
}
