#ifndef HIDDENDSVS_H
#define HIDDENDSVS_H 1

#include "DSV.hpp"
#include "PAV.hpp"
#include "Vector.hpp"

namespace hDSV {

  /// If a grammar is being used as a supervising grammar for learning,
  /// then we want to update its consequents' hiddenDSVs structure.  If
  /// only one of the antecedents has hiddenDSVs, then those hiddenDSVs
  /// are copied to the consequent.  If both
  /// antecedents have hiddenDSVs, then some elements of their
  /// cross-product must be compatible with the input PAV.  Every such
  /// element is summed and copied to the hiddenDSV of the consequent.
  /// If no element is compatible with the PAV, then the antecedents
  /// cannot compose, so we abort.  @return true unless the input
  /// hiddenDSVs are incompatible with the PAV @param outHDSVs is the
  /// output hiddenDSVs, passed by reference so that the pointer can be
  /// changed
  bool crossProdWithPA (const Vector<DSV>* pHDSVs1, const Vector<DSV>* pHDSVs2, 
			const arity_t constrainedDim, const PrecArray& inPA, 
			Vector<DSV>*& outHDSVs);

  /// If a grammar is being used as a supervising grammar for
  /// learning, then we want to update its consequents' hiddenDSVs
  /// structure.  If only one of the antecedents has hiddenDSVs, then
  /// those hiddenDSVs are copied to the consequent.  If both
  /// antecedents have hiddenDSVs, then we take their cross-product.
  /// Every element of the cross-product is summed and copied to the
  /// hiddenDSV of the consequent.  @return true unless the result is
  /// the empty set.  @param outHDSVs is the output hiddenDSVs, passed
  /// by reference so that the pointer can be changed
  bool crossProd (const Vector<DSV>* pHDSVs1, const Vector<DSV>* pHDSVs2, 
		  Vector<DSV>*& outHDSVs);
}


#endif
