#include "IndexedPMTG.hpp"

#include <boost/program_options.hpp>

// added for serialization
IndexedPMTG::IndexedPMTG():_super() {};

/**
 * @param configFile The config file of the grammar.
 */
IndexedPMTG::IndexedPMTG(arity_t dm,
			 const string& configFile,
			 char** overrideArgV ,
			 size_t overrideArgC,
			 BSP<BPO::options_description> priorOptions )
  : _grammar(dm), _super(dm,configFile,overrideArgV,overrideArgC,priorOptions){};

void IndexedPMTG::estimate() {

  // assumed to be not in log space  
  m_prods.normalize();
  m_prodLHSScore.clear();
  iterator i;
  for(i = m_prods.begin(); i != m_prods.end(); i++){
    Production* pProd = i->first;
    SCORE joint = i->second;
    if (joint == 0.0)
      THROW("Zeroed-out prod: " << *pProd);
    m_prodLHSScore.increment(pProd->lhs(), joint);
  }

  for(i = m_prods.begin(); i != m_prods.end(); i++){
    Production* pProd = i->first;
    // the prob of a production's RHS is its joint prob divided by the prob of the LHS
    SCORE lhsCount = m_prodLHSScore.lookup(pProd->lhs());
    if (lhsCount == 0.0)
      THROW("Missing LHS: " << *pProd);
    m_prods[pProd] = i->second/lhsCount;
  }

  m_prods.logThis();
  m_prods *= -1;

  m_prodLHSScore.normalize();
  m_prodLHSScore.logThis();
  m_prodLHSScore *= -1;

  _super::estimate();
};

//! The cost associated with having this item as a root.
SCORE IndexedPMTG::rootCost(const Item* pI) const {
  const Hedge* pH = dynamic_cast<const Hedge*>(pI);
  if (not pH)
    THROW("Attempted to get root cost of a non-hedge.");

  const NTLink& lk = pH->getNTLink();
  SCORE ret;
  try {
    ret = m_prodLHSScore.lookup(lk);
  }catch(KeyNotFound<NTLink>){
    ret = 999999; // very high but not infinite
  }

  return ret;
};

//! input
istream& IndexedPMTG::get(istream& is){
  _super::get(is);
  is >> m_prodLHSScore;
//   m_prodLHSScore.clear();
//   for(iterator i = m_prods.begin(); i != m_prods.end(); i++){
//     Production* pProd = i->first;
//     SCORE negLog = i->second;
//     m_prodLHSScore.insert(pProd->lhs(), negLog);
//   }
  return is;
}

//! output
ostream& IndexedPMTG::put(ostream& os) const {
  _super::put(os);
  os << m_prodLHSScore;
  return os;
}
