
#include "ItemSet.hpp"

static const size_t ITEM_NUMBER_ESTIMATE = 10000;

// Constructor to reset the content.
//ItemSt::
//ItemSt(const arity_t& dim)
//: _super(dim) {
// m_closedList.resize(ITEM_NUMBER_ESTIMATE);
//}

// Clone a Chart of the same type as mine without copying the content.


// Destructor
//ItemSt::~ItemSt() {}

//! Begin iterator for the iteration over the items contained.

ItemSt::iterator
ItemSt::begin() {
  return m_closedList.begin();
}

//! End ITERATOR FOR THE ITERATION OVER THE ITEMS CONTAINED.

ItemSt::iterator
ItemSt::end() {
  return m_closedList.end();
}

//! Const begin iterator for the iteration over the items contained.

ItemSt::const_iterator
ItemSt::begin() const {
  return m_closedList.begin();
}

//! Const end iterator for the iteration over the items contained.

ItemSt::const_iterator
ItemSt::end() const {
  return m_closedList.end();
}

// Reset the content.

void ItemSt :: clear() {

  if (debug(6)){
    cerr << "Deleting closed list " << m_uniqueID 
	 << " of " << m_closedList.size() << " entries:";
    item_table_type::iterator itti;
    while(m_closedList.begin() != m_closedList.end()) {
      itti = m_closedList.begin();
      cerr <<  "deleting " << *itti;
      m_closedList.erase(*itti);
      cerr << "(done)";
    }
  }

  m_closedList.clear();
}

/// insertion into the closed list only, with no indexing (and so no matching)
void ItemSt::insertNoIndex(Item* pItem) {
  ItemPtr spItem( pItem);
  m_closedList.insert(spItem);
};

pair<SCORE, SCORE> ItemSt::getIOScores(const Item* item) const {
  pair<SCORE,SCORE> scorePair;

  item_table_type :: const_iterator iter;
  const ItemPtr key(item->clone());
  if((iter = m_closedList.find(key))!=m_closedList.end()){
    scorePair.first = (*iter)->getInsideScore();
    scorePair.second = (*iter)->getOutsideScore();
  }
  else {
    scorePair.first = scorePair.second = POSINF;
  }
  return scorePair;
}

bool ItemSt :: derived(const Item* item) const {
  item_table_type :: iterator iter;
  const ItemPtr key(item->clone());
  if((iter = m_closedList.find(key))!=m_closedList.end())
    return true;
  else
    return false;
}

void ItemSt :: printStats(ostream& out) const {
  if (debug(6)) {
    cerr << "Dumping closed list " << m_uniqueID << " of " << m_closedList.size() << " entries:" << endl;
    item_table_type::const_iterator ittci;
    for (ittci = begin(); ittci != end(); ++ ittci)
      cerr << *ittci;
  }
}

/// export all items in the chart
void ItemSt::exportAll(Vector<const Item*>& ret) const {
  for(iterator itItem = begin(); itItem != end(); ++itItem)
    ret.push_back( &(*itItem->get()));
};

