#ifndef _ItemSt_H_
#define _ItemSt_H_

#include "COMPILER-DEPENDENT_HEADERS.hpp"

#include <list>

#include "Chart.hpp"
#include "Hedge.hpp"
#include <ext/hash_set>
#include "MoreAlgorithms.hpp"

#ifdef BOOST_MEM_POOL
#include "BoostMemPool.hpp"
#endif

using namespace std;
using namespace HashNS;

//! ItemSt contains only Hedges
class ItemSt : public Chart {
protected:
  typedef Chart _super;

  typedef BSP<Item> ItemPtr;

  //! Type of the item table.
#ifdef BOOST_MEM_POOL
  typedef hash_set<ItemPtr,
                   HashNS::hash<ItemPtr>,
		   equal_to_ptr<ItemPtr>,
		   boost::decorated_fast_pool_allocator<ItemPtr> >
  item_table_type;
#else
  typedef hash_set<ItemPtr,
                   HashNS::hash<ItemPtr>,
		   equal_to_ptr<ItemPtr> > item_table_type;
#endif

  //! Iterator over the contained hedges.
  typedef item_table_type::iterator iterator;
  typedef item_table_type::const_iterator const_iterator;


public:


  //! Constructors
  ItemSt(const arity_t& dim, const arity_t& inputArity)
    : _super(dim, inputArity) {
    m_closedList.resize(10000);
  }
  ItemSt(const arity_t& dim)
    : _super(dim, dim) {
    m_closedList.resize(10000);
  }
 
  //! Destructor.
  virtual  ~ItemSt(){}

  //! Returns the number of item contained
  virtual  size_t size() const {return m_closedList.size();}

  /// insertion into the closed list only, with no indexing (and so no matching)
  void insertNoIndex(Item* pItem);

  //! Get the score of an item. Including both the inside and outside
  //! scores.
  virtual pair<SCORE, SCORE> getIOScores(const Item* item) const;

  //! Check whether a given Item is already on the chart.
  virtual bool derived(const Item* item) const;
 
  /// export all items in the chart
  virtual void exportAll(Vector<const Item*>& VpI) const;

  //! Clears the content.
  virtual void clear();

  //! Prints the statistics of this class.
  virtual void printStats(ostream& out) const;

protected: // methods
  /*
   * ITERATORS.
   */

  //! Begin iterator for the iteration over the items contained.
  iterator begin();

  //! End ITERATOR FOR THE ITERATION OVER THE ITEMS CONTAINED.
  iterator end();

  //! Const begin iterator for the iteration over the items contained.
  const_iterator begin() const;

  //! Const end iterator for the iteration over the items contained.
  const_iterator end() const;

protected: // data

  //! this is the set of all items in the chart, used for existence testing
  item_table_type m_closedList;

};

#endif

