#ifndef _LabelTerminationTest_H_
#define _LabelTerminationTest_H_

#include "TerminationTest.hpp"
#include "Hedge.hpp"
#include <typeinfo>

/// The parsing goal specifying a item label.
template<class Label = NTLink >
class LabelTerminationTest : public TerminationTest {
    typedef TerminationTest _super;
public:
    typedef _super :: HowAchieved HowAchieved;

    /// constructor.
    LabelTerminationTest(const Label& label) : m_label(label) {}

    /// Destructor.
    virtual ~LabelTerminationTest() {}

    /// add a subgl, returning a ID for this sub-goal.
    GoalID add(TerminationTest* subGoal) {
	// cannot add a subgoal because this is a primitive goal.
	THROW("LabelTerminationTest::add not implemented");
	return NONE;
    }

  /// returns true is this goal is achieved.
  /// \param item : the item just derived.
  /// \param which : which subgoal is reached. If no goal is
  /// reached, then NONE, otherwise, 0.
  bool achieved(const Item* i, GoalID& which) {
    bool goalMet = false;
    const Hedge* item;

    if(i){
      if((item = dynamic_cast<const Hedge*>(i))) {
	//assume the goal is met unless proven otherwise
	goalMet = true;
	Label lab =(item->getNTLink());
	for(unsigned int i = 0; i < lab.size() && goalMet; i++)
	  if(lab.getNT(i)->nt() != m_label.getNT(i)->nt()){
	    //non matching nt, so goal not met.
	    goalMet = false;
	  }
      }
    }

    if(goalMet){
      which = 0;
      setGoal(0);
      return true;
    }
    else{
      which = NONE;
      setGoal(NONE);
      return false;
    }

  }

    /// Checks whether the goal is reached. Used after
    /// the above method is called.
    bool achieved() const {
	if(anyAchieved()){
	    return true;
	} else {
	    return false;
	}
    }

    /// assignment.
    LabelTerminationTest& operator=(const LabelTerminationTest& other){
	_super::operator=(other);
	m_label = other.m_label;
	return *this;
    }

  virtual void redefine(NTLink label) {
    TerminationTest::redefine(label);
    m_label = label;
  }

private:
    Label m_label;
};

#endif

