/// namespace for methods that compute item labels
#include "LabelingStrategy.hpp"
#include "GlobalVocab.hpp"

namespace LabelingStrategy {

  // compute a new NT label by joining the two inputs with a '+', put
  // this new label into the NT vocab, and return its index.  order
  // the compound based on the relative order of the antecedent spans
  // that the two input NTs come from.
  VocabularyIndex BottomUpConcat1DHeaded(arity_t dim, VocabularyIndex vi1, VocabularyIndex vi2, 
					 role_t theRole, const PAV& thePAV,
					 role_t heirAntecedent) {
    bool exists;
    string label1 = *(GlobalVocab::s_synCatSet->getWord(vi1, exists));
    assert(exists);
    string label2 = *(GlobalVocab::s_synCatSet->getWord(vi2, exists));
    assert(exists);

    string newLabel;
    // if either is empty, we just use the other
    if (label1 == "NULL")
      newLabel = label2;
    else if (label2 == "NULL")
      newLabel = label1;
    else {
      // if it's already a compound, then bracket it
      if (label1.find('+') != string::npos)
	label1 = "<" + label1 + ">";

      // if it's already a compound, then bracket it
      if (label2.find('+') != string::npos)
	label2 = "<" + label2 + ">";

      // mark the head with an asterisk
      if (heirAntecedent == 1)
	label1 += "*";
      else if (heirAntecedent == 2)
	label2 += "*";
      else
	THROW("heirAntecedent screwed up");

      // order the compound based on the relative order of the antecedent spans
      if (theRole) // ant1 has role 1
	if (thePAV.monotonic(dim))
	  newLabel = label1 + "+" + label2;
	else
	  newLabel = label2 + "+" + label1;
      else  // ant1 has role 2
	if (thePAV.monotonic(dim))
	  newLabel = label2 + "+" + label1;
	else
	  newLabel = label1 + "+" + label2;
    };

    // get the NT encoding, updating the NT vocab if necessary
    VocabularyIndex newNT = GlobalVocab::s_synCatSet->addWord(newLabel);

    return newNT;
  };

  // compute a new NT label by joining the two inputs with a '+', put
  // this new label into the NT vocab, and return its index.  order
  // the compound based on the relative order of the antecedent spans
  // that the two input NTs come from.
  VocabularyIndex BottomUpConcat1D(arity_t dim, VocabularyIndex vi1, VocabularyIndex vi2, 
				   role_t theRole, const PAV& thePAV) {
    bool exists;
    string label1 = *(GlobalVocab::s_synCatSet->getWord(vi1, exists));
    assert(exists);
    string label2 = *(GlobalVocab::s_synCatSet->getWord(vi2, exists));
    assert(exists);

    string newLabel;
    // if either is empty, we just use the other, but bracket it
    // anyway to keep the bracket nesting level consistent in all
    // components
    if (label1 == "NULL")
      newLabel = "<" + label2 + ">";
    else if (label2 == "NULL")
      newLabel = "<" + label1 + ">";
    else {
      // bracket it
      //      if (label1.find('+') != string::npos)
      label1 = "<" + label1 + ">";

      // bracket it
      //      if (label2.find('+') != string::npos)
      label2 = "<" + label2 + ">";

      // order the compound based on the relative order of the antecedent spans
      if (theRole) // ant1 has role 1
	if (thePAV.monotonic(dim))
	  newLabel = label1 + "+" + label2;
	else
	  newLabel = label2 + "+" + label1;
      else  // ant1 has role 2
	if (thePAV.monotonic(dim))
	  newLabel = label2 + "+" + label1;
	else
	  newLabel = label1 + "+" + label2;
    };

    // get the NT encoding, updating the NT vocab if necessary
    VocabularyIndex newNT = GlobalVocab::s_synCatSet->addWord(newLabel);

    return newNT;
  };

  // overload for entire NTLinks
  NTLink BottomUpConcat(const NTLink& ntl1, const NTLink& ntl2, role_t theRole, const PAV& thePAV) {
    arity_t maxAr = ntl1.arity(); // assume both inputs have same arity
    vector<Nonterminal*> vpNT;
    for(arity_t ar = 0; ar < maxAr; ++ar) {
      VocabularyIndex vi1 = ntl1.getNT(ar)->getSynCat();
      VocabularyIndex vi2 = ntl2.getNT(ar)->getSynCat();
      VocabularyIndex vi = BottomUpConcat1D(ar, vi1, vi2, theRole, thePAV);
      Nonterminal* pNT = new Nonterminal(vi);
      vpNT.push_back(pNT);
    }
    NTLink ret(vpNT);

    return ret;
  };

};
