#ifndef LABELING_STRATEGY
#define LABELING_STRATEGY

#include "StringVocabulary.hpp"
#include "PAV.hpp"
#include "NTLink.hpp"

/// namespace for methods that compute item labels
/// For now, this is a namespace rather than a fullblown class, since it doesn't need separate instances
namespace LabelingStrategy {

  /// compute a new NT label by joining the two inputs with a '+', 
  /// put this new label into the NT vocab, and return its index.
  /// order the compound based on the relative order of the antecedent
  /// spans that the two input NTs come from.
  VocabularyIndex BottomUpConcat1D(arity_t dim, VocabularyIndex nt1, VocabularyIndex nt2, 
				   role_t theRole, const PAV& thePAV,
				   const role_t heirAntecedent);

  /// same as unheaded version, except head label gets marked in compounds
  VocabularyIndex BottomUpConcat1DHeaded(arity_t dim, VocabularyIndex nt1, VocabularyIndex nt2, 
					 role_t theRole, const PAV& thePAV,
					 const role_t heirAntecedent);

  /// overload for entire NTLinks
  NTLink BottomUpConcat(const NTLink& ntl1, const NTLink& ntl2, role_t theRole, const PAV& thePAV);

};

#endif
