#ifndef _LazyAgenda_H_
#define _LazyAgenda_H_

#include <queue> // for priority_queue
#include <set>

#include "Item.hpp"
#include "Agenda.hpp"

class Item;

using namespace std;

#ifdef BOOST_MEM_POOL
#include "BoostMemPool.hpp"
typedef vector<Item*, boost::decorated_fast_pool_allocator<Item*> >
			     sequence_type;
#else
typedef vector<Item*> sequence_type;
#endif

//! A normal agenda class, which may store duplicate derived items.
//! This class is suitable to both BottomUp and BestFirst parsing strategies.
template<class Comp>
class LazyAgenda : public priority_queue<Item *, sequence_type, Comp>
	     , public Agenda {
    typedef priority_queue<Item*, sequence_type, Comp> _super;
public:

  using priority_queue<Item *, sequence_type, Comp>::c;

  //! Constructor.
  LazyAgenda() : _super()
  {}

  //! Constructor.
  LazyAgenda(const Comp& comp) : _super(comp)
  {}

  //! Destructor.
  ~LazyAgenda() {
       clear();
      // inference objects are not released.
      // this doesn't matter since we are using
      // the memory pool, which will be cleared
      // at the end of the parsing of a sentence tuple.
  }

  bool empty() const {
      return _super::empty();
  }

  //! Clears the content of the LazyAgenda.
  void clear(){
    for(size_t i = 0; i < c.size(); ++i){
      delete c[i];
    }

    // c is the sequence in base class.
    c.clear();
  }

  //! Dumps the content for diagnostics.
  void dump(ostream& out) const {
    out<<"BEGIN DUMP AGENDA:\n";
    for(unsigned int i = 0; i < size(); i++){
      //c[i]->getConseq()->dump(out); out<<endl;
      c[i]->dump(cout); cout<<endl;
    }
    out<<"END DUMP AGENDA\n\n";
  }

  void push(Item* const& item){
      _super :: push(item);
  }

  Item* const& top() const {
      return _super::top();
  }

  void pop() {
      _super::pop();
  }

  size_t size() const {
      return _super::size();
  }

  //! Returns true if the input item has been in this agenda.
  bool exists(const Item&) const
  { return false;}

  void dump_f_costs(ostream& o) const {
      sequence_type::const_iterator it;
      for(it = c.begin(); it != c.end(); ++it){
	  o<<"FCOST: "<<(*it)->getScore()<<endl;
      }

  }

};

#endif
