#include "LearnableGrammar.hpp"

//! clear the content
void LearnableGrammar::clear() {
//   _super::clear();
//   delete m_pSentTuple;
//   m_pSentTuple = 0;
};

bool LearnableGrammar::prepareFor(SentenceTuple& tuple) {
  m_pSentTuple = &tuple;
  return true;
};

bool LearnableGrammar::prepareFor(const Vector<HeadMTree*>& treeVec, vector<unsigned int>& startingPosns) {
  // first, ensure that the strings are available
  if (! m_pSentTuple) {
    THROW("No sentenceTuple set" << endl;);
  }

  const Vector<HeadMTree*>::const_iterator sentinel = treeVec.end();
  Vector<HeadMTree*>::const_iterator vtci = treeVec.begin();
  for(; vtci != sentinel; ++vtci) {
    (*vtci)->appendSpans(startingPosns);

    bool ret = prepareFor(*vtci);
    if (not ret)
      return false;
  }
  return true;
};

bool LearnableGrammar::prepareFor(HeadMTree* pTree) {
  m_pTree = pTree;
  return true;
};

//! Decompose a multitree into inferences in the way specified by the grammar.
void LearnableGrammar::tree2infs(Vector<Inference*>& ret, HeadMTree& theTree) {
  // do NOT clear ret, in case the caller wants to loop over a bunch of trees
  //  ret.clear();

  // first, build a map of nodes to DSVs
  hash_map<size_t, DSV> it2dsv;
  HeadMTree::post_order_iterator postIt = theTree.begin_post();
  HeadMTree::post_order_iterator sentinel = theTree.end_post();
  unsigned fanout, nodeID;
  DSV theDSV;
  while (postIt != sentinel) {
    fanout = postIt.number_of_children();
    if (fanout == 0) {
      // the leaves have span info already, so just grab it
      if(!boost::get<const Pair<TLink,DSV> >(&BGET_EITHER_LABEL(*postIt))){
	THROW("Expected Spans to be read in for tree2infs. No Spans found");
      }
      theDSV = BGET_TERM_SPAN_LABEL(*postIt).second;
    } else {
      // compute each node's DSV by summing over its children
      // post-order iteration assures that childrens' DSVs are available when needed
      HeadMTree::sibling_iterator sit = theTree.begin(postIt);
      theDSV = it2dsv[sit->getUniqueID()];
      ++sit;
      while (sit != theTree.end(postIt)) {
	theDSV += it2dsv[sit->getUniqueID()];
	++sit;
      }
    }
    nodeID = postIt->getUniqueID();
    it2dsv[nodeID] = theDSV;
    if (debug(5))
      cerr << "LG::tree2infs mapping node " << nodeID << " to DSV " << theDSV << endl;
    ++postIt;
  }

  // next, traverse the tree and get inference from each node
  static DSV emptyDSV;
  for(postIt = theTree.begin_post(); postIt != sentinel; ++postIt) {
    short fanout = postIt.number_of_children();
    if (fanout == 0) {
      // do nothing
    } else if (fanout == 1) {
      treeNode2infs( ret, postIt, it2dsv[postIt->getUniqueID()], emptyDSV, emptyDSV);
    } else if (fanout == 2) {
      HeadMTree::sibling_iterator sit = theTree.begin(postIt);
      nodeID = sit->getUniqueID();
      DSV ant1DSV = it2dsv[nodeID];
      if (debug(5))
	cerr << "LG calling treeNode2infs with antNode1 " << nodeID << " with DSV " << ant1DSV << " " << *sit << endl;
      ++sit;
      nodeID = sit->getUniqueID();
      DSV ant2DSV = it2dsv[nodeID];
      if (debug(5))
	cerr << " antNode2 " << nodeID << " with DSV " << ant2DSV << " " << *sit << endl;
      treeNode2infs( ret, postIt, it2dsv[postIt->getUniqueID()], ant1DSV, ant2DSV);
    }
  }
  
}
