#include "LengthBasedEstimator.hpp"
#include "Item.hpp"
#include "DSV.hpp"

//! Precompute the maximum log probability of any production.
void LengthBasedEstimator::precompute(NaiveWMTG *grammar) {
  assert(grammar != NULL);
  SCORE min = POSINF;
  for(NaiveWMTG::iterator itr = grammar->begin(); itr!=grammar->end(); itr++) {
	SCORE score = itr->second;
	if(score < min)
      min = score;
  }

  m_pMinProductionProb = min;
}

//! returns the outside cost estimate of the input item
//! based on the context of the input sntT.
SCORE LengthBasedEstimator::operator()(SentenceTuple &sent, const Item* item) {

  // maximum number of words remaining on any dimension.
  size_t maxWdRemaining = 0;

  for(arity_t d = 0; d < sent.arity(); ++d) {
    size_t temp = sent[0].size() - item->getDSV()->length(d);
    if (temp > maxWdRemaining)
      maxWdRemaining = temp;
  }

  //cout << "prob " << m_pMinProductionProb << " " 
       //<< maxWdRemaining * m_pMinProductionProb << endl;
  return maxWdRemaining * m_pMinProductionProb;
}

//! Read the data in. The data is read from the first file, therefore,
//! we only need to specify the first file.
istream& LengthBasedEstimator::get(istream& in) {
    in >> m_pMinProductionProb;
    return in;
}

//! output the data.
ostream& LengthBasedEstimator::put(ostream& out) const {
    out << m_pMinProductionProb;
    return out;
}

