#ifndef _LengthBasedEstimator_H_
#define _LengthBasedEstimator_H_

#include "OutsideCostEstimator.hpp"
#include "SentenceTuple.hpp"
#include "IndexedWMTG.hpp"

//! Length-based outside cost estimate.
/*!
 * This computes the outside cost estimate only based on the minimum cost
 * item in the entire grammar and the remaining number of words to parse.
 */
class LengthBasedEstimator : public OutsideCostEstimator {
  typedef OutsideCostEstimator _super;
public:

  //! Constructor.
  LengthBasedEstimator(const string& configFile) 
    : _super(configFile), m_pMinProductionProb(0.0) {}

  //! Destructor.
  virtual ~LengthBasedEstimator() {}

  //! Compute the maximum value over all productions.
  void precompute(NaiveWMTG *grammar);

  //! Returns the reverse value.
  virtual SCORE operator()(SentenceTuple&, const Item*);

  //! Read the data in.
  istream& get(istream&);

  //! Write the data out.
  ostream& put(ostream&) const;
protected:

  //! The minimum prob of nonterminal productions.
  SCORE m_pMinProductionProb;
};

#endif

