#include "LexNT.hpp"
#include "GlobalVocab.hpp"
#include "Exceptions.hpp"

//! Default constructor.
LexNT::LexNT() {
}

//! Constructor from the NT and its head.
LexNT::LexNT(const SynCat& a, const Terminal& b) :
  Nonterminal(a){
  m_terminal = b;
}

//! output with vocab deciphering
ostream& LexNT::put(ostream& out, StringVocabulary* ntset,
		    StringVocabulary* terminalset, bool isPositionTree) const {

  /*nonterminal*/
  bool exists;
  const string* nt = ntset->baseVocabulary().getWord((*this).nt(), exists);
  if(!exists){
    THROW((*this).nt() << " cannot be found!" << endl);
  }
  out << " { ";
  out<< *nt <<" [ ";

  if(isPositionTree)
    out << this->lex() << " ] ";
  else{
    /*terminal*/
    const string* term = terminalset->getWord((*this).lex(), exists);
    if(!exists){
      THROW( (*this).lex() << " cannot be found!" << endl );
    }
    out<< *term <<" ] ";
  }
  out << " }";
  return out;
}

Terminal& LexNT::lex() { return m_terminal;}

Terminal LexNT::lex() const { return m_terminal;}

bool LexNT::isInactive() const{
  return (Nonterminal::isInactive()
	  && ((lex() == NULLT)||(lex() == UNDEF_T))) ;
}

//! Convert a LexNT to a VocabularyIndex via bit manipulation.
VocabularyIndex LexNT::toID() const {
  static const short TAGSHIFT = 18;
  return (nt() << TAGSHIFT )| (lex() & ((1<<TAGSHIFT)-1));
}

//! To compare with others like me, use the implementer class.
bool LexNT::operator!=(const LexNT& other) const{
  return m_synCat != (other.nt()) || m_terminal != (other.lex());
}

size_t LexNT::hash() const {
  return nt() * sm_bigValForHash + lex();
}

////////////////////////////Operators from Pair//////////////////

LexNT& LexNT::operator= (const LexNT& other) {
  m_terminal = other.lex();
this->setSynCat(other.nt());
  return *this;
}

bool LexNT::operator<(const LexNT& other) const {
  pair<SynCat, Terminal> pe(nt(), lex());
 pair<SynCat, Terminal> peOther(other.nt(), other.lex());
  return pe < (peOther);
}

bool LexNT::operator > (const LexNT& other) const {
  pair<SynCat, Terminal> pe(nt(), lex());
  pair<SynCat, Terminal> peOther(other.nt(), other.lex());
  return (!(pe < (peOther)) && !(pe == peOther));
}

int LexNT::cmp(const Nonterminal& other) const{

  int ret;
  if((ret = nt() - other.nt()) != 0){
      return ret;
  }

  const LexNT& castOther = static_cast<const LexNT&>(other);

  ret = lex() - castOther.lex();

  return ret;
}

int LexNT::cmp(const Nonterminal* other) const{
  return cmp(*other);
}

//! return a delexicalized version of this NTLink
Nonterminal* LexNT::delex() const{
  Nonterminal* ret = new LexNT(*this);
  static_cast<LexNT*>(ret)->m_terminal = 0;
  return ret;
}

//! Output.
ostream& LexNT::put(ostream& out) const
{ out<<"{" << nt()<<"["<<lex()<<"]" << "}"; return out; }

//! serialize.
ostream& LexNT::serialize(ostream& out) const
{ out<<nt()<<PAIRDELIM<<lex(); return out; }

//! Input.
istream& LexNT::get(istream& in) {
  if(GlobalVocab::s_noVocabsOut){
    return get(in, *GlobalVocab::s_synCatSet, *GlobalVocab::s_terminalSet);
  }
  else{
    char ch; // used to skip brackets
    in >>ch >> this->nt()>>ch>>this->lex()>>ch >> ch;
  }
  return in;

}

//! Input with a particular string vocab that's passed in.
istream& LexNT::get(istream& in, StringVocabulary& ntVocab, StringVocabulary& termVocab, bool isPositionTree) {
  char ch; // used to skip brackets
  char ch1;
  string strnt;
  string strt;
  Terminal t;
  in >>ch >> strnt >>ch1;
  SynCat nt = ntVocab.addWord(strnt);
  // if we are dealing with "position trees" we dont
  // deintegerize the terminals.
  if(isPositionTree)
    in >> t >> ch >> ch;
  else{
    in >> strt >> ch >> ch;
    t = termVocab.addWord(strt);
  }
  this->nt() = nt;
  this->lex() = t;

  return in;
 }

//! Exports the object content using a vector.
//! The order is SynCat, and then Terminal.
vector<unsigned int> LexNT::sequentialize() const {
  vector<unsigned int> ret(2);
  ret[0] = nt();
  ret[1] = lex();
  return ret;
}

vector<unsigned short> LexNT::vectorDecompose()const{
  vector<unsigned short> ret;
  ret.push_back(m_synCat);
  ret.push_back(m_terminal);
  return ret;
}

void LexNT::setNull(){
  m_synCat = NULLNT;
  m_terminal = NULLT;
}

bool LexNT::isDummy()const{
  return (m_synCat == RootTag && m_terminal == EOSH);
}

bool LexNT::isNull()const{
  return (m_synCat == NULLT && m_terminal == NULLNT);
}
