#ifndef _LinearPrecedenceComparator_H_
#define _LinearPrecedenceComparator_H_

#include "Comparator.hpp"
#include "DSV.hpp"
#include "Hedge.hpp"

using namespace std;

//! This is a comparator for agenda items that lets us use
//! agenda-based parsing as if it were BottomUp parsing.
class LinearPrecedenceComparator : public Comparator{
public:

  LinearPrecedenceComparator() {}
  virtual ~LinearPrecedenceComparator() {};

  virtual bool operator()(const Item* p1,
			  const Item* p2) const {

    //First compare ordering dimension-wise.
    for(arity_t d = 0; d < p1->arity(); ++d){

      DSV::Direction di = p1->getDSV()->to(*(p2->getDSV()),d);
      if(di == DSpan::LEFT) {
	return false;
      }
      else if (di == DSpan::RIGHT){
	return true;
      }
      if (di == DSpan::OVERLAP) {
	THROW("exportMaxCover botched: " << endl
	      << "Item 1: " << *static_cast<const Hedge*>(p1) << endl
	      << "Item 2: " << *static_cast<const Hedge*>(p2) << endl);
      }
      if(di == DSpan::EQUAL){
	if(p1 == p2){
	  return false;
	}
	THROW("EQUAL: " << endl
	      << "Item 1: " << *static_cast<const Hedge*>(p1) << endl
	      << "Item 2: " << *static_cast<const Hedge*>(p2) << endl);
      }
      assert(di == DSpan::UNDEF);
    }
     
    return false; // to avoid compiler warning
  }

};

#endif
