#include "Exceptions.hpp"
#include "LogicBuilder.hpp"

#include "GPDefs.hpp"
#include "Path.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"

// #include "LogicA.hpp"
// #include "LogicB.hpp"
#include "BottomUpLogic.hpp"
#include "BottomUpTranslationLogic.hpp"
#include "FasterBottomUpTranslationLogic.hpp"
// #include "CKYTranslator.hpp"
// #include "LogicD.hpp"
// #include "LogicE.hpp"
// #include "LogicF.hpp"

LogicBuilder* LogicBuilder::_instance = 0;

LogicBuilder* LogicBuilder::Instance(){
  if(_instance == 0){
    _instance = new LogicBuilder;
  }
  return _instance;
}

//! build logic
Logic* LogicBuilder::build(const string& configFile) {
  // the variables defined in the config file.
  string strLogicType;
  string optLogicConfigFile;

  // compile the config file.
  BPO::options_description config("Configuration");
  config.add_options()
    ("LogicType", BPO::value<string>(&strLogicType), "The Logic type.")
    ("LogicConfigFile", BPO::value<string>(&optLogicConfigFile), "The Logic config file.");
  BPO::variables_map vm;
  cerr << "LogicBuilder: Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
    cerr << "Option values of \"LogicBuilder\"" << endl << endl;
    cerr << OptionsPrinter(config, vm) << endl;
  }
  if(strLogicType == "") {
    THROW("Error: the type of Logic is not specified!"<<endl);
  }

  char logicType = strLogicType[0];
  string logicConfigFile = Path::maybe_compose(configFile, optLogicConfigFile);
  Logic* pLogic;

  switch(logicType){
  case 'C':
    pLogic = new BottomUpLogic(logicConfigFile.c_str());
    break;
  case 'T':
    pLogic = new BottomUpTranslationLogic(logicConfigFile.c_str());
    break;
  case 'F':
    pLogic = new FasterBottomUpTranslationLogic(logicConfigFile.c_str());
    break;
//   case 'K':
//     pLogic = new CKYTranslator(logicConfigFile.c_str());
//     break;
  default:
    THROW("Error: unknown Logic type!"<<endl);
  }

  return pLogic;
}

