#include "LongestSpanComparator.hpp"

// just use default constructor
// LongestSpanComparator::LongestSpanComparator(): {}

//! Compares two Items, based on the length of their spans, retrieved from the Chart
//! \returns true if the span of item b is greater than the span of item a, meaning that b should be preferred over a

bool LongestSpanComparator::operator()(const Item* a,
				     const Item* b) const {
  int ret;

  
  FanoutVect fo1 = a->getFanoutV();  
  FanoutVect fo2 = b->getFanoutV();
  
  for(unsigned int i = 0; i < fo1.size(); i++){
    if(fo1[i] != fo2[i]){
      return (fo1[i] > fo2[i]);

    }
  }
  

  //get the sizes (span length) of the derived items
  size_t volA = a->volume();
  size_t volB = b->volume();
  if(volA != volB) {
    return (volA < volB); //
  }

  SCORE aH = a->getScore();
  SCORE bH = b->getScore();

  // if the f-cost of items a and b are significantly different
  if(fabs(aH - bH) > TINYSCORE)
		// the smaller score (cost) is the better one
    return (aH > bH);

  //test number of non null dims.
  unsigned short nndA = a->numActiveDims();
  unsigned short nndB = b->numActiveDims();

  if(nndA != nndB) { // if different
    return (nndA < nndB);
  }

  // return the younger item, i.e. the one with higher BatchID
  unsigned int aID = a->getBatchID();
  unsigned int bID = b->getBatchID();

  if(aID != bID)
    return (aID < bID);

  // compare inferences
  Inference* infOther = b->getInference().get();
  Inference* infThis = a->getInference().get();

  if ((ret = infThis->cmp(infOther)) != 0)
    return ret;

  //last, use comparator on item itself.
  int x = b->cmp(a) ;

  if(x < 0)
    return true;
  if(x > 0)
    return false;

  // return the younger item, i.e. the one with higher uniqueID
  aID = a->getUniqueID();
  bID = b->getUniqueID();

  if(aID != bID)
    return (aID < bID);

  cerr << "identical items being compared."
       << endl;
  a->dump(cerr);
  cerr << endl;
  b->dump(cerr);
  cerr << endl;

  assert(false);

  return false; // to avoid compiler warning

};
