#include "MTree.hpp"

arity_t MTree::s_arity = 2;

//MTree output
ostream& operator<<(ostream& out, const MTree& mTree) {
  const MTree* pT = addressof(mTree);
  pT->print(out, pT, mTree.begin());
  return out;
}
ostream& operator<<(ostream& out, MTree& mTree) {
  const MTree* pT = addressof(mTree);
  pT->print(out, pT, mTree.begin());
  return out;
}

// //! The Charniak dummy begin in the multitree
// //! is the first kid of the multitree begin.
// void MTree::removeCharniakDummyRoot( ) {
//   set<iterator> itsets;
//   iterator beginIt = begin();
//   sibling_iterator sib_i = beginIt.begin();
//   for(; sib_i != beginIt.end(); ++sib_i){
//     itsets.insert(sib_i);
//   }
//   if(itsets.size() >= 2){
//     iterator from = *itsets.begin();
//     iterator to = *itsets.end();
//     reparent(begin(), from.begin(), to.end());
//     erase(*itsets.begin());
//   }
// }


//! Copy the nonterminal label from the first dim
//! to the remaining dims in the same NTLink.
void MTree::copySynCats( ) {
  iterator i;
  for(i = begin(); i != end(); ++i){
    arity_t d;
    for(d = 1; d < BGET_NT_LABEL(*i).arity(); ++d){
      Nonterminal* nt = BGET_NT_LABEL(*i).getNT(d)->clone();
      nt->nt() = (BGET_NT_LABEL(*i).getNT(0))->nt();
      BGET_NT_LABEL(*i).setNT(d,nt);
    }
  }
}


//! Remove the dummy begin and EOS node in a parse tree of our
//! own format.
void MTree::removeDummyNodes( ) {
  /*delete the dummy end-of-sentence node*/
  for(iterator mti = begin(); mti != end(); ++mti){
    /*checks the label of the node*/
    if(boost::get<NTLink>(&BGET_EITHER_LABEL(*mti))){
      NTLink lab = BGET_NT_LABEL(*mti);
      if(lab.isDummy()){
	erase(mti);
	break;
      }
    }
  }

}

//! Naturalizes a parse tree in the way of removing
//! dummy nodes (e.g., EOSH or BEGIN), and of organizing
//! siblings in linear order.
void MTree::naturalize(StringVocabulary& synCatVcb) {
  THROW("MTree::naturalize not implemented");

  // /*remove dummy nodes*/
  // removeDummyNodes(parse);

  // /*[A]+B --> A->B*/
  // extendUnary(parse, parse.begin(), synCatVcb);

  // /*order the siblings*/
  // MTree::pre_order_iterator i;
  // for(i = parse.begin(); i != parse.end(); ++i){
  // 	parse.sort(i.begin(), i.end());
  // }

}


SentenceTuple MTree::yield(iterator it){
  PAV pav= BGET_PAV(*it);
  
  NTLink lnt = BGET_NT_LABEL(*it);

  SentenceTuple ret;
  if(boost::get<TLink>(&BGET_EITHER_LABEL(*it))){
    TLink lnt = BGET_TERM_LABEL(*it);
    ret = SentenceTuple(lnt.arity());
  }
  if(boost::get<Pair<TLink,DSV> >(&BGET_EITHER_LABEL(*it))){
    TLink lnt = BGET_TERM_SPAN_LABEL(*it).first;
    ret = SentenceTuple(lnt.arity());
  }
  if(boost::get<NTLink>(&BGET_EITHER_LABEL(*it))){
     NTLink lnt = BGET_NT_LABEL(*it);
    ret = SentenceTuple(lnt.arity());
  }

  for(arity_t d = 0; d < ret.size(); d++){
    Vector<Terminal> os;
    MTree::yield_helper(os, it, 1, d);
    ret[d] = os;
  }
  return ret;

}

//puts the leaves in a sentencetuple and returns them in
// linear order in each dimension.
SentenceTuple MTree::yield( ) {
  iterator it = begin();
  return MTree::yield(it);
}

//Segment is the continuos segment number you are adding to the os here.
void MTree::yield_helper(Vector<Terminal>& os, iterator it, unsigned int numSegs, arity_t d) {
  MTree tr;
  if(!(tr.is_valid(it)))
    return;

  //If it's an external node..
  if(boost::get<TLink>(&BGET_EITHER_LABEL(*it))){
    TLink lab = BGET_TERM_LABEL(*it);

    //a leaf cannot have multiple discontinuities in one dimension.
    assert(numSegs == 1);
    if(lab[d] != NULLT && lab[d] != EOSH)
      os.push_back(lab[d]);
    return;
  }
  else if(boost::get<Pair<TLink,DSV> >(&BGET_EITHER_LABEL(*it))){
    TLink lab = BGET_TERM_SPAN_LABEL(*it).first;

    //a leaf cannot have multiple discontinuities in one dimension.
    assert(numSegs == 1);
    if(lab[d] != NULLT && lab[d] != EOSH)
      os.push_back(lab[d]);
    return;
  }
  //else, it's an internal node.
  else{

    PAV pav = BGET_PAV(*it);
    //this vector stores the children in order.
    vector<iterator> mtVector;
    for(MTree::sibling_iterator k = it.begin(); k != it.end(); ++k){
      mtVector.push_back(k);
    }

    //counts the number of times an iterator has been recursed on.
    //(used for discontinuities)
    Vector<unsigned int> cntVector(mtVector.size(), 0);
    unsigned int numSegments = 1;

    const PrecArray& pa = pav.getPA(d);
 
    for(unsigned int i = 0; i < pa.size(); i++){
      if(pa[i] == 0){
	numSegments++;
	continue;
      }
      //only recurse on the segment we are interested in.
      if(numSegments == numSegs){
	yield_helper(os, mtVector[pa[i]-1], ++cntVector[pa[i]-1], d);
	
      }else{
	++cntVector[pa[i]-1];
      }
    }
  }
}

/** Returns true if the node yields a base phrase */
bool MTree::isBasePhrase(iterator& it) const {
  for(MTree::sibling_iterator it_sib = it.begin();
      it_sib != it.end(); ++it_sib){
    if(it_sib.number_of_children()){
      return false;
    }
  }
  return true;
}

/*
 * Reintegerize the NTs in the multitree based on new
 * NT Vocabs. NTs will be added into the new NT
 * Vocabs if they don't occur in the new ones.
 *
 * @param tr The multitree.
 * @param newNtVcb The new nonterminal vocabs, each per language.
 * @param oldNtVcb The old nonterminal vocabs, each per language.
 * @param newTvcb The new terminal vocabs, each per language.
 * @param oldTvcb The old terminal vocabs, each per language.
 */
void MTree::reintNTs(vector<StringVocabulary*> newNtVcb,
			 vector<StringVocabulary*> oldNtVcb,
			 vector<StringVocabulary*> newTVcb,
			 vector<StringVocabulary*> oldTVcb) {
  arity_t d;
  bool exists;
  VocabularyIndex index;
  iterator it;

  for(it = begin(); it != end(); ++it){
    NTLink& l = BGET_NT_LABEL(*it);
    for(d = 0; d < l.arity(); ++d){
      if(newNtVcb.size() > d && newNtVcb[d] &&
	 oldNtVcb.size() > d && oldNtVcb[d]){

	const string* word = oldNtVcb[d]->getWord(l.getNT(d)->nt(), exists);
	if(!exists){
	  cerr<<l.getNT(d)->nt()<<" "<<(int)d<<endl;
	}
	assert(exists);

	index = newNtVcb[d]->addWord(*word);
	Nonterminal* nt = l.getNT(d)->clone();
	nt->nt() = SynCat(index);
	l.setNT(d,nt);
      }

      if(dynamic_cast<const LexNT*>(l.getNT(d))){
	// re-integerize the terminals.
	if(newTVcb.size() > d && newTVcb[d] &&
	   oldTVcb.size() > d && oldTVcb[d]){
	  const string* word = oldTVcb[d]->getWord((static_cast<const LexNT*>(l.getNT(d)))->lex(), exists);
	  assert(exists);

	  index = newTVcb[d]->addWord(*word);
	  LexNT* nt = static_cast<LexNT*>(l.getNT(d)->clone());
	  nt->lex()  = Terminal(index);
	  l.setNT(d,nt);	  
	}
      }
    }
  }
}

void MTree::getLeftIters(iterator& tr, arity_t& d, vector<iterator>& ret){

  vector<iterator> vec;
  iterator it;
  unsigned int i = 0;

  //base case:
  if(tr.begin() == tr.end()){
    if(BGET_NT_LABEL(*tr).getNT(d)->isDummy()){
      ret.push_back(tr);
    }
    return;
  }

  for(it = tr.begin(); it != tr.end(); ++it){
    vec[i] = it;
    i++;
  }

  PAV pav = BGET_PAV(*tr);
  const PrecArray& pa = pav.getPA(d);
  for(i = 0; i < pa.size(); i++){
    getLeftIters((vec[pa[i-1]]), d, ret);
  }

  return;
}


void MTree::delexTree( ) {
  iterator it = begin();
  delexTree_helper(it);
}

void MTree::delexTree_helper(iterator it) {
  //if it's a leaf node.
  if ( boost::get<TLink>( &BGET_EITHER_LABEL(*it) ) ){
    return;
  }
  // if it's not a leaf node.
  else{
    MTree tr;
    for(unsigned int i = 0; i < BGET_NT_LABEL(*it).size(); i++){
      Nonterminal* nt = new Nonterminal(BGET_NT_LABEL(*it).getNT(i)->nt());
      BGET_NT_LABEL(*it).setNT(i,nt);
    }

    MTree::sibling_iterator sib_it;

    if(tr.is_valid(it) && it != tr.end()){
      for(sib_it = it.begin(); sib_it != it.end(); ++sib_it)
	delexTree_helper(sib_it);
    }

    return ;
  }
}

void MTree::appendSpans(vector<unsigned int>& startingPosition) {
  iterator it = begin();
  NTLink lab = BGET_NT_LABEL(*it);
  for(arity_t i = 0; i < lab.arity(); i++){
    appendSpansHelper(it, (arity_t)i, startingPosition[i], 1);
    it = begin();
  }
}

void MTree::appendSpansHelper(iterator& it, arity_t dim, unsigned int& position, int numSegs) {
  if(!(is_valid(it)))
    return;

  //If it's a leaf
  if(boost::get<TLink>(&BGET_EITHER_LABEL(*it))){
    TLink lab = BGET_TERM_LABEL(*it);
    DSV dsv(lab.arity());
    dsv[dim] = DSpan( DSpan::Span(position, position+1));

    Pair<TLink,DSV> p(lab,dsv);
    (BGET_EITHER_LABEL(*it)) = p;

    //a leaf cannot have multiple discontinuities in one dimension.
    assert(numSegs == 1);

    if(lab[dim] != NULLT && lab[dim] != EOSH)
      position++;
  }
  //else, it's an internal node.
  else{
    //this vector stores the children in order.
    vector<iterator> mtVector;
    for(sibling_iterator k = it.begin(); k != it.end(); ++k){
      mtVector.push_back(k);
    }

    //counts the number of times an iterator has been recursed on.
    //(used for discontinuities)
    vector<unsigned int> cntVector(mtVector.size(), 0);
    int numSegments = 1;

    PAV pav = BGET_PAV(*it);
    const PrecArray& pa = pav.getPA(dim);
    for(unsigned int i = 0; i < pa.size(); i++){
      if(pa[i] == 0){
	numSegments++;
	continue;
      }
      //only recurse on the segment we are interested in.
      if(numSegments == numSegs)
	appendSpansHelper(mtVector[pa[i]-1], dim, position, ++cntVector[pa[i]-1]);
    }
  }
}



void MTree::findFirstDiscontinuity(iterator& outit) {
  iterator it = begin();
  MTree::findFirstDiscontinuityHelper(it, outit);
}
//helper class
void MTree::findFirstDiscontinuityHelper(iterator it, iterator& outit) {
  MTree tr;

  //if it's a leaf node.
  if ( boost::get<TLink>( &BGET_EITHER_LABEL(*it) ) ){
    return;
  }
  // if it's not a leaf node.
  else{
    MTree::sibling_iterator sib_it;
    vector<MTree::sibling_iterator> itVec;
    if(tr.is_valid(it) && it != tr.end()){
      for(sib_it = it.begin(); sib_it != it.end(); ++sib_it){
	PAV pav = BGET_PAV(*sib_it);
	FanoutVect fv = pav.getFanoutV();
	if(fv[0]  > 1 || fv[1] > 1){
	  outit = it;
	  return;
	}else{
	  findFirstDiscontinuityHelper(sib_it, outit);
	}
      }
    }
  }
}


// The purpose of this method is to abstract away from design
// decisions about whether leaves should contain only terminals
// or also the preterminal labels.  As of Oct'06, MTrees store
// only terminal info in leaves, but PositionTrees store both the
// terminal and its POS tag.
// if iter is not a leaf, then throw exception
const SynCat MTree::pretermLabel(const iterator& iter, arity_t dim) {
    if (iter.number_of_children() > 0) {
      THROW("iter does not point to a leaf");
    }

    MTree dummyTree;
    iterator parIt = dummyTree.parent(iter);
    const SynCat ret = BGET_NT_LABEL(*parIt).getNT(dim)->getSynCat();
    return ret;
};
