#include "MTreeNode.hpp"

bool MTreeNode::includingSpans = false;

PAV& MTreeNode::getPAV() {
  return m_PAV;
};
const PAV& MTreeNode::getPAV() const {
  return m_PAV;
};

RHSElem& MTreeNode::getRHS() {
  return m_RHS;
};
const RHSElem& MTreeNode::getRHS() const {
  return m_RHS;
};

NTLink& MTreeNode::getNTLink() {
  return boost::get<NTLink>(m_RHS);
};
const NTLink& MTreeNode::getNTLink() const {
  return boost::get<NTLink>(m_RHS);
};

TLink& MTreeNode::getTLink() {
  return boost::get<TLink>(m_RHS);
};
const TLink& MTreeNode::getTLink() const {
  return boost::get<TLink>(m_RHS);
};

bool MTreeNode::empty(arity_t dim) const {
  // if it's a leaf, then it has no PAV, so use the DSV
  const Pair<TLink, DSV>* thePair = boost::get<Pair<TLink, DSV> >(addressof(m_RHS));
  if (thePair)
    return thePair->second.empty(dim);

  // o.w. use the PAV
  return m_PAV.rank(dim) == 0;
};

bool MTreeNode::hasLabel(arity_t dim) const {
  // test if it's a leaf
  const Pair<TLink, DSV>* thePair = boost::get<Pair<TLink, DSV> >(addressof(m_RHS));
  if (thePair)
    return false;
  else
    return true;
};

/*
 * Input of MTreeNode.
 */
istream& operator>>(istream& in, MTreeNode& data) {
  in >> ws;

  bool isInternalNode = false;
  //indicates a PAV, so it's an internal node.
  if(in.peek() == '[') {
    char c;
    in >> c;
    //if this is a lone [, its a PAV starting.
    //Otherwise, it's part of a larger token.
    if(in.peek() == ' '){
      in.putback('[');
      isInternalNode = true;
      //read in PAV
      in >> BGET_PAV(data);
    }
    else{
      in.putback('[');
    }
  }
  in >> ws;
  // if its an Internal node (NTLink is label)
  if(isInternalNode){
    in >> BGET_NT_LABEL(data);
  }
  // it is not an internal node. (TLink is label)
  else{
    bool bracketFound = false;
    TLink tl;
    DSV dsv;
    in >> tl;
    in >> ws;
    if(in.peek() == '['){
      bracketFound = true;
      in >> dsv;
    }
    if(MTreeNode::includingSpans && bracketFound)
      BGET_EITHER_LABEL(data) = Pair<TLink,DSV> (tl,dsv);
    else
      BGET_EITHER_LABEL(data) = tl;

    in >> ws;
  }

  return in;
}

/*
 * Output of MTreeNode.
 */
ostream& operator<<(ostream& out, const MTreeNode& data) {
  if (Dbug::isDebugAll(3))
    out << data.getUniqueID() << " ";
  //only print PAVs for NTLinks (internal nodes)
  if ( boost::get<NTLink>( &BGET_EITHER_LABEL(data) ) ){
    out << BGET_CONST_PAV(data);
  }

  out<<" "<<BGET_EITHER_LABEL(data);
  return out;
}
