/// \brief node of a Multitree

#ifndef _MTreeNode_H_
#define _MTreeNode_H_

#include "BoostMacros.hpp"
#include "Dbug.hpp"
#include "DSV.hpp"
#include "PAV.hpp"
#include "NTLink.hpp"
#include "RHSElem.hpp"

//These are boost "tuple" commands. See boost.org for more info.
#define BGET_CONST_PAV(it) (it).getPAV()
#define BGET_PAV(it) (it).getPAV()
#define BGET_NT_LABEL(it) (it).getNTLink()
#define BGET_TERM_LABEL(it) (it).getTLink()
#define BGET_TERM_SPAN_LABEL(it) boost::get<Pair<TLink, DSV> >((it).getRHS())
#define BGET_EITHER_LABEL(it)((it).getRHS())
#define BGET_CONST_HEIR(it) static_cast<const unsigned int>((it))

class MTreeNode: public UniqueID {
public:
  MTreeNode(unsigned int d): m_PAV(d), m_RHS(NTLink(d)) {
    for(unsigned int i = 0; i < d; i++){
      BGET_NT_LABEL(*this).setNT(i,NULL);
    }
  };

  PAV& getPAV();
  const PAV& getPAV()const ;
  RHSElem& getRHS();
  const RHSElem& getRHS()const ;
  NTLink& getNTLink();
  const NTLink& getNTLink() const;
  TLink& getTLink();
  const TLink& getTLink() const;

  bool empty(arity_t dim) const;
  bool hasLabel(arity_t dim) const;

  friend bool operator<(const MTreeNode& t1, const MTreeNode& t2);

  //! MTreeNode input.
  //! If the data indicates the heir, the heir pointer is 0x01, otherwise,
  //! it is 0x0.
  friend istream& operator>>(istream&, MTreeNode&);

  //! MTreeNode output.
  friend ostream& operator<<(ostream&, const MTreeNode&);

  //! true if we want to read in spans on the leaves of the tree
  static bool includingSpans;

protected:
  PAV m_PAV;
  RHSElem m_RHS;
};

#endif
