#include "NTLink.hpp"
#include "LexNT.hpp"
#include "GlobalVocab.hpp"
#include "MoreComparators.hpp"
#include "Exceptions.hpp"
#include "Set.hpp"

Vocabulary<RawNTLink> NTLink::sm_voc;

NTLink::
NTLink(): m_index(sm_voc.addWord(_super()))
{};

// Constructor which sets dimension. (Does not init pointers)
NTLink::
NTLink(const arity_t d): m_index(sm_voc.addWord(_super(d))) {}


// Constructor which does init pointers to Nonterminals
NTLink::
NTLink(const arity_t& _d, const Nonterminal& nt):m_index(sm_voc.addWord(_super(_d,nt))){}

// Constructor which does init pointers to LexNTs
NTLink::
NTLink(const arity_t& _d, const LexNT& nt):m_index(sm_voc.addWord(_super(_d,nt))){}

// Constructor from two nonterms, one in each dimension.
NTLink::
NTLink(Nonterminal* nt1, Nonterminal* nt2):m_index(sm_voc.addWord(_super(nt1,nt2))){}
						   
NTLink::NTLink(const RawNTLink& raw) 
{
  m_index = sm_voc.addWord(raw);
}

 

// Constructor
NTLink::NTLink(const vector<Nonterminal*> &vec):m_index(sm_voc.addWord(_super(vec))){}

//Constructor
NTLink::NTLink(const NTLink& ntLink,const TLink& tlink){
  m_index = sm_voc.addWord(_super( *(sm_voc.getWord(ntLink.m_index)),tlink));
};
								
//Clear
void NTLink::clear(){
  RawNTLink raw(*sm_voc.getWord(m_index));
  raw.clear();
  NTLink x(raw);
  *this = x;
}

//Returns vector of terminals. (only works with LexNT elements.)
TLink NTLink::getTLink() const {
  return  sm_voc.getWord(m_index)->getTLink();
}

//sets vector of terminals. (only works with LexNT elements.)
void NTLink::setTLink(const TLink& tl) {
  RawNTLink raw(*sm_voc.getWord(m_index));
  raw.setTLink(tl);
  NTLink x(raw);
  *this = x; 
}

//sets vector of terminals. (only works with LexNT elements.)
void NTLink::setTerminal(const Terminal& t, arity_t d) {
 RawNTLink raw(*sm_voc.getWord(m_index));
  raw.setTerminal(t,d);
  NTLink x(raw);
  *this = x; 
}

//returns reference to element
void NTLink::setNT(unsigned int i, Nonterminal* nt){
  RawNTLink raw = *sm_voc.getWord(m_index);
  raw.setNT(i,nt);
  Vocabulary< RawNTLink >::const_iterator it;
  NTLink x(raw);
  //  Nonterminal* ntp = raw[0];

  *this = x;  
}

void NTLink::delex(){
  RawNTLink raw(*sm_voc.getWord(m_index));
  raw.delex();
  NTLink x(raw);
  *this = x; 
}

// reverses all elements
void NTLink::reverse(){
 RawNTLink raw(*sm_voc.getWord(m_index));
  raw.reverse();
  NTLink x(raw);
  *this = x; 
}

//! D-dimensional cross-product
//! takes a vector of sets of Xs and returns a set of vectors of Xs.
//! \todo recode this recursively over dim pairs and make it generic
void NTLink::
hyperCrossProduct(vector<Set<Nonterminal*, less<Nonterminal*> > >& sets, set<NTLink>& result) {
  set<RawNTLink> rset;
  RawNTLink::hyperCrossProduct(sets,  rset);
  
  set<RawNTLink>::iterator i;
  for(i = rset.begin(); i != rset.end(); i++){
    result.insert(NTLink(*sm_voc.getWord(sm_voc.addWord(*i))));
  }
}

//! D-dimensional cross-product
//! takes a vector of sets of Xs and returns a set of vectors of Xs.
//! \todo recode this recursively over dim pairs and make it generic
void NTLink::
hyperCrossProduct2(vector<set<Nonterminal*, less<Nonterminal*> > >& sets, set<NTLink>& result) {
  set<RawNTLink> rset;
  RawNTLink::hyperCrossProduct2(sets,  rset);
  
  set<RawNTLink>::iterator i;
  for(i = rset.begin(); i != rset.end(); i++){
    result.insert(NTLink(*sm_voc.getWord(sm_voc.addWord(*i))));
  }
}

//! Returns a NTLink with each element to be NULLNT.
NTLink NTLink::nullNTLink(const arity_t& dim) {
  return NTLink(*sm_voc.getWord(sm_voc.addWord(RawNTLink::nullNTLink(dim))));
}

//! Returns a Unk NTLink.
NTLink NTLink::unkNTLink(const arity_t& d) {
  return NTLink(*sm_voc.getWord(sm_voc.addWord(RawNTLink::unkNTLink(d))));
}

//! Returns a unknown preterminal NTLink.
NTLink NTLink::prtmUnkNTLink(const arity_t& d) {
  return NTLink(*sm_voc.getWord(sm_voc.addWord(RawNTLink::prtmUnkNTLink(d))));
}

//! Returns a Dummy NTLink.
NTLink NTLink:: dummyNTLink(const arity_t& dim) {
  return NTLink(*sm_voc.getWord(sm_voc.addWord(RawNTLink::dummyNTLink(dim))));
}

//! Write.
ostream& NTLink::put(ostream& out) const{
  return  sm_voc.getWord(m_index)->put(out);
}

// For ordering. Arbitrary.
int NTLink::cmp(const NTLink& other) const {
  return MC::cmp((int)m_index, (int)(other.m_index));
}

//output
ostream& NTLink::put(ostream& out, vector<StringVocabulary*> ntvVec)const{
   return  sm_voc.getWord(m_index)->put(out,ntvVec);
}

//output
ostream& NTLink::put(ostream& out, StringVocabulary* ntSet, StringVocabulary* terminalSet )const{
  return  sm_voc.getWord(m_index)->put(out, ntSet, terminalSet);
}

//! Overload only not to output the space at the end.
ostream& operator << (ostream& out, const NTLink& obj){
  out << *(NTLink::sm_voc.getWord(obj.m_index));
  return out;
}

//! Read.
istream& NTLink::get(istream& in){
  RawNTLink raw;
  raw.get(in);
  NTLink nt(raw);
  *this = nt;
  return  in;
}

//! If arity is not specified, reads line by line.
//! Otherwise, reads in terms of arity number.
istream& operator >> (istream& in, NTLink& obj){
  RawNTLink s;
  in >> s;
  obj = NTLink(s);
  return in;
}

void NTLink::setNull(){
  RawNTLink raw(*sm_voc.getWord(m_index));
  raw.setNull();
  NTLink x(raw);
  *this = x; 
}

bool NTLink::isDummy() const{
  return  sm_voc.getWord(m_index)->isDummy();
}
