#include "NaiveItemIndex.hpp"

// Retrieves all the items indexed by the DSV
// d is dummy. Return inf if no edges found.
SCORE NaiveItemIndex::
retrieve(const DSV& span, arity_t d, vector<Item*>& ret) const {
    SCORE min = POSINF;
    hash_map<DSV, pair<vector<Item*>, SCORE> >::const_iterator it;

    it = m_index.find(span);
    if(it != m_index.end()){
      // the set of items index with the span key.
      ret = it->second.first;
      // the maximum f-value over all the items in this index entry.
      min = it->second.second;
    }
    // if there are no items in this index, we return nothing.
    else {
      ret.clear();
    }
    return min;
}

void NaiveItemIndex::insert(const Item* item) {
    const DSV* span = item->getDSV();
    // insert the item.
    m_index[*span].first.push_back(const_cast<Item*>(item));

    // update the minimum of the scores of inserted items.
    // if there is only one item that has been inserted into
    // this index entry,
    if(m_index[*span].first.size() == 1){
      // the minimum f-value is the f-value of the item.
      m_index[*span].second = item->getScore();
    } else {
      // otherwise, we update the minimum f-value field by comparisons.
      SCORE ss = m_index[*span].second = item->getScore();
      if(ss > item->getScore()){
	m_index[*span].second = item->getScore();
      }
    }
}
