#ifndef _NaiveItemIndex_H_
#define _NaiveItemIndex_H_
#include "ItemIndex.hpp"
#include "DSV.hpp"

//! The index from a DSV to edges in Chart.
class NaiveItemIndex : public ItemIndex {
public:

    NaiveItemIndex() {}
    virtual ~NaiveItemIndex() {}

    //! Retrieves all the edges covering the span. If no edges found,
    //! the returned score is inf.
    //! \param span : the span used for the retrieval operation
    //! \param d : dummy here just for consistent interface
    //! \param ret : the set of items with that span
    //! \return the minimum cost of all items in ret
    SCORE retrieve(const DSV& span,
		    arity_t d,
		    vector<Item*>& ret) const;

    //! Insert the index to an item.
    void insert(const Item*);

    //! How many factored indexes are there?
    size_t numFactors() const { return 1;}

    void clear() { m_index.clear();}
protected:

    //! The index.
    //! The value of the hash map is a vector of items
    //! and the minimum cost of these items.
    hash_map<DSV, pair<vector<Item*>, SCORE> > m_index;
};
#endif
