#include "Nonterminal.hpp"
#include "GlobalVocab.hpp"
#include "Exceptions.hpp"
#include "MoreComparators.hpp"
#include "LexNT.hpp"

//! Default constructor (mainly used for i/o)
Nonterminal::Nonterminal(){}

//! Constructor with category
Nonterminal::Nonterminal(SynCat cat){
  m_synCat = cat;
}

//! destructor
Nonterminal::~Nonterminal() {}

//! Getter for Syntactic Category
SynCat Nonterminal::getSynCat()const{
  return m_synCat;
}

//! Getter for Syntactic Category
SynCat& Nonterminal::getSynCat(){
  return m_synCat;
}

//! Setter for Syntactic Category
void Nonterminal::setSynCat(const SynCat c){
  m_synCat = c;
}

// THE FOLLOWING NEED TO BE OVERLOADED WHEN YOU SUBCLASS!

bool Nonterminal::isInactive()const{
  if (m_synCat == UNDEF_PRTM || m_synCat == UNDEF_NT)
    return true;
  else
    return false;
}

ostream& Nonterminal::put(ostream& out)const{
  out << (*this);
  return out;
}

ostream& Nonterminal::put(ostream& out,StringVocabulary* ntset)const{
  /*nonterminal*/
  bool exists;
  const string* nt = ntset->baseVocabulary().getWord(m_synCat, exists);
  if(!exists){
    THROW((*this).nt() << " cannot be found!" << endl);
  }
  out << " { ";
  out << *nt;
  out << " }";
  return out;
}

Nonterminal* Nonterminal::clone() const{
  Nonterminal *newv = new Nonterminal(*this);
  return newv;
};

//! output
ostream& operator << (ostream& out, const Nonterminal& obj){
  out << "{" << obj.getSynCat() << "}";
  return out;
}

//! input
istream& operator >> (istream& in, Nonterminal& obj) {
  char ch;
  SynCat cat;
  string str;
  in >> ws;
  in >> ch;
  assert(ch == '{');
  in >> ws;
  //if integerized input
  if(!GlobalVocab::s_noVocabsIn){
    in >> cat;
  }
  else{
    in >> str;
    cat = GlobalVocab::s_synCatSet->addWord(str);
    in >> ws;
  }
  in >> ws;
  in >> ch;
  assert(ch == '}');
  obj.setSynCat(cat);
  return in;
}

istream& Nonterminal::get(istream& in){

  in >> (*this);
  return in;
}

//! = operator
void Nonterminal::operator =(const SynCat cat){
  m_synCat = cat;
}

//! < operator
bool Nonterminal::operator <(const Nonterminal& other)const{
  return (m_synCat < other.getSynCat());
}

bool Nonterminal::operator <(const Nonterminal* other)const{
  return (m_synCat < other->getSynCat());
}

//! == operator
bool Nonterminal::operator ==(const Nonterminal& other)const{
  return( m_synCat == other.m_synCat);
}
bool Nonterminal::operator ==(const Nonterminal* other)const{
  return( m_synCat == other->m_synCat);
}

int Nonterminal::cmp(const Nonterminal& other) const{
  return MC::cmp(m_synCat, other.m_synCat);
}

int Nonterminal::cmp(const Nonterminal* other) const{
  return MC::cmp(m_synCat, other->m_synCat);
}

size_t Nonterminal::hash() const {
  return nt();
}

//reads a nonterminal from the input and returns a pointer.
// currently only implemented for Nonterminal and LexNT.
Nonterminal* Nonterminal::readNonterminal(istream& in){
  char ch;
  string str;
  Terminal t;
  Nonterminal* ret;
  in >> ws;
  in >> ch; //" {"
  in >> ws;
  SynCat s;
  if(!GlobalVocab::s_noVocabsIn){
    in >> s;
  }
  else{
    in >> str >> ws;
    s = GlobalVocab::s_synCatSet->addWord(str);

  }
  if(in.peek() == '['){
    in >> ch;
    if(!GlobalVocab::s_noVocabsIn){
      in >> t;
    }
    else{
      in >> str >> ws;
      t = GlobalVocab::s_terminalSet->addWord(str);
    }
    in >> ch;
    ret = new LexNT(s,t);
  }
  else{
    ret = new Nonterminal(s);
  }
  in >> ch; // "}"
  in >> ws;
  return ret;
}

vector<unsigned short> Nonterminal::vectorDecompose()const{
  vector<unsigned short> ret;
  ret.push_back(m_synCat);
  return ret;
}

void Nonterminal::setNull(){
  m_synCat = NULLNT;
}

bool Nonterminal::isDummy() const{
  return m_synCat == RootTag;
}

bool Nonterminal::isNull() const{
  return m_synCat == NULLNT;
}
