#ifndef _OptimalEstimator_H_
#define _OptimalEstimator_H_

#include "OutsideCostEstimator.hpp"
#include "GPDefs.hpp"
#include "Grammar.hpp"
#include "NaiveWMTG.hpp"
#include "Dbug.hpp"
#include "MoreAlgorithms.hpp"
#include "ParseStats.hpp"
#include "HeadPMTGA.hpp"
#include "Parser.hpp"

using namespace std;

//! The best possible outside cost estimator.
/*!
 * This estimator gives the theoretical best outside cost estimate.  It
 * computes the full final parse tree using the multilingual parser.  
 * Then, these costs are used to compute the outside cost.
 */
class OptimalEstimator : public OutsideCostEstimator {
  typedef OutsideCostEstimator _super;
public:

  //! Constructor.
  OptimalEstimator(const string& configFile);

  //! Destructor.
  virtual ~OptimalEstimator();

  //! Precompute grammar information
  //! \param grammar : the grammar model from which we precompute 
  //!                  values.  Note that this model should
  //!                  be estimated from the following treebank
  //!                  grammar BEFORE this method is called.
  virtual void precompute(NaiveWMTG *grammar);

  void initialize(SentenceTuple &sent);

  //! Returns the estimate to the outside score (logprob) of the item.
  //! \param sntT : the sentence tuple, the subtuple of which the item
  //! concerned is covering.
  //! \param item : the item, for which we are computing the estimate to
  //! its outside score.
  virtual SCORE operator()(SentenceTuple &sent, const Item* item);

  //! Resets the object.
  virtual void reset();

protected:
  //! The grammar
  Grammar *m_grammar;

  //! The single dimension parsers
  Parser *m_parser;

  //! The scores for the 1-D parses
  SCORE m_score;

  //! The config file for building the parsers
  string m_parserConfigFile;
};

#endif
