#ifndef _OutsideCostEstimator_H_
#define _OutsideCostEstimator_H_

#include "GPDefs.hpp"
#include "Grammar.hpp"
#include "NaiveWMTG.hpp"
#include "Dbug.hpp"
#include "MoreAlgorithms.hpp"
#include "ParseStats.hpp"

using namespace std;

//! The abstract class of outside cost estimator (OCE) class family.
//! At the same time, this class implements the NULL estimate.
class OutsideCostEstimator : public Dbug {
public:

  //! Constructor.
  OutsideCostEstimator(const string& configFile) {}

  //! Destructor.
  virtual ~OutsideCostEstimator();

  //! Precompute grammar information
  //! \param grammar : the grammar model from which we precompute 
  //!                  values.  Note that this model should
  //!                  be estimated from the following treebank
  //!                  grammar BEFORE this method is called.
  virtual void precompute(NaiveWMTG *grammar);

  //! Returns the estimate to the outside score (logprob) of the item.
  //! \param sntT : the sentence tuple, the subtuple of which the item
  //! concerned is covering.
  //! \param item : the item, for which we are computing the estimate to
  //! its outside score.
  virtual SCORE operator()(SentenceTuple &sent, const Item* item);

  //! Resets the object.
  virtual void reset() {}

  //! Initializes the estimator based on the sentence tuple. This sentence
  //! tuple is to be parsed by the main multiparser.
  virtual void initialize(SentenceTuple &sent) {};

  //! Reads in data.
  virtual istream& get(istream&)
  { THROW("OutsideCostEstimator::get not implemented");}

  //! Write the data.
  virtual ostream& put(ostream&) const
  { THROW("OutsideCostEstimator::put not implemented");}

  //! Adaptor of reading.
  friend istream& operator >> (istream& in, OutsideCostEstimator& object)
  { return object.get(in);}

  //! Adaptor of writing.
  friend ostream& operator << (ostream& out, const OutsideCostEstimator& object)
  { return object.put(out);}

  //! Exports the statistics from the computation of the heuristic.
  virtual void exportComputationStatistics(ParseStats& stats)
  { stats.clear();}

  //! Returns the model used in the split parsers.
  virtual Grammar* model(arity_t d) const { return NULL;}

};

#endif
