#ifndef _PAOrderEvent_H_
#define _PAOrderEvent_H_

#include "PAV.hpp"
#include "ImDom.hpp"
#include "Pair.hpp"
// #include "MazMemPool.hpp"
#include "Nonterminal.hpp"
#include "GlobalVocab.hpp"
#include <boost/serialization/serialization.hpp>

//! PAOrderEvent is a triple with two Nonterminals and an 1D PAV. Its
//! functionality is to specify the relative arrangement of the DSVs of
//! an Inference antecedents.
class PAOrderEvent :
  // public MazMemPool,
  public ImDom<Pair<Nonterminal*, Nonterminal*>, PAV> {
public:

  typedef Pair<Nonterminal*, Nonterminal*> NTPair;
  typedef ImDom<NTPair, PAV> _super;
  typedef ImDom<NTPair, PAV>::FirstType FirstType;
  typedef ImDom<NTPair, PAV>::SecondType SecondType;

  //! Constructs according to dimensionality.
  //! The link will always be dim 2 because one
  //! PAOrderEvent is for the parent kid pair in
  //! 1 dimension.
  PAOrderEvent() : _super(NTPair(), PAV(1)){}

  //! Constructor from 2 Nonterminals and an PAV
  PAOrderEvent( Nonterminal* lftsd, Nonterminal* rghtsd,
		const PAV& pav,	arity_t arity)
    : _super (NTPair(lftsd, rghtsd), pav) {
    if(child().improperOrder()){
      child() = PAV::swapElement(child());
      swap(parent().first, parent().second);
    }
    m_arity = arity;
  }

  //! Destructor.
  virtual ~PAOrderEvent() { }

  //! Equality test.
  bool operator == (const PAOrderEvent& other) const {
    return (*parent().first == *other.parent().first &&
	    *parent().second == *other.parent().second &&
	    child() == other.child());
  }

  bool operator<(const PAOrderEvent& other) const {
    return (_super::operator<(other));
  }

  //! For hashing.
  virtual size_t hash() const {
    return (parent().first->hash()* 10000 + parent().second->hash()) * 2 + child().hash();
  }

  virtual size_t getBigValForHash() const { return 2;}

  void setArity(arity_t a){
    m_arity = a;
  }

  //! Output.
  ostream& put(ostream& out) const{
    if(GlobalVocab::s_noVocabsOut){
      (parent().first)->put(out,GlobalVocab::s_synCatSet, GlobalVocab::s_terminalSet);
      out << " ";
      parent().second->put(out,GlobalVocab::s_synCatSet, GlobalVocab::s_terminalSet);
      out << " " << child() << " ";
    }
    else{
      parent().first->put(out);
      out << " ";
      parent().second->put(out);
      out <<" "<< child();
    }
    return out;
  }

  //! Input.
  istream& get(istream& in) {
    in>> *parent().first >> *parent().second >> child();
    return in;
  }

  //! Output.
  friend ostream& operator<<(ostream& out, const PAOrderEvent& ordE){
    return ordE.put(out);
  }

  //! Input.
  friend istream& operator>>(istream& in, PAOrderEvent& ordE) {
    if(ordE.parent().first != NULL)
      delete ordE.parent().first;
    ordE.parent().first = Nonterminal::readNonterminal(in);
    if(ordE.parent().second != NULL)
      delete ordE.parent().second;
    ordE.parent().second = Nonterminal::readNonterminal(in);
    in >> ws;
    if(in.peek() != '\n'){
      in >> ordE.child();
    }
    else{
      ordE.child() = PAV();
    }

    return in;
  }

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    // ar & boost::serialization::base_object<MazMemPool>(*this);
    ar & boost::serialization::base_object<_super>(*this);
  }

private:
  //! Marks which dimension this event is in.
  arity_t m_arity;

};

namespace HashNS {

  typedef Pair<Nonterminal*> NTPair;
  //! Template specialization of STL's hash object
  //! for PAOrderEvent.
  template<>
  struct hash<PAOrderEvent> {
    size_t operator()(const PAOrderEvent& obj) const{
      return obj.hash();
    }
  };

  template<>
  struct hash<NTPair> {
    size_t operator()(const NTPair& obj) const {
      return obj.hash();
    }
  };
}

#endif

