#ifndef _PAVOrderEvent_H
#define _PAVOrderEvent_H

#include "Vector.hpp"
#include "PAV.hpp"
#include "NTLink.hpp"
#include "PAOrderEvent.hpp"
#include <boost/serialization/serialization.hpp>

/**
 * \brief PAVOrderEvent encodes the multi-dimensional linear order of the
 * RHS of an MTG production.
 *
 * It is implemented by a pair of a vector event and an PAV event. Each
 * vector element stores a Link whose index equal the vector element
 * index plus 1. Link index corresponds to the roles in the PAV.
 */
class PAVOrderEvent :
  public ImDom<ImDom<NTLink, NTLink>, PAV> {
  typedef ImDom<ImDom<NTLink, NTLink>, PAV> _super;

 public:

  using _super::first;
  using _super::second;

  typedef PAOrderEvent ElementType;
  typedef _super::FirstType FirstType;
  typedef _super::SecondType SecondType;

  /** Empty constructor. */
  PAVOrderEvent() {}

  PAVOrderEvent(const arity_t d) : _super(ImDom<NTLink, NTLink>
					 (NTLink(d), NTLink(d)),
					 PAV(d)) {}

  bool operator==(const PAVOrderEvent& other) const{
    return _super::operator==(other);
  }

  bool operator<(const PAVOrderEvent& other) const{
    return _super::operator<(other);
  }

  /** Constructor.
   * \param links : indexed links, the index (in the MTG sense) of each
   * link equals the vector index to the link PLUS 1.
   * \param pav : the PAV with each role corresponding to the link
   * with index (in the MTG sense) equaling the role.
   */
  PAVOrderEvent(const FirstType& links, const SecondType& pav);

  /** Destructor. */
  virtual ~PAVOrderEvent() {}

  SecondType& pav() { return second;}
  SecondType pav() const {return second;}

  FirstType& links() { return first;}
  FirstType links() const { return first;}

  arity_t arity() const {return pav().arity();}

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super>(*this);
  }

  //! A hashing function.
  virtual size_t hash() const{
    return _super::hash();
  }

};

namespace HashNS {
  //! Template specialization of STL's hash object
  //! for PAVOrderEvent.
  template<>
  struct hash<PAVOrderEvent> {
    size_t operator()(const PAVOrderEvent& obj) const
    { return obj.hash();}
  };
};
#endif

