/*
 * A Tree structure, with some d-span involving operations, designed
 * to incrementally decorate another type of tree structures.
 */
#ifndef _PAVTree_H_
#define _PAVTree_H_

#include "PAV.hpp"

//! A PAVTree is a static decorator pattern intended to add d-span
//! related functionalities to another decorated tree.
template<class _DecoratedTree>
class PAVTree : public _DecoratedTree {
protected:
  typedef typename _DecoratedTree::tree_node tree_node;
public:
  typedef typename _DecoratedTree::iterator iterator;
  typedef _DecoratedTree _super;
  typedef typename _super::value_type value_type;

  //! Empty constructor.
  PAVTree() ;

  //! Constructor. Takes the decorated object as input.
  PAVTree(const _super& base);

  //! Destructor. Doesn't actually release the memory, but
  //! only restore the "head" pointer in tree to be _head_bk.
  ~PAVTree();

  //! appends child
  template<class iter> iter append_child(iter position, const value_type& x);

  //! Sets PAV to the data pointed to by the input iterator
  void setPAV(iterator it, const PAV& pav);

private:
  //! Backup my own head.
  // tree_node* __head_bk;
};

#ifndef _PAVTree_C_
#include "PAVTree.cpp"
#endif

#endif
