/*
  Holds the statistics for a parse. IT is used in the Chart and Parser
  for example.

*/
#ifndef _ParseStats_H_
#define _ParseStats_H_

#include <iostream>
#include <vector>
#include <stdlib.h>
#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "ScoreDefs.hpp"
//#include "MemSnapshot.hpp"

using namespace std;

static const short MAX_SENT_LEN = 500;

class ParseStats {
public:
  /// constructor.
  ParseStats() : numGenInf(0),
		 filtered(0),
		 numPopInf(0),
		 maxAgdSize(0),
		 elapsedSec(0.0),
		 numItems(0),
		 numMatches(0),
		 m_pRecur(NULL) {
    // memU.clear();
  }

  /// clear the content.
  void clear() {
    numGenInf = 0;
    filtered = 0;
    numPopInf = 0; maxAgdSize = 0; elapsedSec =0.0;
    //	memU.clear();
    numItems = 0;
    numMatches = 0;
    if(m_pRecur){
      m_pRecur->clear();
    }
  }

  ParseStats& operator = (const ParseStats& other) {
    numGenInf = other.numGenInf;
    filtered = other.filtered;
    numPopInf = other.numPopInf;
    maxAgdSize = other.maxAgdSize;
    elapsedSec = other.elapsedSec;
    //	memU = other.memU;
    numItems = other.numItems;
    m_pRecur = other.m_pRecur;
    numMatches = other.numMatches;
    return *this;
  }

  // accumulate. (maxAgenda size will be the maximum).
  ParseStats& operator+= (const ParseStats& other){
    numGenInf += other.numGenInf;
    filtered += other.filtered;
    numPopInf += other.numPopInf;
    numMatches += other.numMatches;
    maxAgdSize = (maxAgdSize > other.maxAgdSize) ? maxAgdSize :
      other.maxAgdSize;
    elapsedSec += other.elapsedSec;
    //	memU += other.memU;
    numItems += other.numItems;
    return *this;
  }

  //// acc the item number.
  void accItemStats(vector<size_t> stats) {
    vector<size_t> :: iterator it;
    for(it = stats.begin(); it != stats.end(); ++it){
      numItems += *it;
    }
  }

  /// output.
  friend ostream& operator << (ostream& o, const ParseStats& stats){
    o <<"MAX AGENDA SIZE:\t"<<stats.maxAgdSize;
    if(stats.getRecursive()){
      o <<"\t"<<stats.getRecursive()->maxAgdSize;
    }
    o<<endl;

    o <<"INFERENCES GENERATED:\t"<<stats.numGenInf;
    if(stats.getRecursive()){
      o <<"\t"<<stats.getRecursive()->numGenInf;
    }
    o<<endl;

    o <<"INFERENCES POPPED:\t"<<stats.numPopInf;
    if(stats.getRecursive()){
      o <<"\t"<<stats.getRecursive()->numPopInf;
    }
    o<<endl;

    o <<"MATCHES PROPOSED:\t"<<stats.numMatches;
    if(stats.getRecursive()){
      o <<"\t"<<stats.getRecursive()->numMatches;
    }
    o<<endl;


    // Only print parsing time if it was set
    if(stats.elapsedSec != 0) {
      o << "PARSING TIME:\t"<<stats.elapsedSec;
      if(stats.getRecursive()){
        o << "\t" << stats.getRecursive()->elapsedSec;
      }
      o<<endl;
    }

    // 	o << "MEMORY -- VIRTUAL, RESIDENT (kB):\t"
    // 	  <<stats.memU.vsz<<" "<<stats.memU.rss<<endl;
    // 	o <<"ITEMS IN CHART:\t"<<stats.numItems<<endl;
    return o;
  }

  /// Number of inferences generated.
  size_t numGenInf;

  /// the number of inferences filtered.
  size_t filtered;

  /// Number of inferences Popped.
  size_t numPopInf;

  /// Maximum size of agenda.
  size_t maxAgdSize;
  
  //! parsing time (in secs).
  SCORE elapsedSec;

  // //! MemUsage
  // MemUsage memU;

  /// the number of items in chart.
  size_t numItems;

  // number of chart matches
  size_t numMatches;

  // returns the recursive stats
  // possibly useful for keeping track of sub-parsers
  // but currently unused
  ParseStats* getRecursive() const {return m_pRecur;}
  void setRecursive(ParseStats* ps){ m_pRecur = ps; }

private:

  /// recursive statistics, for example, to export
  /// the statistics from split parsers
  ParseStats* m_pRecur;
};
#endif
