#ifdef USE_SERIALIZATION
#include "ArchiveMaker.hpp"
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#endif

#include "File.hpp"
#include "Exceptions.hpp"
#include "DebugMacros.hpp"
#include "OptionsPrinter.hpp"
#include <boost/program_options.hpp>
#include "ParserBuilder.hpp"
#include "GrammarBuilder.hpp"
#include "AbstractParser.hpp"
#include "DummyParser.hpp"
#include "Parser.hpp"
#include "DiscrimLearner.hpp"
#include "Path.hpp"

#ifdef USE_SERIALIZATION
#include <boost/serialization/serialization.hpp>
#endif

ParserBuilder* ParserBuilder::_instance = 0;

ParserBuilder* ParserBuilder::Instance(){
  if(_instance == 0){
    _instance = new ParserBuilder;
  }
  return _instance;
}

//! build grammar
AbstractParser* ParserBuilder::build(const string& configFile,
                               BSP<BPO::options_description> priorOptions) {
  string optGrammarBuilderFileName;
  string optParserFileName;
  string optParserType;
  AbstractParser* parser = NULL;

  string optParserConfigFile;

  BPO::options_description config("Grammar config file options");
  config.add_options()
    ("GrammarBuilderFile", BPO::value<string>(&optGrammarBuilderFileName), "GrammarBuilder specifications")
    ("ParserFileName", BPO::value<string>(&optParserFileName), "The file where parser specifications are stored.")
    ("ParserType", BPO::value<string>(&optParserType), "The type of parser to be constructed")
    ;

  BPO::variables_map vm;

  if(configFile != ""){
    cerr << "GrammarBuilder: Compiling config file " << configFile << endl;
    ifstream ifs(configFile.c_str());
    store(parse_config_file(ifs, config), vm);
  }

  notify(vm);

  if(debug(1)) {
    cerr << OptionsPrinter(config, vm) << endl;
  }

  if (optParserType == ""){
    THROW("Parser type not specified" << endl);
  }

  Grammar* grammar = NULL;
  // construct the grammar for our parser
  if (optGrammarBuilderFileName != ""){
    string grammarBuilderConfigFile = Path::maybe_compose(configFile, optGrammarBuilderFileName);
    grammar = GrammarBuilder::Instance()->build(grammarBuilderConfigFile);
  }

  // handle parser config file
  string parserConfig = Path::maybe_compose(configFile, optParserFileName);

  // construct the parser itself
  if (optParserType == "DUMMY"){
    parser = new DummyParser(parserConfig, grammar);
  } else if (optParserType == "DISCRIM_LEARN"){
    parser = new DiscrimLearner(parserConfig, grammar);
  } else if (optParserType == "PARSER"){
    parser = new Parser(parserConfig, grammar);
  } else {
    THROW (cerr << optParserType << " is not a recognized parser. " << endl);
  }

  return parser;

}

