/*! \file ParserBuilder.hpp
 * \brief ParserBuilder class.
 * created Wed Mar 2 21:13:50 EST 2005
 * header by mkh script
 */

#ifndef PARSERBUILDER_H
#define PARSERBUILDER_H

#include "Dbug.hpp"
#include "AbstractParser.hpp"
#include <vector>

class Parser;
class StringVocabulary;

using namespace std;

class ParserBuilder : public Dbug {

 public:
  //! Returns a pointer to the instance of this class.
  static ParserBuilder* Instance();

  /**
  * Builds the grammar based on a configure file. If the grammar
  * file name is specified in the config file, then load the grammar.
  *
  * @param configFile The config file of the grammar.
  * @param overrideArgV The argument vector used to override
  * the config file.
  * @param overrideArgC The size of the override argv.
  */
  AbstractParser* build(const string& configFile,
		 BSP<BPO::options_description> priorOptions =
	         BSP<BPO::options_description>(new BPO::options_description()));

 protected:
  ParserBuilder(){}

 private:
  static ParserBuilder* _instance;

};

#endif
