#ifndef _PartialWMTG_H_
#define _PartialWMTG_H_

#include "File.hpp"
#include "DSV.hpp"
#include "StringVocabulary.hpp"
#include "MultiMap.hpp"
#include "MoreAlgorithms.hpp"
#include "WMTG.hpp"
#include "PositionTree.hpp"
#include "HeirEvent.hpp"

class PartialWMTG: public WMTG {
  typedef WMTG _super;
  typedef Grammar _grammar;

  BSP<BPO::options_description> 
  addOptionDescriptionsTo(BSP<BPO::options_description> optionsDescription);

 public:

  /**
   * Constructor.
   * @param configFile The config file of the grammar.
   */
  PartialWMTG(arity_t d, const string& configFile,
	   BSP<BPO::options_description> priorOptions = 
	   BSP<BPO::options_description>(new BPO::options_description()));

  virtual ~PartialWMTG();

  // Updates the content according to a parse and a sentence tuple.
  // The MTG is passed out.
  bool prepareFor(SentenceTuple& sntT);

  /// by default, no filter
  void filterArrangementFor(BSP<set<Pair<bool,PAV> > >& arrgs, const Item* trigger) const {};

  //! Returns all Z's given X and Y such that Z->XY (or Z->YX).
  //! \param heir : the heir constituent.
  //! \param depend : the dependent constituent.
  //! \param conss : the parent constituents that will be returned.
  void getPossibleCons(const Item* heir, const Item* depend, const Pair<bool,PAV>& pav,
		       vector<Item*>& cons);

  void getPossibleConsHeaded(const Item* heir, const Item* depend, const Pair<bool,PAV>& pav,
			     vector<pair<HeirRoleVector,Item*> >& cons);

  virtual void getPossibleConsForTLink(const TLink& t, const DSV& d, vector<Item*>& vItem) const;

  virtual std::istream& get(std::istream&){THROW("Not Implemented");};
  virtual std::ostream& put(std::ostream&) const{THROW("Not Implemented");};

protected: // methods

  /// This is a helper function for the various getPossibleCons methods.
  /// If this grammar is being used as a supervising grammar for
  /// learning, then we want to update the item's hiddenDSVs structure.
  /// If only one of the antecedents has hiddenDSVs, then those
  /// hiddenDSVs are copied to the consequent.  If both antecedents have
  /// hiddenDSVs, then some elements of their cross-product must be
  /// compatible with the input PAV.  Every such element is summed and
  /// copied to the hiddenDSV of the consequent.  If no element is
  /// compatible with the PAV, then the antecedents cannot compose, so
  /// we abort.
  /// @return true unless the input hiddenDSVs are incompatible with the PAV
  /// @param outHDSVs is the output hiddenDSVs, passed by reference so
  /// that the pointer can be changed
  bool computeHiddenDSVs(const Vector<DSV>* pHDSVs1, const Vector<DSV>* pHDSVs2, 
			 const PrecArray& inPA, Vector<DSV>*& outHDSVs) const;
  
protected: // data

  //The dimension that this constraint is applying to
  arity_t m_constrainedDim;

  //! true if 1D attachments can only attach on one side.
  bool m_rightAttach;

  //! remember the current sentence
  Vector<Terminal>* m_pCurrSent;
};

#endif
