
#include "PositionTreeNode.hpp"
#include "GlobalVocab.hpp"

DSV& PositionTreeNode::getDSV() {
  return m_DSV;
};
const DSV& PositionTreeNode::getDSV() const {
  return m_DSV;
};

RHSElem& PositionTreeNode::getRHS() {
  return m_RHS;
};
const RHSElem& PositionTreeNode::getRHS() const {
  return m_RHS;
};

Vector<unsigned int>& PositionTreeNode::getHRV() {
  return m_HRV;
};
const Vector<unsigned int>& PositionTreeNode::getHRV() const {
  return m_HRV;
};

NTLink& PositionTreeNode::getNTLink() {
  return boost::get<NTLink>(m_RHS);
};
const NTLink& PositionTreeNode::getNTLink() const {
  return boost::get<NTLink>(m_RHS);
};


bool PositionTreeNode::empty(arity_t dim) const {
  return m_DSV.empty(dim);
};

// init the PositionTreeNode
void PositionTreeNode::InitPositionTreeNode(arity_t d) {
  for(unsigned int i = 0; i < d; i++){
    BGET_HEIRVEC(*this)[i] = 0;
  }
}

/*
 * Input of {\em 1-D} PositionTreeNode.
 * Used to read Charniak parse.
 */
void PositionTreeNode::readPositionTreeNode(istream& in, arity_t d) {
  InitPositionTreeNode(1);
  BGET_SPAN(*this)[0].clear();

  // read NTLink.
  LexNT* lnt = new LexNT();
  string str;

  if(!GlobalVocab::s_noVocabsIn){
    in >> *lnt;
    in >> ws;
  }
  else{
    lnt->get(in, *GlobalVocab::s_synCatSet, *GlobalVocab::s_terminalSet, true);
    in >> ws;
  }

  BGET_POSITION_LABEL(*this) = NTLink(1);
  BGET_POSITION_LABEL(*this).setNT(0,lnt);
}

/*
 * Output of PositionTreeNode
 */
void PositionTreeNode::printPositionTreeNode(ostream& out, bool useGlobalVocab) const {
  out << m_uniqueID;
  //heir vector of size 0 means yield.
  bool found = false;
  for(unsigned int i = 0; i < (BGET_HEIRVEC(*this).size()); i++)
    if((BGET_HEIRVEC(*this)[i] != 0)){
      found = true;
      break;
    }

  if(!found)
    out << " " << BGET_DSV(*this) << " ";

  if(useGlobalVocab)
    static_cast<const LexNT*>((BGET_POSITION_LABEL(*this).getNT(0)))->put(out, GlobalVocab::s_synCatSet, GlobalVocab::s_terminalSet, true);
  else
    static_cast<const LexNT*>((BGET_POSITION_LABEL(*this).getNT(0)))->put(out);
}

ostream& operator<<(ostream& out, const PositionTreeNode& node) {
  node.printPositionTreeNode(out);
  return out;
}
ostream& operator<<(ostream& out, PositionTreeNode& node) {
  node.printPositionTreeNode(out);
  return out;
}
