
#ifndef PositionTreeNode_H_
#define PositionTreeNode_H_

#include "DSV.hpp"
#include "NTLink.hpp"
#include "RHSElem.hpp"
//#include "PositionTree.hpp"
#include "boost/tuple/tuple.hpp"

#define BGET_DSV(it) (it).getDSV()
#define BGET_POSITION_LABEL(it) (it).getNTLink()
#define BGET_CONST_HEIR(it) static_cast<const unsigned int>((it))
#define BGET_HEIR(it) static_cast<unsigned int>((it))
#define BGET_HEIRVEC(it) (it).getHRV()
#define BGET_SPAN(it) (it).getDSV()
#define BGET_CONST_SPAN(it) (it).getDSV()

#define DECLARE_POSITION_TREE_DATA(data) PositionTreeNode data; \
                                          data.InitPositionTreeNode( );

class PositionTreeNode : public UniqueID {
  typedef PositionTreeNode _self;
public:

  PositionTreeNode(int i): m_DSV(i), m_HRV(i), m_RHS(NTLink(1)) {};

  const DSV& getDSV() const;
  DSV& getDSV();
  Vector<unsigned int>& getHRV();
  const Vector<unsigned int>& getHRV() const;
  RHSElem& getRHS();
  const RHSElem& getRHS() const;
  NTLink& getNTLink();
  const NTLink& getNTLink() const;

  bool empty(arity_t dim) const;
  bool hasLabel(arity_t dim) const { return true;};

  //! Initialization of PositionTreeNode
  void InitPositionTreeNode(arity_t );

  //! PositionTreeNode output
  void printPositionTreeNode(ostream& out, bool useGlobalVocab = false) const;

  //! PositionTreeNode input.
  void readPositionTreeNode(istream&, arity_t d);

  friend bool operator<(const PositionTreeNode& t1, const PositionTreeNode& t2);

  friend ostream& operator<<(ostream&, const _self&);
  friend ostream& operator<<(ostream&, _self&);

protected:
  DSV m_DSV;
  Vector<unsigned int> m_HRV;
  RHSElem m_RHS;
};

#endif
