#ifndef _RelativePruningStrategy_H_
#define _RelativePruningStrategy_H_

#include "PruningStrategy.hpp"
#include "ItemIndex.hpp"

class Item;

//! Strategy of item pruning. This strategy applies to
//! both FactoredItemIndex and NaiveItemIndex.
class RelativePruningStrategy : public PruningStrategy {

  typedef PruningStrategy _super;

public:

  RelativePruningStrategy(const string& configFile);

  ~RelativePruningStrategy() {
    delete m_pItemIndex;
  }

  /**
   * Returns true if an item is to be pruned. Note that
   * if the relative pruning threshold is larger than 1, then
   * there will be no pruning at all.
   */
  bool prunable(const Item* inf) const;

  /** Inserts a chart index into the item index table.*/
  void memoize(const Item* item);

  /** Gets the original threshold. */
  SCORE getOrigTh() const { return m_t; }

  /** Gets the current threshold (possibly raised from the original). */
  SCORE getCurTh() const { return m_t; }

  void reset();

private:
  bool prunable(SCORE score, SCORE best) const;

  /**
   * The threshold.
   */
  SCORE m_t;

  /**
   * The item index.
   */
  ItemIndex* m_pItemIndex;

  bool m_pruneDummiesAggressively;

};

#endif
