
#include "Scan.hpp"
#include "Hedge.hpp"

//! Constructors
Scan :: Scan(Hedge* hedgeCon, const TLink& tl)
  : _super(PAV(hedgeCon->getFanoutV()), true),
    m_pHedgeCon(hedgeCon),
    m_antecedentTLink(tl) {
  m_pHedgeCon->setInference(this);
}

//! dimensionality.
arity_t Scan :: arity() const {
  return m_pHedgeCon->arity();
}

//! Return the rank of this inference rule.
unsigned int Scan::rank() const {
  return 0;
}

//! Get the size of the derived item.
size_t Scan :: volume() const {
  return ((DSV*)m_pHedgeCon->getDSV())->getSize();
}

//! Gets the consequence item.
Item* Scan :: getConseq() const {
  return m_pHedgeCon;
}

//! compare two Scans
int Scan::cmp(const Inference* other) const {
  unsigned int aID = m_pHedgeCon->getBatchID();
  unsigned int bID = other->getConseq()->getBatchID();

  if(aID != bID)
    return (aID < bID);
  else
    return 0;
}

 vector<const Item*>::iterator Scan::begin(){
  return vector<const Item*>::iterator();
}

 vector<const Item*>::iterator Scan::end(){
  return vector<const Item*>::iterator();
}

//! returns value of "scan" as a rank. Used to compare
//! inferences of different types.
int Scan::typeRank() const { return SCAN_RANK; }

//! sets the consequent to item h. Note that this function
//! deletes the item that it is currently pointing to because
//! it is only used when we copy an item (into the chart).
void Scan::setConseq(const Item* h){
  delete m_pHedgeCon;
  m_pHedgeCon = (Hedge*)h;
}

//! returns true if inferences are equal (based on PAV only).
bool Scan::equalTo(const Inference& inf) const {
  if(const Scan* sc = dynamic_cast<const Scan*>(&inf)){
    if (! (*m_pHedgeCon == *(sc->m_pHedgeCon)))
	return false;
    if (! (m_antecedentTLink == sc->m_antecedentTLink))
	return false;

    return true;
  } else
    return false;
}

TLink Scan::getTLink()const{
  return m_antecedentTLink;
}

void Scan::dumpAnts(ostream& out) const {
  out << "TLink = " << m_antecedentTLink;
}
