
#ifndef _Scan_H_
#define _Scan_H_

#include "Inference.hpp"
#include "Hedge.hpp"

static const short SCAN_RANK = 4;

//! Scan inference rule. Used in bottom-up logic
class Scan : public Inference {
  typedef Inference _super;
public:

  //! constructor
  Scan( Hedge* con, const TLink& tl);

  //! destructor
  virtual ~Scan() {}

  // returns a copy of this
  virtual Inference* clone() const{
    return new Scan(*this);
  };

  //! dimensionality
  arity_t arity() const;

  //! how many antecedents are there?
  virtual unsigned int rank() const;

  //! Get the size of the span of the derived item.
  virtual size_t volume() const;

  //! Antecedents.
  vector<const Item*> getAnt() const
  { vector<const Item*> ret; return ret; }

  //! Get the consequent item.
  Item* getConseq() const;

  virtual vector<const Item*>::iterator begin();

  virtual vector<const Item*>::iterator end();


  //! Returns the number of non-null dims.
  size_t numActiveDims() const
  { return m_pHedgeCon->numActiveDims(); }

  //! Assigns a rank to the type of this inference.
  int typeRank() const ;

  //! sets the consequent to item h. Note that this function
  //! deletes the item that it is currently pointing to because
  //! it is only used when we copy an item (into the chart).
  void setConseq(const Item* h);

  //! equality test (based on PAV)
  bool equalTo(const Inference& inf) const;

  //compare two inferences based on PAV
  int cmp(const Inference* inf) const;

  // returns the TLink that was the antecedent to this Scan.
  TLink getTLink()const;

  void dumpAnts(ostream& out) const;

  // find the smallest descendant containing this item's head on the given dimension
  virtual Item* headItem(arity_t dim) const { return m_pHedgeCon;}

protected:

  //! This is inference consequent.
  Hedge* m_pHedgeCon;

  //! TLink holds the axiom which created this scan
  TLink m_antecedentTLink;

};

#endif

