#ifndef _SEARCHSTRATEGY_H_
#define _SEARCHSTRATEGY_H_

#include "Dbug.hpp"
#include "UniqueID.hpp"
#include "SentenceTuple.hpp"
#include "Grammar.hpp"

using namespace std;

class Item;

//! An abstract SearchStrategy class.
class SearchStrategy : public UniqueID {
public:
  //! Default Constructor
  SearchStrategy() {}

  //! Constructor.
  SearchStrategy(const string &configFile) {}

  //! Destructor.
  virtual ~SearchStrategy() {}

  //! Alerts the search strategy to changes in the grammar
  virtual void setGrammar(Grammar *g) {}

  //! Initialize for parsing a sentence
  virtual void initialize(SentenceTuple &sent) {
    clear();
  }

  //! Add an item to be ordered
  virtual void addItem(Item* item) = 0;

  //! get the next item to be parsed
  virtual Item* nextItem() = 0;

  //! Clears the content of the SearchStrategy.
  virtual void clear() = 0;

  virtual size_t size() const = 0;

  virtual bool empty() const =0;

  //! Dumps the content for diagnostics.
  virtual void dump(ostream& out) const = 0;

  //! Check whether an item has already been seen by the search strategy
  virtual bool exists(const Item&) const = 0;

protected:
};

#endif
