#include "SearchStrategyBuilder.hpp"
#include <boost/program_options.hpp>
#include <fstream>
#include "OptionsPrinter.hpp"
#include "Exceptions.hpp"
#include "MoreStrings.hpp"
#include "AgendaSearchStrategy.hpp"
#include "AStarSearchStrategy.hpp"
#include "GPDefs.hpp"
#include "Path.hpp"

using namespace std;

SearchStrategyBuilder* SearchStrategyBuilder::_instance = 0;

//! Singleton pattern
SearchStrategyBuilder* SearchStrategyBuilder::Instance() {
  if(_instance == 0) {
    _instance = new SearchStrategyBuilder();
  }
  return _instance;
}

//! Build search strategy from config file
SearchStrategy* SearchStrategyBuilder::build(const string& configFile) {

  // strings to find in the config file
  string strategyType;
  string stratConfigFile;

  // Compile the config file
  BPO::options_description config("Configuration");
  config.add_options()
      ("StrategyType", BPO::value<string>(&strategyType), "The search strategy of the parser")
      ("StrategyConfigFile", BPO::value<string>(&stratConfigFile), "The config file for the search strategy")
      ;
  BPO::variables_map vm;
  cerr << "SearchStrategyBuilder: Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
    cerr << "Option values of \"SearchStrategy\"" << endl << endl;
    cerr << OptionsPrinter(config, vm) << endl;
  }

  // Make sure we know what strategy to use
  if(strategyType == "") {
    THROW("Error: the type of SearchStrategy is not specified!" << endl);
  }

  string strStrategyType = tolower(strategyType);

  stratConfigFile = Path::maybe_compose(configFile, stratConfigFile);

  // Create the correct search strategy
  if(strStrategyType == "agenda") {
    return new AgendaSearchStrategy(stratConfigFile);
  } else if(strStrategyType == "astar") {
    return new AStarSearchStrategy(stratConfigFile);
  } else {
	THROW("Error: search strategy " + strStrategyType + " is unknown");
  }
}

