#include <iostream>
#include "SentenceTuple.hpp"
#include "GlobalVocab.hpp"
#include "Exceptions.hpp"
#include "Dbug.hpp"

// Each sentence tuple is a record. The field of the sentence
// tuple is separated by
#define FIELD_SEP '\n'

SentenceTuple::SentenceTuple(){
  // m_count = 1;
}

SentenceTuple::SentenceTuple(const arity_t& dim) { 
  resize(dim);
  // m_count = 1; 
}

SentenceTuple::
SentenceTuple(const Vector<Vector< Terminal> > in) {
  _super::operator=(in); 
  // m_count = 1; 
}

vector<short>
SentenceTuple::lengths() const {
  vector<short> ret;
  for(arity_t d = 0; d < size(); ++d)
    ret.push_back(this->operator[](d).size());
  return ret;
}

void SentenceTuple::
setSentenceTuple(const Vector<Vector< Terminal> > in) {
  _super::operator=(in); 
  // m_count = 1; 
}

//! concatenation
SentenceTuple& SentenceTuple::operator+= (const SentenceTuple& snt) {
  for(unsigned int i = 0; i < snt.size(); i++){
    for(unsigned int j = 0; j < snt[i].size(); j++){
      ((*this)[i]).push_back(snt[i][j]);
    }
  }
  return *this;
}

istream& operator>>(istream& in, SentenceTuple& sntT) {
  return sntT.get(in);
}

//!input operator
istream& SentenceTuple::get(istream& in) {
  string buffer;
  if(!getline(in, buffer, FIELD_SEP)) {
    return in;
  };
  if (Dbug::isDebugAll(1))
    cerr << buffer + FIELD_SEP;

  // start line of a sentence tuple.
  if(!(buffer == "<s>")){
    THROW("Error: sentence tuple should start with <s>"<<endl);
  }

  // sentences on different dimension.
  bool metEOST = false; // met with the end of sentence tuple
  arity_t d = 1;

  while(getline(in, buffer, FIELD_SEP)){
    if (Dbug::isDebugAll(1))
      cerr << buffer + FIELD_SEP;
    if(buffer == "</s>") {
      metEOST = true;
      break;
    }

    if(arity() < d){
      (*this).setDimity(arity() +1); 
    }
    istringstream ist1(buffer);

    if(!GlobalVocab::s_noVocabsIn){
      ist1 >> (*this)[d-1];
    }
    else{// no vocab passed in. Need to create it on the fly!
      Vector<string> os;
      ist1 >> os;
      Vector<string>::iterator it;
      (*this)[d-1].clear();
      
      for(it = os.begin(); it != os.end(); it++){
	VocabularyIndex vi = GlobalVocab::s_terminalSet->addWord(*it);
	(*this)[d-1].push_back(vi);
      }
    }
    (*this)[d-1].push_back(EOSH);
    d++;
  }

  if(d-1 != arity()){
    THROW("Error: less than "<<(int)arity()<<" dimensions are read!"<<endl);
  }

  if(!metEOST){
    THROW("Error: sentence tuple end with no </s>"<<endl);
  }

  // add the EOSH automatically.
  for(d = 0; d < arity(); ++d){
    if((*this)[d].size() && !((*this)[d].back() == EOSH)){
      (*this)[d].push_back(EOSH);
    }
  }

  return in;
}

//! output operator for snt
ostream& operator<<(ostream& out, const SentenceTuple& sntT) {
  return sntT.put(out); 
}

void SentenceTuple::dump(ostream& out, StringVocabulary* vcb) const
{ put(out); }

ostream& SentenceTuple::put(ostream& out, StringVocabulary* vcb) const {
  unsigned int i, j;
  out<<"<s>"<<endl;
  // out<<m_count<<endl;
  for(i = 0; i < arity(); i++) {
    size_t l = (*this)[i].size();
    if((*this)[i].size() && (*this)[i].back() == EOSH){
      l = (*this)[i].size() - 1;
    }
    for(j = 0; j < l ; j++) {
      bool exists = false;
      const string* word = vcb->getWord((*this)[i][j],exists);
      if(!exists) {
	cerr<<"WARNING: Word ID "<<(*this)[i][j]
	    <<" is not in vocabulary (SentenceTuple::put())\n";
      }
      else out<< *word <<" ";
    }
    out<<endl;
  }
  out<<"</s>"<<endl;
  return out;
}

SentenceTuple& SentenceTuple::rotate() {
  if(arity() > 1){
    Vector<Terminal> snt = (*this)[0];
    for(arity_t d = 1; d < arity(); ++d){
      (*this)[d-1] = (*this)[d];
    }
    (*this).back() = snt;
  }
  return *this;
}

//! Counts of the terminals.
vector<hash_map<Terminal, size_t> > SentenceTuple::countTerminals() const {
  vector<hash_map<Terminal, size_t> > wordCounts(arity());
  for(arity_t d = 0; d < arity(); ++d){
    for(size_t i = 0; i < (*this)[d].size(); ++i){
      if(wordCounts[d].find((*this)[d][i]) == wordCounts[d].end()){
	wordCounts[d][(*this)[d][i]] = 1;
      } else {
	wordCounts[d][(*this)[d][i]]++; 
      }
    }
  }

  return wordCounts;
}

//! Clears the sentence tuple while preserving the dimensionality.
void SentenceTuple::clear() 
{ arity_t dm = arity(); _super::clear(); setDimity(dm); }

bool SentenceTuple::empty() const {
  if( // !m_count || 
     !arity() ) return true;

  for(arity_t d = 0; d < arity(); d++) {
    if((*this)[d].size() > 0 ) {return false;}
  }
  return true;
}

