#ifndef _SentenceTuple_H_
#define _SentenceTuple_H_

#include <vector>
#include <string>
#include "MoreStrings.hpp"
#include "GPDefs.hpp"
#include "GlobalVocab.hpp"
#include "Vector.hpp"
#include "StringVocabulary.hpp"
#include <boost/serialization/serialization.hpp>

using namespace std;

//! This class represents a tuple of sentences.
class SentenceTuple : public Vector< Vector<Terminal> > {
  typedef Vector< Vector<Terminal> > _super;

public:
  //! Empty constructor. Constructors an empty sentence tuple consisting of
  //! no components.
  SentenceTuple() ;

  //! Constructor.
  SentenceTuple(const arity_t& dim);

  SentenceTuple(const Vector<Vector< Terminal> > in);

  //! Destructor.
  virtual ~SentenceTuple() {}

  //! sets the sentence tuple
  void setSentenceTuple(const Vector<Vector< Terminal> > in);

  //! vector of lengths
  vector<short> lengths() const;

  //! Returns arity.
  arity_t arity() const { return (arity_t)size();}

  //! Sets the arity (number of sentences will be contained
  //! in this tuple).
  void setDimity(const arity_t& d) { resize(d); }

  //! Clears the sentence tuple while preserving the dimensionality.
  void clear();

  /** @brief Checks whether the sentence tuple is empty.
   * 
   * Returns true if (i) the count field is 0, or (ii)
   * dimensionality is 0, or (iii) all sentence lengths
   * are 0 (EOSH is counted into the sentence length).
   */
  bool empty() const;

  /// assignment.
  SentenceTuple& operator= (const SentenceTuple& snt) {
    _super::operator=(snt);
    return *this;
  }

  /// concat.
  SentenceTuple& operator+= (const SentenceTuple& snt);

  //! Input a sentence tuple from the in stream.
  friend istream& operator>>(istream& in, SentenceTuple& sntT); 
  istream& get(istream& in);

  //! Output.
  friend ostream& operator<<(ostream& out, const SentenceTuple& sntT);
  void dump(ostream& out, StringVocabulary*) const;
  ostream& put(ostream& out, StringVocabulary* vcb = GlobalVocab::s_terminalSet) const;

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object<_super>(*this);
  };

  //! Sentence 1D -> 2D -> ... -> 1D.
  SentenceTuple& rotate();

  //! Counts of the terminals.
  vector<hash_map<Terminal, size_t> > countTerminals() const;

  // //! Returns the full span covering D dimensions.
  // DSV fullSpan() const;

};

#endif
