

#ifndef _SpanTerminationTest_H_
#define _SpanTerminationTest_H_

#include "TerminationTest.hpp"
#include "DSV.hpp"

/// The parsing goal specifying a item label.
class SpanTerminationTest : public TerminationTest {
    typedef TerminationTest _super;
public:
    typedef _super :: HowAchieved HowAchieved;

    /// constructor.
    SpanTerminationTest(const DSV& span) : m_span(span) {}

    /// Destructor.
    virtual ~SpanTerminationTest() {}

    /// add a subgl, returning a ID for this sub-goal.
    GoalID add(TerminationTest* subGoal) {
	// cannot add a subgoal because this is a primitive goal.
	THROW("SpanTerminationTest::add not implemented");
    }

    /// returns true is this goal is achieved.
    /// \param item : the item just derived.
    /// \param which : which subgoal is reached. If no goal is
    /// reached, then NONE, otherwise, returned 0.
    bool achieved(const Item*item, GoalID& which) {
	if(item){

	  DSV dsv = *(item->getDSV());
	  
	  arity_t min = (m_span.arity() < dsv.arity())?m_span.arity():dsv.arity();
	  
	  for(unsigned int i = 0; i < min; i++){
	    if(!(dsv[i] == m_span[i])){
	      which = NONE;
	      setGoal(NONE);
	      return false;	  
	    }
	  }
	  which = 0;
	  setGoal(0);
	  return true;
	}
	which = NONE;
	setGoal(NONE);
	return false;	  
    }

    /// Checks whether the goal is reached. Used after
    /// the above method is called.
    bool achieved() const {
	if(anyAchieved()){
	    return true;
	} else {
	    return false;
	}
    }

    /// assignment.
    SpanTerminationTest& operator = (const SpanTerminationTest& other){
	_super::operator=(other);
	m_span = other.m_span;
	return *this;
    }

  //! reset the goals
  virtual void reset(DSV& span){
    TerminationTest::reset(span);
    m_span = span;
  }

private:
    DSV m_span;

};

#endif

