/*
 * A Tree structure, with some d-span involving operations, designed
 * to incrementally decorate another type of tree structures.
 */
#ifndef _SpanTree_H_
#define _SpanTree_H_

#include "DSV.hpp"

//! A SpanTree is a static decorator pattern intended to add d-span
//! related functionalities to another decorated tree.
template<class _DecoratedTree>
class SpanTree : public _DecoratedTree {
protected:
  typedef typename _DecoratedTree::tree_node tree_node;
public:
  typedef typename _DecoratedTree::iterator iterator;
  typedef _DecoratedTree _super;
  typedef typename _super::value_type value_type;

  //! Empty constructor.
  SpanTree() ;

  //! Constructor. Takes the decorated object as input.
  SpanTree(const _super& base);

  //! Destructor. Doesn't actually release the memory, but
  //! only restore the "head" pointer in tree to be _head_bk.
  ~SpanTree();

  //! appends child and propagate the spans.
  template<class iter> iter append_child(iter position, const value_type& x);
  template<class iter> iter append_child(iter position);
  //template<class iter> iter append_child(iter position, const value_type& x);

  //! (1) Adds span to the data pointed to by the input iterator,
  //! (2) Propagates the span to the ancestors.
  void addSpan(iterator it, const DSV& span);

  //! Removes span from the current node pointed to by the input iterator,
  //! Removes span from the ancestors.
  void removeSpan(iterator it, const DSV& span);

private:
  //! Backup my own head.
  // tree_node* __head_bk;
};

#ifndef _SpanTree_C_
#include "SpanTree.cpp"
#endif

#endif
