#include "TLink.hpp"
#include "GlobalVocab.hpp"
#include "Exceptions.hpp"

TLink::TLink(const _super vect) {
  for(_super::const_iterator it = vect.begin(); it != vect.end(); ++it){
    push_back(*it);
  }
}

//output
ostream& TLink::put(ostream& out, StringVocabulary* tvoc)const{
  for(arity_t d = 0; d < arity(); ++d){
    if(d){ out << " | "; }

    if(tvoc){
      string t = "";
      bool exists;
      t += *(tvoc->getWord((*this)[d], exists));
      assert(exists);
      out<<t << " ";
    } else { ; out<<(*this)[d]; }
  }
  return out;
}

bool TLink::operator==(const TLink & lexlink) const {
  //arity of each side must be the same to compare
  if(arity() != lexlink.arity()){
    return false;
  }
  
  for(arity_t i= 0; i < lexlink.arity(); i++)
    if(lexlink[i] != (*this)[i])
      return false;
  return true;
}

size_t TLink::hash() const {
  size_t ret = 0;
  for(arity_t i = 0; i < arity(); i++)
    {ret *= sm_bigValForHash; ret += (*this)[i];}
  return ret;
}

/*
 * Merge a TLink with d1 dimensions with another TLink
 * with d2 dimensions to a TLink with d1+d2 dimensions.
 */
TLink TLink::operator+(const TLink& other) const {
  TLink ret = *this;
  ret.insert(ret.end(), other.begin(), other.end());
  return ret;
}

/*
 * Splits the TLink into a pair of TLinks,
 * the first one is of the specified dimension.
 */
pair<TLink, TLink> TLink:: split(const arity_t& dim) const {
  assert(arity() >= dim);

  TLink link1(dim);
  TLink link2(arity() - dim);

  arity_t d;
  for(d = 0; d < dim; d++){
    link1[d] = (*this)[d];
  }

  while(d < arity()){
    link2[d-link1.arity()] = (*this)[d];
    d++;
  }

  pair<TLink, TLink> linkPair(link1, link2);

  return linkPair;
}

//! How many nonnull terminals are there in the TLink.
size_t TLink::numNonnulls() const {
  size_t ret = 0;
  for(arity_t d = 0; d < arity(); ++d){
    if(!((*this)[d] == NULLT)){
      ret++;
    }
  }
  return ret;
}

//! Is the TLink active in the given component?
bool TLink::active(const arity_t d) const {
  return (*this)[d] != NULLT;
};

//! Find the lowest active component
arity_t TLink::firstActiveComponent() const {
  for(arity_t d = 0; d < arity(); ++d){
    if(!((*this)[d] == NULLT)){
      return d;
    }
  }
  THROW("No active components found.");
};

ostream& operator << (ostream& out, const TLink& obj){
  //if we are not using vocabs...
  if(GlobalVocab::s_noVocabsOut){
    obj.put(out,GlobalVocab::s_terminalSet);
  }
  else{ //else, we are using vocabs. (integerize)
    for(arity_t d = 0; d < obj.arity(); ++d){
      if(d){ out << "|"; }
      out<<obj[d];
    }
  }
  return out;
}

istream& operator >> (istream& in, TLink& obj){

  if(!GlobalVocab::s_noVocabsIn){ //If we are integerizing for i/o
    Terminal t;
    char ch;
    obj.clear();
    in >> ws;
    while(in >> t >> ws){
      obj.push_back(t);
      if(in.peek() != '|')
	break;
      else{
	in >> ch;
	in >> ws;
      }
    }
  } else{ //else, we are not using vocabs externally.
    string str;
    char ch;
    obj.clear();
    unsigned int d = 0;
    // for each terminal to be put in the tlink
    while(in >> str >> ws){
      // add it to the vocabulary, getting back its index
      VocabularyIndex vi = GlobalVocab::s_terminalSet->addWord(str);
      // put its index into the tlink
      obj.push_back(vi);
      if(in.peek() != '|')
	break;
      else
	in >> ch;
      d++;
    }
  }

  return in;
}

