#ifndef _TLink_H_
#define _TLink_H_

#include <sstream>
#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "Vector.hpp"
#include "GPDefs.hpp"
#include "Vocabulary.hpp"
#include "StringVocabulary.hpp"
#include <boost/serialization/serialization.hpp>

using namespace std;

//! Terminal Link class.  A terminal link is a vector
// of terminals, where the terminals are said to be translationally
// equivalent. 
class TLink : public Vector<Terminal> {
  
  typedef Vector<Terminal> _super;

public:
  //! Needed for hashing. Set to max number of nonterminals in the
  //! second language.
  //! Need to add code here!!!
  static const size_t sm_bigValForHash = 1109;

  //! Default constructor.
  TLink(const arity_t& dim) : Vector<Terminal>(dim) { }

  //! Constructor from terminal.  Puts compy of terminal in each dimension.
  TLink(const arity_t& dim, const Terminal& t) : Vector<Terminal>(dim,t) { }

  //! constructor from Vector of Terminals.
  TLink(const _super vect);

  //! empty constructor
  TLink() { }

  //! Destructor.
  virtual ~TLink() {}

  //! Returns the arity (dimensionality).
  arity_t arity() const { return (arity_t)size();}

  //! Sets language dimensionality.
  void setDimity(const arity_t& d) { resize(d);}

  //! Compares if two TLinks are eual
  virtual bool operator==(const TLink & lexlink) const;

  //! Merge a TLink with d1 dimensions with another TLink
  //! with d2 dimensions to a TLink with d1+d2 dimensions.
  TLink operator+(const TLink& other) const;

  // returns a hash value for the TLink.
  size_t hash() const;

  //write the TLink to out.  Use the given vocabulary to output words.
  ostream& put(ostream& out, StringVocabulary* tvoc)const;

  //! Splits the TLink into a pair of TLinks,
  //! the first one is of the specified dimension.
  pair<TLink, TLink> split(const arity_t& dim) const;

  //! How many nonnull terminals are there in the TLink.
  size_t numNonnulls() const;

  //! Is the TLink active in the given component?
  bool active(const arity_t d) const;

  //! Find the lowest active component
  arity_t firstActiveComponent() const;

  //! Overload only not to output the space at the end, and also use | and
  //! { }
  friend ostream& operator << (ostream& out, const TLink& obj);

  //! Overload input operator. uses | and { and }
  friend istream& operator >> (istream& in, TLink& obj);

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    ar & boost::serialization::base_object< _super >(*this);
  };

};

namespace HashNS {
  template<> class hash< TLink > {
     public:
     size_t operator() (const TLink& ll) const
     { return ll.hash(); }
   };
}

#endif
