#ifndef _TerminationTest_H_
#define _TerminationTest_H_

//#include "Logic.hpp"
#include "NTLink.hpp"

class Item;
class Logic;
class DSV;

//! The base class of parsing goals.
class TerminationTest {
 public:
  //! How the goal is achieved? ALL means that all
  //! subgoals are achieved. NONE means that none
  //! subgoal is achieved. In other cases, the
  //! Goal ID will be used.
  enum HowAchieved { ALL=-2, NONE=-1};

  //! The ID type of subgoals.
  typedef int GoalID;

  TerminationTest() : m_goal(NONE) {}

  //! Destructor.
  virtual ~TerminationTest() {

}

  //! add a subgl, returning a ID for this sub-goal.
  virtual GoalID add(TerminationTest* subGoal) = 0;

  //! returns true is this goal is achieved.
  //! \param item : the item just derived.
  //! \param logicptr a pointer to the logic that will be consulted
  //! \param which : which subgoal is reached. If no goal is
  //! reached, then NONE. If all goals are reached, then
  //! ALL.
  virtual bool achieved(const Item* item,
			const Logic* logicptr, GoalID& which) {
    return achieved(item, which);
  }

  virtual bool achieved(const Item*item,
			GoalID& which) = 0;

  //! Checks whether the goal is reached. Used after
  //! the above method is called.
  virtual bool achieved() const = 0;

  //! reset this goal such that no subgoal has been achieved.
  virtual void reset() { m_goal = NONE;}

  //! redefine the label, if applicable
  virtual void redefine(NTLink label) { m_goal = NONE; }

  //! reset the span for each sentence, if applicable
  virtual void reset(DSV& span) { m_goal = NONE;}

  //! trace which goal has been achieved.
  GoalID trace() const { return m_goal;}

  //! Assignment.
  TerminationTest& operator=(const TerminationTest& other){
    m_goal = other.trace();
    return *this;
  }

protected:
  //! set the goal
  void setGoal(GoalID id) { m_goal = id; }

  bool allAchieved() const { return (m_goal == ALL); }

  bool anyAchieved() const { return (m_goal != NONE); }

private:
  GoalID m_goal;

};

#endif
