#include "TerminationTestBuilder.hpp"

#include <fstream>
#include "Path.hpp"
#include "Exceptions.hpp"
#include <boost/program_options.hpp>
#include "OptionsPrinter.hpp"

#include "LabelTerminationTest.hpp"
#include "SpanTerminationTest.hpp"
#include "ThresholdTerminationTest.hpp"
#include "TimeTerminationTest.hpp"
#include "ConjunctCompositeTerminationTest.hpp"
#include "DisjunctCompositeTerminationTest.hpp"

TerminationTestBuilder* TerminationTestBuilder::_instance = 0;

TerminationTestBuilder* TerminationTestBuilder::Instance(){
  if(_instance == 0){
    _instance = new TerminationTestBuilder;
  }
  return _instance;
}

//! build parsing goal
TerminationTest* TerminationTestBuilder::build(const string& configFile, NTLink label, DSV span) {
  string optTerminationTest;
  string optTerminationTestConfigFile;

  // compile the config file.
  BPO::options_description config("Configuration");
  config.add_options()
    ("TerminationTest", BPO::value<string>(&optTerminationTest), "The type of the parsing goal.")
    ("TerminationTestConfigFile", BPO::value<string>(&optTerminationTestConfigFile), "The parsing goal config file.");
  BPO::variables_map vm;
  cerr << "TerminationTestBuilder: Compiling config file " << configFile << endl;
  iFile ifs(configFile.c_str());
  store(parse_config_file(ifs, config), vm);
  notify(vm);

  if(debug(1)) {
    cerr << "Option values of \"TerminationTestBuilder\"" << endl << endl;
    cerr << OptionsPrinter(config, vm) << endl;
  }
  if(optTerminationTest == ""){
    THROW("Error: Parsing goal unspecified!"<<endl);
  }

  string strTerminationTest = optTerminationTest;
  strTerminationTest = tolower(strTerminationTest);

  TerminationTest* goal;

  if (strTerminationTest == "conjunct" || strTerminationTest == "disjunct") {
    if (strTerminationTest == "conjunct")
      goal = new ConjunctCompositeTerminationTest();
    else goal = new DisjunctCompositeTerminationTest();
    vector<string> strvec = split(optTerminationTestConfigFile, "|");
    for (unsigned int i = 0; i < strvec.size(); i++) {
      vector<string> goalDefine = split(strvec[i], ",");
      if (goalDefine.size() != 2) {
	THROW("The config file is incorrectly formatted. It should be "
	      << "of the form: "
	      << "\"time,timeConfigFileName|threshold,thresholdConfigFileName [...] \""
	      << endl);
      }
      if (strTerminationTest == "conjunct")
	((ConjunctCompositeTerminationTest*)goal)->add(buildOne(goalDefine[0], goalDefine[1], label, span));
      else
	((DisjunctCompositeTerminationTest*)goal)->add(buildOne(goalDefine[0], goalDefine[1], label, span));
    }
    return goal;
  }

  string parsingGoalConfigFile = Path::maybe_compose(configFile, optTerminationTestConfigFile);

  if(strTerminationTest == "label" || strTerminationTest == "span" ||
     strTerminationTest == "threshold" || strTerminationTest == "time") {
    goal = buildOne(strTerminationTest, parsingGoalConfigFile, label, span);
  } else {
    THROW("Error: unknown parsing goal!"<<endl);
  }
  return goal;
}

TerminationTest* TerminationTestBuilder::buildOne(const string& goalType, const string& configFile, NTLink label, DSV span) {
  TerminationTest* goal;
  if (goalType == "conjunct" || goalType == "disjunct")
    goal = build(configFile, label, span);
  else if(goalType == "label"){
    goal = new LabelTerminationTest<NTLink>(label); // no config file needed
  } else if(goalType == "span"){
    goal = new SpanTerminationTest(span); // no config file needed
  } else if(goalType == "threshold"){
    goal = new ThresholdTerminationTest(configFile);
  } else if(goalType == "time"){
    goal = new TimeTerminationTest(configFile);
  } else {
    THROW("Error: unknown parsing goal!"<<endl);
  }
  return goal;
}
