#ifndef _ThresholdTerminationTest_H_
#define _ThresholdTerminationTest_H_

#include "TerminationTest.hpp"
#include "Item.hpp"
#include "OptionsPrinter.hpp"
#include <boost/program_options.hpp>

/// The parsing goal specifying a item label.
class ThresholdTerminationTest : public TerminationTest {
    typedef TerminationTest _super;
public:
    typedef _super :: HowAchieved HowAchieved;

    /// constructor.
    ThresholdTerminationTest(const SCORE & th) : m_th (th) {}

    /// construct from config file
    ThresholdTerminationTest(const string& configFile) {
      SCORE threshold;
      BPO::options_description config("Threshold config file options");
      config.add_options()
	("th", BPO::value<SCORE>(&threshold), "Threshold value");
      BPO::variables_map vm;
      iFile ifs(configFile.c_str());
      store(parse_config_file(ifs, config), vm);
      notify(vm);
      m_th = threshold;
    }

    /// Destructor.
    virtual ~ThresholdTerminationTest() {}

    /// add a subgoal, returning a ID for this sub-goal.
    GoalID add(TerminationTest* subGoal) {
			// cannot add a subgoal because this is a primitive goal.
			THROW("ThresholdTerminationTest::add not implemented");
    }

    /// returns true is this goal is achieved.
    /// \param item : the item just derived.
    /// \param which : which subgoal is reached. If no goal is
    /// reached, then NONE, otherwise, returned 0.
    bool achieved(const Item*item, GoalID& which) {
			if(item){
	    	if(item->getScore() <= m_th){
					which = 0;
					setGoal(0);
					return true;
	    	}
			}
			which = NONE;
			setGoal(NONE);
			return false;
    }

    /// Checks whether the goal is reached. Used after
    /// the above method is called.
    bool achieved() const {
			if(anyAchieved()){
	    	return true;
			} else {
	    	return false;
			}
    }

    /// assignment.
    ThresholdTerminationTest& operator = (const ThresholdTerminationTest& other){
			_super::operator=(other);
			m_th = other.m_th;
			return *this;
    }

private:
    SCORE m_th;

};

#endif

