#ifndef _TimeTerminationTest_H_
#define _TimeTerminationTest_H_

#include "TerminationTest.hpp"
#include "Timer.hpp"
#include "OptionsPrinter.hpp"
#include <boost/program_options.hpp>

/// The parsing goal specifying a time limit to the parse.
class TimeTerminationTest : public TerminationTest {
    typedef TerminationTest _super;
public:
    typedef _super :: HowAchieved HowAchieved;

    /// constructor.
    TimeTerminationTest(clock_t time):m_limitTime(time) {m_timer.start();}

    /// construct from config file
    TimeTerminationTest(const string& configFile) {
      clock_t time;
      BPO::options_description config("Time config file options");
      config.add_options()
	("time", BPO::value<clock_t>(&time), "Time limit");
      BPO::variables_map vm;
      iFile ifs(configFile.c_str());
      store(parse_config_file(ifs, config), vm);
      notify(vm);
      m_limitTime = time;
      m_timer.start();
    }

    /// Destructor.
    virtual ~TimeTerminationTest() {}

    GoalID add(TerminationTest* subGoal) {
	// cannot add a subgoal because this is a primitive goal.
	THROW("TimeTerminationTest::add not implemented");
    }

    /// returns true is this goal is achieved.
    /// \param item : the item just derived. (not used)
    /// \param which : which subgoal is reached. If no goal is
    /// reached, then NONE, otherwise, returned 0. (Not used)
    bool achieved(const Item* item, GoalID& which) {
      bool b = (m_timer.sofar() > m_limitTime);
      if(b){
	cerr << "Warning: Time parsing goal of " << m_limitTime << " seconds has been achieved. " << endl;
      }
	return b;
    }

    /// Checks whether the goal is reached. Used after
    /// the above method is called.
    bool achieved() const {
	if(anyAchieved()){
	    return true;
	} else {
	    return false;
	}
    }

  virtual void reset(){
    TerminationTest::reset();
    m_timer.start();
  }

private:
  //! The timer for this goal
  Timer m_timer;
  //! time limit for parse passed in constuctor
  std::clock_t m_limitTime;
};

#endif

