#ifndef _TranslationEvaluator_H_
#define _TranslationEvaluator_H_

#include "UpdateRule.hpp"
#include "ItemIndex.hpp"
#include "LearnableGrammar.hpp"
#include "HeadMTree.hpp"

//!  EMUpdate rule encapsulates comparson between
//!  proposed and supervisory standard parses, as well as
//!  updates to the learner's model.
 
class TranslationEvaluator : public Dbug {

public:

  TranslationEvaluator();

  ~TranslationEvaluator() {
  }

  /**
   * This method will compare a supervisory parse with a 
   * candidate parse using a unigram comparison of the 
   * lexical items, and will return the resulting score
   * which is calculated as the number of matched words
   * in the two sentences, divided by the total number
   * of words in the two sentences.
   */
  virtual SCORE evalByWords(WTreeVec &supervisorParse, 
                 WTreeVec &candidateParse) const; 

  /**
   * This method will compare a supervisory parse with a
   * candidate parse by comparing the tree structure of 
   * the two parses.  This will work by dividing the 
   * number of matching productions in a parse by
   * the total number of productions in the parse.
   * It will then return the resulting score.
   */
  virtual SCORE evalByParse(WTreeVec &supervisorParse,
                    WTreeVec &candidateParse) const;

};

#endif
