
#include <string>
#include "Unbinarizer.hpp"
#include "HeadMTree.hpp"
#include "IOUtil.hpp"

///////////////////////////////////////////////////////////
void Unbinarizer::operator()(HeadMTree& tree,
			     vector<StringVocabulary* > NTVocabularyVec){
  operator()(tree, tree.begin(), NTVocabularyVec);
}

void Unbinarizer::
operator()(HeadMTree& tree,
           HeadMTree::pre_order_iterator rootIter,
	   vector<StringVocabulary* > NTVocabularyVec) {

  if(rootIter.number_of_children() == 0) return;

  unsigned int i;
  arity_t d;
  bool gotIt;
  size_t slashified;
  // typename HeadMTree<D>::NodeTypeptr rootPtr = tree.getData();
  // DSV<D> parentSpan = static_cast<DSV<D> >(*rootPtr);

  vector<short> binnodes;
  //vector<short>::const_iterator vsci;

  //uncomment for debugging
  // cerr<< rootPtr->getLabel() << endl;
  HeadMTree::sibling_iterator sibIt;

  for(sibIt = rootIter.begin(), i=0; sibIt != rootIter.end(); ++sibIt, i++) {
    // first, unbinarize the child node
    (*this).operator()(tree, HeadMTree::pre_order_iterator(sibIt), NTVocabularyVec);

    // second, adopt grandkids whenever a kid's NT has a slash
    slashified = 0;

    NTLink kidLabel = BGET_NT_LABEL(*sibIt);
    NTLink kidNTLink = kidLabel.getNTLink();
    arity_t dim = kidLabel.arity();

    for(d = 0; d < dim; ++d) {
      SynCat kidNT = kidNTLink[d];
      string ntName = "";
      ntName += NTVocabularyVec[d]->getWord(kidNT.value(), gotIt);
      // cerr << "NT " << kidNT.value();
      if (!gotIt)
	die("NTVocabulary problem");
      // cerr << "stands for " << ntName << endl;

      // see if the NT is compositional.
      if(SynCat::compositional(ntName)){
	slashified ++;
      }
    }

    if (slashified == dim) {
      // record this child's index
      // cerr << "Unbinarizing" << endl;
      binnodes.push_back(i);
    };

  };

  int jj = (int)binnodes.size() - 1;
  //assert(binnodes.size()<=1);
  // if(binnodes.size() <= 1) {
  // cerr<<"Warning: bin node size less than or equal to 1."<<endl;
  // return;
  // }

  while(jj >= 0){
    // delete this child and adopt grandkids
    i = binnodes[jj];
    //tree.removeChild(i);
    HeadMTree::sibling_iterator kid_it = rootIter.begin();
    kid_it += i;

    // NOTE that we have not assumed that the
    // complex category is always the heir.
    bool isheir = false;
    if(tree.isHeir(kid_it)){ isheir = true; }

    // find the heir of kid_it.
    HeadMTree::sibling_iterator k;
    if(isheir){
      bool noheir = true;
      for(k = kid_it.begin(); k != kid_it.end(); ++k){
	if(tree.isHeir(k)){ noheir = false; break;}
      }
      assert(!noheir);
    }
    tree.reparent(rootIter, kid_it.begin(), kid_it.end());
    // set heir since the original heir will be deleted.
    if(isheir){ tree.setHeir(k);}
    //assert(rootIter
    tree.erase(kid_it);
    jj--;
  }
}
