#include "UniformPerceptronUpdateRule.hpp"
#include "Production.hpp"

/// compute the gradient for each feature -- here is the default for MLE
void UniformPerceptronUpdateRule::
getFeatureGradients( size_t supervisorMarginSize, size_t learnerMarginSize,
		     size_t intersectionSize,
		     SCORE& supervisorGradient, SCORE& learnerGradient, 
		     SCORE& intersectionGradient,
		     SCORE& unseenFeatGradient) const {

  

  // it's possible for the learnerMargin to be zero and the
  // supervisorMargin to be non-zero if the learner doesn't smooth due
  // to discontinuities and gets stuck, even though it knows about
  // some of the features necessary further in the parse 
  if (supervisorMarginSize + intersectionSize> 0 && learnerMarginSize > 0)
    supervisorGradient = -1.0 / (SCORE) (supervisorMarginSize+intersectionSize);
  else
    supervisorGradient = 0;

  // it's possible for the supervisorMargin to be zero and the
  // learnerMargin to be non-zero if the learner doesn't know about
  // any of the features in the supervisorMargin
  if (learnerMarginSize > 0 && supervisorMarginSize + intersectionSize > 0)
    learnerGradient = 1.0 / (SCORE)  (supervisorMarginSize+intersectionSize);
    //    learnerGradient = 1.0 / (SCORE) learnerMarginSize;
  else
    learnerGradient = 0;

  if (learnerMarginSize > 0 && supervisorMarginSize + intersectionSize > 0)
    intersectionGradient = -1.0 / (SCORE) (supervisorMarginSize+intersectionSize);
  else
    intersectionGradient = 0;
  
  unseenFeatGradient = 0.0;
};
