#include "MoreAlgorithms.hpp"
#include "UpdateRule.hpp"
#include "LearnableGrammar.hpp"
#include "HeadMTree.hpp"

SCORE UpdateRule::getGradientHelper(const WTreeVec& learnerParses, 
				    const WTreeVec& supervisoryParses,
				    vector<HeadMTree::iterator>& supervisorMargin, 
				    vector<HeadMTree::iterator>& learnerMargin, 
				    vector<HeadMTree::iterator>& vecIntersection) const {

  // find the nodes in each tree that don't correspond to anything in the other tree
  HeadMTree::twoSidedDiff(supervisoryParses, learnerParses, 
			  supervisorMargin, learnerMargin, 
			  vecIntersection, m_keepW2W);
  return 0; // dummy for now
}

/// compute the cost or margin gradient vector, and return the sum of its elements
SCORE UpdateRule::getGradient(AbstractParser* supervisor, Parser* learner, 
			      SentenceTuple& sntT, LearnableGrammar* gradient,
			      ostream& treeStream) const {
  SCORE cost = 0.0;
  WTreeVec learnerParses, supervisoryParses;

  learnerParses = learner->getParse(sntT);

  if(treeStream) {
    if (not learnerParses.empty()){
      WTreeVec::const_iterator cit = learnerParses.begin();
      treeStream << *cit->first;
      cost += cit->second;
      ++cit;
      for(; cit != learnerParses.end(); ++cit){
	treeStream << "\t" << *cit->first;
	cost += cit->second;
	// this is if we want the convergenceValue to be the sum of the SCORES
	//      convergenceValue += cit->second;
      }
    }
    treeStream << endl;
  }
  if (debug(2))
    cerr << "Learner cost = " << cost << endl;

  if ( supervisor){
    supervisoryParses = supervisor->getParse(sntT);
  }
  vector<HeadMTree::iterator> supervisorMargin, learnerMargin, vecIntersection;
  getGradientHelper(learnerParses, supervisoryParses,
		    supervisorMargin, learnerMargin, vecIntersection);

  Grammar* pCurrModel = learner->getGrammar();
  // use call-back pattern to simulate virtual template function
  //  convergenceValue = 
  gradient->trees2gradient(supervisorMargin, learnerMargin, vecIntersection, 
			   pCurrModel, this);

  //convergenceValue is defined as number of mistakes made.
  size_t supervisorMarginSize = supervisorMargin.size();
  size_t learnerMarginSize = learnerMargin.size();
  SCORE convergenceValue =  supervisorMarginSize+learnerMarginSize;
  //  convergenceValue = cost;

  if (debug(1)) {
    cerr << "Size of intersection = " << vecIntersection.size()
	 << " LearnerMargin = " << learnerMarginSize 
	 << " SuperMargin = " << supervisorMarginSize << endl;
    cerr << "Convergence value = " << convergenceValue << endl;
  }

  //parses need to be deleted to avoid memory leaks
  WTreeVec :: iterator it;
  for(it = learnerParses.begin(); it != learnerParses.end(); ++it){
    delete it->first;
  }
  for(it = supervisoryParses.begin(); it != supervisoryParses.end(); ++it){
    delete it->first;
  }

  return convergenceValue;
}
