#include "UpdateRuleBuilder.hpp"

#include <fstream>
#include "Exceptions.hpp"

#include "EStep.hpp"
#include "UniformPerceptronUpdateRule.hpp"
#include "NoiseFilterUpdateRule.hpp"

UpdateRuleBuilder* UpdateRuleBuilder::_instance = 0;

UpdateRuleBuilder* UpdateRuleBuilder::Instance(){
  if(_instance == 0){
    _instance = new UpdateRuleBuilder;
  }
  return _instance;
}

//! build updateRule
UpdateRule* UpdateRuleBuilder::build(const string& ruleType) {
  UpdateRule* rule;
  if (ruleType == "EM"){
    rule = new EStep();
  } else if (ruleType == "UniformPerceptron") {
    rule = new UniformPerceptronUpdateRule();
  } else if (ruleType == "NoiseFilterUpdateRule") {
    rule = new NoiseFilterUpdateRule();
  } else {
    THROW( ruleType << " is not a valid update rule." << endl);
  }

  return rule;
}
