#ifndef _ValencyEvent_H_
#define _ValencyEvent_H_

#include "COMPILER-DEPENDENT_HEADERS.hpp"
#include "GPDefs.hpp"
#include "Vector.hpp"
#include "Pair.hpp"
#include "NTLink.hpp"
#include "MoreAlgorithms.hpp"
// #include "MazMemPool.hpp"
#include <boost/serialization/serialization.hpp>

using namespace std;

//! A valency model event
class ValencyEvent :
// public MazMemPool,
    public Pair<NTLink, short> {
 public:
  typedef Pair<NTLink, short> _super;
  typedef _super::FirstType FirstType;
  typedef _super::SecondType SecondType;

  /*
   * Constructors.
   */
  //! Empty constructor.
  ValencyEvent();

  //! Default constructor.
  ValencyEvent(const arity_t& dim);

  //! Useful constructor.
  ValencyEvent(const NTLink& l, const short& v);

  virtual ~ValencyEvent() {};

  arity_t arity() const;

  void setDimity(const arity_t& d);

  /*
   * Operators.
   */
  //! Equality test.
  bool operator == (const ValencyEvent& other) const;

  //! Assignment.
  ValencyEvent& operator = (const ValencyEvent& other);

  //! Ordering.
  bool operator< (const ValencyEvent& other) const;

  /*
   * Member access.
   */
  //! Returns the link.
  FirstType& link();
  FirstType link() const;

  //! Returns the valency value.
  SecondType& valency();
  SecondType valency() const;

  /*
   * I/O
   */
  //! Input line by line.
  friend istream& operator>>(istream& in, ValencyEvent& valE);

  //! Output.
  friend ostream& operator<<(ostream& out, const ValencyEvent& valE);

  //! Serialization
  template<class Archive>
  void serialize(Archive & ar, unsigned int version) {
    // ar & boost::serialization::base_object<MazMemPool>(*this);
    ar & boost::serialization::base_object<_super >(*this);
  };

  //! For hashing.
  virtual size_t hash () const;

  ostream& put(ostream& out) const;

  //! Read.
  istream& get(istream& in);
};

/*******************************************************************/

namespace HashNS {
  //! Template specialization of STL's hash object (a hash function) for
  //! ValencyEvents.
  template<> struct hash<ValencyEvent> {
    size_t operator() (const ValencyEvent& vE) const{ return vE.hash();}
  };
}

#endif
